package Modules::Publish;
#----------------------------------------------------------------------------
#   The Publish module is a user-friendly interface to publish.cgi;
#   it also is a shortcut to the publish functions in the content management
#   modules such as Website Manager and MySite.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2006 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Module;
use ExSite::Util;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# write : this is used to invoke the Publish plug-in from normal site 
# page views.  Typically, this is for the purpose of auto-publishing
# on a pseudo-scheduled basis.  The $options are:
#
# hourly - republish the current page if more than one hour has transpired
#          since the previous publish
# daily - republish the current page if more than one day has transpired
#          since the previous publish
# weekly - republish the current page if more than one week has transpired
#          since the previous publish
# [n] - republish the current page if more than [n] minutes have transpired
#       since the previous publish
#
# Note that publishing does not affect the current page view;  subsequent 
# views can see the newly published page (modulo caching).

sub write {
    my ($this,$options) = @_;
    # this only works as an AJAX plugin; abort if not in AJAX mode
    return undef if (! $this->{input}{_ref} && ! $this->{input}{_ajax});
    # abort if in an administrator preview
    return undef if ($this->{input}{_cms});
    my $out = " ";
    return $out if ($share{Page}->is_publishing);
#    return $out if (! $config{Publish}{username});

    # FIXME: return in preview modes

    my $elapsed;  # seconds since last publication
    my ($y,$m,$d);
    if ($options =~ /^\d+$/) {
	$elapsed = $options * 60;
    }
    elsif ($options =~ /^(\d\d\d\d)-(\d\d)-(\d\d)$/) {
	$y = $1;
	$m = $2;
	$d = $3;
	### TODO - publish once on or after this date
    }
    else {
	my %elapsed_names = ( hourly => 3600,
			      daily => 86400,
			      weekly => 604800 );
	$elapsed = $elapsed_names{$options} || 0;
    }
    return $out if (! $elapsed);

    my $age = 0;
    my $filename = $share{Page}->diskpath;
    if (-e $filename) {
	my @filestat = stat($filename);
	my $mtime = $filestat[9];
	$age = time - $mtime;
    }

    #$out = "page not republished";
    if ($age > $elapsed) {

	# looks like it's time to republish!

	my $u = $share{DB}->fetch_owner();
	return "No site owner to publish as!" if (ref $u ne "HASH");
	my $uname = $u->{$config{auth}{user_login_column}};
	return "No user ID to publish under!" if (! $uname);

	my $id = $share{Page}->id;
	my $publish_url = "$config{server}{CGIpath}/$config{prog}{publish}?page=$id";
	my $authtoken = $share{DB}->make_authtoken($uname,0,$publish_url);
	$out .= "$publish_url<br>$authtoken<br>\n";
	$out .= &run_publisher("page",$id,0,{_auth=>$authtoken});
	# we ignore the output and write just a comment
	$out = "<!-- published page $id -->\n";
    }
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Publish plug-in provides a simple interface to ExSite's publishing tools.";
    }
    elsif (/ControlPanel/) {
	return \&ControlPanel;
    }
    elsif (/DynContentParam/) {
	return "<b>Frequency:</b>
<select name=\"param\">
<option value=''>(default)</option>
<option>weekly</option>
<option>daily</option>
<option>hourly</option>
<option value='30'>every 30 minutes</option>
<option value='15'>every 15 minutes</option>
<option value='10'>every 10 minutes</option>
<option value='5'>every 5 minutes</option>
</select>\n";
    }
}

sub ControlPanel {
    my $this = shift;
    my $db = $share{DB};
    my $out = "<h1>Exsite Publisher</h1>\n";

    # sites

    $out .= "<table cellspacing=20><tr>\n";
    my $nicon;
    foreach my $site ($db->my_real_sites) {
	$nicon++;
	my $url = "$config{server}{CGIpath}/publish.cgi?section=$site->{section_id}";
	$out .= "<td align=center><a href=\"$url\" target=\"publish\"><img src=$config{server}{HTMLpath}/_Modules/CMS/site.gif height=48 width=48 border=0><br>$site->{title}</a></td>\n"; 
	if ($nicon % 4 == 0) { $out .= "</tr><tr>\n"; }
    }

    # modules

    foreach my $mod (&ExSite::Module::my_module_list) {
	if (eval("require Modules::$mod")) {
	    my $m = eval("new Modules::$mod");
	    if ($m) {
		if (ref $m->ioctl("Publish") eq "CODE") {
		    $nicon++;
		    my $modname = $m->ioctl('ModuleName') || $mod;
		    my $icon = $m->icon;
		    my $url = "$config{server}{CGIpath}/publish.cgi?module=$mod";
		    $out .= "<td align=center><a href=\"$url\" target=\"publish\"><img src=\"$icon\" height=\"64\" width=\"64\" border=\"0\"><br>$modname</a></td>\n"; 
		    if ($nicon % 4 == 0) { $out .= "</tr><tr>\n"; }
		}
	    }
	}
    }
    $out .= "</tr></table>
<p>Click on a website (folder) icon, or a web application icon to publish
its files.</p>\n";
    return $out;
}

1;

