package Modules::Search;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
#
#  The search tool builds a search index using 2 tables:
#  - searchurl: every URL that is searched is represented once here
#  - searchterm: every indexed word at each url is represented once here,
#                with a weight (based on location and number of ocurrences)
#
#  The search index is constructed by stripping out all extraneous tags and
#  text, and counting the incidence of each remaining word in the body of 
#  the html at each URL.  Words in filenames, titles, descriptions, and 
#  keywords are counted and weighted separately.
#
#  This version searches only the default representation of each page, 
#  and does not follow links to recursive instances of the same page.
#
#  The search logic is somewhat crude, and does not allow for searching
#  for partial words, quoted phrases, absense of words, etc.
#
#-----------------------------------------------------------------------
#
#  TODO:
#  - the query logic for looking up search terms could be more efficient
#  - do we need to do a full page build to get the searchable text?
#  - would be nice to exclude menus from the searchable text
#  - follow recursive page links at least one level
#

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Auth;
use ExSite::Page;
use ExSite::Search;
use ExSite::Module qw(&site_license);
use Modules::BaseDCD;
use Digest::MD5;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD ExSite::Search);

sub new {
    my $this = shift;
    my $obj = {};
    my $class = ref($this) || $this;
    bless $obj, $class;

    $obj->read_conf;

#    $obj->{weight} = $config{Search}{weight};
#    $obj->{title} = {};
#    $obj->{descr} = {};
#    $obj->{site} = {};

#    my @skipwords = split / /,$config{Search}{skipwords};
#    $obj->{skipwords} = \@skipwords;

    $obj->read;
    return $obj;
}

sub write {
    my ($this, $opt) = @_;
    my $a = new ExSite::Auth;
    if (&site_license("Search",$this->{input}{section})) {
	my $in = new ExSite::Input;
	my $sdata = $in->post_or_query;
	my $term = $sdata->{searchterm} || $opt;
	if ($term) {
	    return $this->do_search($term);
	}
	else {
	    my ($size,$title);
	    my $size = $config{Search}{size} || 20;
	    if ($config{Search}{showtitle}) {
		$title .= 'Search <!--$site_title-->';
	    }
	    return $this->search_form($term,$title,$size);  # in ExSite::Search
	}
    }
    return "Search: no license for this web application";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isRestricted/) {
	return 1;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/ModuleInfo/) {
	return "Search is a standard website search tool.  It allows you
to build search indexes for your website, and easily search for any set of
terms.  Search results are ranked and reported with the page title and
description.  Search indexes can be manually edited by experts, if you 
want to boost or lower the relevance of certain terms on certain pages.";
    }
    return;
}

sub ctrl_panel {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    my $donewrite;
    if ($share{DB}->authorize < $config{auth}{root_auth_level} - 1) {
	return "Permission denied.";
    }

    if ($in->{action} eq "search") {
	$out .= $this->write;
	$donewrite = 1;
    }
    elsif ($in->{action} eq "Build index for selected site") {
	if ($in->{section}) {
	    if (! $share{DB}->owns($in->{section})) {
		return $this->error("Index section $in->{section}: Permission denied");
	    }
	    $out .= $this->index_site($in->{section});
	    #$out .= $this->clear_site($in->{section});
	    $out .= $this->update($config{search}{threshold});
	}
    }
    elsif ($in->{action} eq "Build index for ALL sites") {
	my @sites = $share{DB}->my_real_sites;
	foreach my $site (@sites) {
	    $out .= $this->index_site($site);
	    #$out .= $this->clear_site($site);
	}
	$out .= $this->update($config{search}{threshold});
    }
    elsif ($in->{action} eq "Clear index for selected site") {
	if ($in->{section}) {
	    if (! $share{DB}->owns($in->{section})) {
		return $this->error("Index section $in->{section}: Permission denied");
	    }
	    $out .= $this->clear_site($in->{section});
	    $out .= $this->info("Index for section $in->{section} cleared.");
	}
    }
    elsif ($in->{action} =~ /Manually edit/) {
	if ($in->{section}) {
	    if (! $share{DB}->owns($in->{section})) {
		return $this->error("Index section $in->{section}: Permission denied");
	    }
	    my $p = $share{Page} || $share{PageBuf};
	    $p->location("$config{server}{CGIpath}/webdb.cgi?tab=searchurl&ptab=section&pid=$in->{section}");
	    return undef;
	}
    }

    my $a = new ExSite::Auth;
    my @sites = $a->my_real_sites;
    
    $out .= "<h1>Search Administration</h1>
<p>You have Search enabled on the following sites:</p>
<form>
<select name=\"section\">\n";
    my $nsite = 0;
    foreach my $site (@sites) {
	if (&site_license("Search",$site->{section_id})) {
	    $out .= "<option value=\"$site->{section_id}\">$site->{title}</option>\n";
	    $nsite++;
	}
    }
    if ($nsite == 0) {
	$out .= "<option value=\"0\">No search-enabled websites found!</option>\n";
    }
    $out .= "</select>
<p><input type=\"submit\" name=\"action\" value=\"Build index for selected site\"><br>
<input type=\"submit\" name=\"action\" value=\"Build index for ALL sites\"><br>
<input type=\"submit\" name=\"action\" value=\"Clear index for selected site\"><br>
<input type=\"submit\" name=\"action\" value=\"Manually edit search index for selected site\"></p>\n</form>\n";
    #if (! $donewrite && $in->{section}) { $out .= $this->write; }
    return $out;
}

1;
