package Modules::VersionMenu;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

=pod

=head1 Modules::VersionMenu

VersionMenu generates a menu to choose from the various versions 
of the page that are available.

A version is any alternative presentation of the same information.
Alternative versions may be translations into other languages,
high/low bandwidth versions, advanced/simplified versions, and so on.
The most common application is for alternate language translations.

VersionMenu allows you to switch to an alternate version of the page
you are currently on.  It only shows the available versions of the
current page.  For instance, if the site supports French pages, but
the current page has no French version available, then we do not give
you the option of going to the French version.

Note that the built-in menu and sitemap constructors in ExSite are
context sensitive, and automatically build menus to pages that match
the current version, falling back to the default version if nothing is
found.  For example, if currently viewing a French page, the menus
will attempt to link to French versions of all other pages, but will
link to the default version (presumably English) if no French version
is found.

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Page;
use ExSite::Input;
use ExSite::Tree;
# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this, $opt) = @_;
    my $db = $share{DB};
    my @vitems;

    # setup 

    my $Page = $share{Page};
    return if (!$Page); 
    my $sid = $Page->id("section");
    if ($Page->is_page()) {

	# get current version
	my $version = $Page->get_my("version");

	# get known versions of this page
	my @page;
	# get the master (default) version
	my $p = $db->fetch("page", $Page->get_my("type") eq "alternate" ?  $Page->get_my("parent_id") : $Page->id);
	push @page, $p;
	# get all alternate versions
	push @page, $db->fetch_match("page",{type=>"alternate",
					     parent_id=>$p->{page_id} });
	my $ver = keywise("version",\@page);

	if (@page == 1) {
	    # don't output a menu if there is only one version available
	    return undef;
	}

	# get list of all possible versions
	my @vlist = split /\|/,$db->{map}->regexp("list:version");
	if (@vlist == 0) { @vlist = ( "English" ); }

	# default version is shown first
	if ($ver->{$config{default_version}}) {
	    push @vitems, $this->make_link($ver->{$config{default_version}},
					   $config{default_version},
					   $version);
	    delete $ver->{$config{default_version}};
	}
	elsif ($ver->{''}) {
	    push @vitems, $this->make_link($ver->{''},
					   $config{default_version},
					   $version);
	    delete $ver->{''};
	}

	# show other possible versions in order
	foreach my $v (@vlist) {
	    if ($ver->{$v}) {
		push @vitems, $this->make_link($ver->{$v},$v,$version);
		delete $ver->{$v};
	    }
	}

	# lastly, show any additional versions 
	foreach my $v (keys %$ver) {
	    push @vitems, $this->make_link($ver->{$v},$v,$version);
	    delete $ver->{$v};
	}
    }
    
    # output the menu
    my $vmenu = join("&nbsp|&nbsp;",@vitems);
    if ($vmenu) {
	return "<span class=\"VersionMenu\">$vmenu</span>\n";
    }
    return undef;
}

sub make_link {
    my ($this,$pdata,$version,$current_version) = @_;
    my $p = new ExSite::Page(page=>$pdata);
    my $url = $p->get_url();
    if ($version eq $current_version || 
	($version eq $config{default_version} && $current_version eq '')) {
	return "$version";
    }
    else {
	return "<a href=\"$url\">$version</a>";
    }
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleInfo/) {
	return "VersionMenu builds a menu to select from the various
languages/versions you have installed on your site.";
    }
    return undef;
}

1;
