#!/usr/bin/perl

=pod

=head1 Modules::Zine

This is a multi-purpose plug-in for the ExSite article management subsystem.
It provides end-user and administrator tools for reading, browsing, posting,
and managing various types of e-zines, articles, and comments.

The article management subsystem is contained in the Zine subdirectory.
This includes the following packages:

    Modules::Zine::Config  - defines basic rules and behaviours
    Modules::Zine::Base    - data and security management
    Modules::Zine::Article - formatting and output of articles
    Modules::Zine::Form    - handling all forms and input
    Modules::Zine::Index   - generic indexing engine
    Modules::Zine::Index*  - alternate indexing engines

Any plug-in can make use of this framework.  This package, the generic
Zine plug-in, provides a simple, flexible interface to all of the
features in the framework.  It also adds some high-level functions
that do not pertain to specific articles or e-zines, such as:

=over 4

=item directories of zines

=item highlight recent articles or comments

=item highlight a random article

=back

=cut

package Modules::Zine;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Tree;
use ExSite::HTML;
use Modules::Zine::Article;
use Modules::Zine::Form;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Article Modules::BaseDCD);

sub read {
    my ($this,$opt) = @_;
    # fetch overlayed query and form input
    my $in = new ExSite::Input;
    $this->{input} = $in->combine;
    # add path info to input hash as "path"
    if ($ENV{PATH_INFO}) {
	$this->{input}{path} = $ENV{PATH_INFO};
    }
}

sub write {
    my ($this,$options) = @_;
    my (%opt,$in,$id,$cmd,$n);
    
    # allow bare ID, bare zine type, or bare article name to be passed

    %opt = &DecodeString($options);
    $this->{opt} = \%opt;
    $in = $this->{input};
    $cmd = $in->{zine} || $opt{zine};
    $id = $in->{aid} || $opt{aid};
    $this->load_article($id);
    # variable n = number of articles to show for new articles, latest comments
    $n = $in->{n} || $opt{n};
    
    my $out;
    # modified to pass id and n / moved to top of write function

    # process any hard-coded command requests

    if ($opt{zine} eq "newarticles") {
	$out = $this->index_latest_articles();
    }
    elsif ($opt{zine} eq "newcomments") {
	$out = $this->index_latest_comments();
    }
    elsif ($cmd eq "rss") {
	my $p = $share{Page} || $share{PageBuf};
	$p->cancel;
	print "Content-type: text/xml\n\n";
	print $this->rss();
    }
    elsif ($cmd eq "toggle") {
	$out = $this->index_toggle("article",$id,$n);
    }	
    elsif ($cmd eq "random") {
	return $this->index_random($id);
    }	
    elsif ($cmd =~ /zine?dex/) {
	return $this->zinedex($in->{type} || $opt{type});
    }	
    return $out if ($out);
    
    # process Zine forms

    if ($cmd =~ /^(submit|new|reply|post|newarticle|edit|approve|reject)$/) {
	my $formout .= $this->form($cmd);
	if ($formout) {
	    $out .= $formout;
	}
	elsif ($id) {
	    $out .= $this->show;
	}
	else {
	    $out = $this->zinedex($in->{type},$this->{input}{section_id});
	}
    }

    # display or update articles

    elsif ($this->{article_id}) {
	if ($cmd eq "index") {
	    $out = $this->index(%opt,archive=>$in->{archive});
	}
	elsif ($cmd eq "del") {
	    $out = $this->delete();
	}
	else {
	    $out = $this->show(archive=>$in->{archive});
	}
    }
    elsif (! $out) {
	$out = $this->topview($options);
    }
    return $this->wrap("Zine",$out);
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "e-Zines";
    }
    elsif (/isRestricted/) {
	return $this->{config}{isRestricted};
    }
    elsif (/isService/) {
	return $this->{config}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::Base::publish;
    }
    elsif (/Search/) {
	return \&Modules::Zine::Base::search_index;
    }
    elsif (/DynContentParam/) {
	my $section_id = shift;
	my @zine = $this->our_zines($section_id);
	my $out .= "<b>E-Zines:</b>&nbsp;<select name=\"param\">
<option value=\"\">&lt;default&gt;</option>
<option value=\"\">show all e-zines</option>\n";
	foreach my $zine (@zine) {
	    $out .= "<option value=\"aid=$zine->{article_id}&zine=show\">$zine->{title}</option>";
	}
	$out .= "</select>\n";
	return $out;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    # insert some style rules for better formatting in control panels
    $out .= "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Zine/zine.css\">\n";
    my $sid = $this->get_section_id();

    my $in = $this->{input};
    $_ = $in->{zadm};
    $this->{admin} = 1;  # set admin flag to add admin tools

    if ($in->{zine} =~ /^(submit|new|reply|post|newarticle|edit|conf|newzine)$/) {
	my $formout .= $this->form($in->{zine});
	if ($formout) {
	    $out .= $formout;
	}
	elsif ($in->{aid}) {
	    $this->load($in->{aid});
	    $out .= $this->show;
	}
	else {
	    $out = $this->topview_admin;
	}
    }
    elsif ($in->{zine} || $in->{aid}) { 
	$out .= $this->write; 
    }
    elsif (/list/) {
	/^list(.*)$/;
	my $type = $1;
	$out .= $this->topview_admin($type);
	$out .= "<br>\n";
	my $url = $this->link(zadm=>"new$type");
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a new $type");
    }
    elsif (/new/) {
	/^new(.*)$/;
	my $type = $1;
	if ($type eq "raw") { $type = "article"; }
	$out .= $this->makeform({type=>$type});
    }
    elsif (/publish/) {
	$out .= "<h3>Publishing Images</h3>
<iframe height=\"300\" width=\"600\" src=\"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine\">
<a href=\"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine\" target=\"publish\">Click here to publish images.</a>
</iframe>\n";
    }
    elsif (/search/) {
	$out .= $share{DB}->make(table=>"article",search=>1);
    }
    else {
	if ($share{DB}->is_admin()) {
	    $out .= "<table><tr><td>\n";
	    $out .= $this->set_section_id();
	    $out .= "</td><td><form method=\"GET\">Show: <select name=\"type\" onchange=\"document.forms[1].submit()\">
<option value=\"\">== select ==</option>
<option value=\"\">All zines</option>
<option value=\"e-zine\">e-Zines</option>
<option value=\"blog\">Blogs</option>
<option value=\"forum\">Forums</option>
<option value=\"book\">Books</option>
<option value=\"wiki\">Wikis</option>
<option value=\"photoblog\">Photo Blogs</option>
<option value=\"photoforum\">Photo Forums</option>
</select>
<input type=\"hidden\" name=\"section_id\" value=\"$sid\">
</form>
</td></tr></table><br>\n";
	}
        $out .= $this->topview_admin;
	$out .= "<br>\n";
	#my $url = $this->link(zadm=>"newraw");
	#my $url = "$config{server}{CGIpath}/webdb.cgi?tab=article&action=new";
	my $url = $this->link(zine=>"newzine");
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a new Zine");
    }

    return $out;
}

# load_article : fetch the base article
#
# The parameter $id is the default article to load, but may be over ridden 
# by the aid parameter or the path info.

sub load_article {
    my ($this,$id) = @_;
    my $in = $this->{input};
    if ($id) { 
	$this->load($id); 
    }
    elsif ($in->{path} && ! $this->{admin}) { 
	# we don't use path method of referencing articles in admin mode
	$this->find($in->{path}); 
    }
    return;
}

# form : all posts go through here

sub form {
    my ($this,$cmd) = @_;
    my $out;
    my $art = $this->{article} || $this->{input}{aid};
    my $f = new Modules::Zine::Form($art,{admin=>$this->{admin}});
    if ($cmd eq "submit") {
	my @err = $f->doform();
	if (@err) {
	    $out = "<h3>Problem:</h3>\n".
		$this->error(@err).
		"<p>[<a href=\"javascript:history.back()\">Back</a>]</p>\n";
	    return $out;
	}
	if ($f->{moderation_requested}) {
	    $out .= "<p class=\"ZineWarning\">Your post has been received, 
but will not appear until it has been approved by the moderator.</p>\n";
	    $this->load($art);
	    $out .= $this->show;
	    return $out;
	}
	return undef;
    }
    elsif ($cmd eq "new") {
	$out = $f->newmsg;
    }
    elsif ($cmd eq "reply") {
	$out = $f->reply;
    }
    elsif ($cmd eq "post") {
	$out = $f->post;
    }
    elsif ($cmd eq "newarticle") {
	$this->load($art);
	$out = $f->makeform({type=>"article"},$this->{article});
    }
    elsif ($cmd eq "edit") {
	$this->load($art);
	$out = $f->makeform($this->{article});
    }
    elsif ($cmd eq "conf") {
	$this->load($art);
	$out = $f->confform();
    }
    elsif ($cmd eq "newzine") {
	$out = $f->newzine();
    }
    elsif ($cmd =~ /approve|reject/) {
	my $stat = $f->status($cmd);
	if ($stat) {
	    $out .= "<p class=\"ZineWarning\">$stat
This action can be un-done using the $this->{config}{tool}{configure} 
function, by selecting a different status.</p>\n";
	}
	$out = $this->show(archive=>$this->{input}{archive});
    }
    return $out;
}

#--------------------------------------------------------------------------

# high-level indexes

# index of available zines (site-specific)

sub topview {
    my ($this,$opt) = @_;
    my $stat = $this->handler("ZineTopView",$opt);
    return $stat if ($stat);
    my %match = (status=>"active", section_id=>0, reference=>0);
    my (@art,$type);
    if ($opt && $opt !~ /=/) { 
	# request to view a particular class of top-level articles
	$type = $opt; 
    }

    # get all zines for this site
    @art = $this->our_zines();

    my $out;
    if (@art == 1) {
	$this->save();
	$this->set($art[0]);
	$out = $this->show();
	$this->restore();
    }
    elsif (@art > 0) {
	$out .= "<dl class=\"ZineIndex\">\n";
	foreach my $art (@art) {
	    next if ($type && $art->{type} ne $type);
	    $this->set($art);
	    my $url = $this->link(zine=>"show",aid=>$art->{article_id});
	    my $title = $this->title;
	    my $author = $this->author;
	    my $summary = $this->show_data("summary");
	    $out .= "<dt class=\"ZineItem\"><a href=\"$url\">$title</a>&nbsp;&nbsp;&nbsp;";
	    if ($author && $art->{type} !~ /forum|wiki/) {
		$out .= "by $author";
	    }
	    $out .= "</dt>\n";
	    if ($summary) {
		$out .= "<dd><i>$summary</i>";
	    }
	    $out .= "<p>\n";
	}
	$out .= "</dl>\n";
    }
    return $out;
}

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return "<p class=error>Permission denied.</p>\n";
    }

    return $this->zinedex($this->{input}{type},$this->{input}{section_id});

    my (@art,$match);
    if ($share{DB}->is_admin) {
	$match = {reference=>0};
    }
    else {
	$match = {reference=>0,owner=>$share{DB}->my_uid()};
    }
    if ($opt) { $match->{type} = $opt; }
    @art = $share{DB}->fetch_match("article",$match);

    my $out;
    my @adata;
    if (@art > 0) {
	foreach my $art (@art) {
	    $this->set($art);
	    my $url = $this->link(zine=>"show",zadm=>undef,aid=>$art->{article_id});
	    my $title = "<a href=\"$url\">".$this->title,"</a>";
	    my $summary = $this->show_data("summary");
	    my $site;
	    if ($art->{section_id}) {
		my $s = $share{DB}->fetch("section",$art->{section_id});
		$site = $s->{title};
	    }
	    else {
		$site = "(shared)";
	    }
	    my $type = $this->type;
	    my $tools = $this->tools($art);
	    push @adata, [$title,$type,$site,$tools];
	}
    }
    my @head = ("Zine","Type","Section","Options");
    my $zrpt = new ExSite::ReportBuilder(
					 title=>"Zines",
					 headers=>\@head,
					 data=>\@adata,
					 );
    return $zrpt->make;
}

# zinedex - tabular index of zines, with activity summaries

sub zinedex {
    my ($this,$type,$section_id,$zine_id) = @_;
#    return undef if (! $type);
    my $stat = $this->handler("ZineDex",$type);
    return $stat if ($stat);

    my $db = $share{DB};
    my ($sid,$see_general_zines);
    if ($db->is_admin()) {
	$sid = $section_id || ($share{Page} ? $share{Page}->id("section") : 0); 
	$see_general_zines = 1;
    }
    else {
	$sid = $share{Page} ? 
	    $share{Page}->id("section") : 
	    $db->my_user_record()->{section_id};
	$see_general_zines = 0;
    }
    my @allzine = $this->all_zines();
    my $tree = new ExSite::Tree("article_id","parent_id",@allzine);

    # walk our tree, and disable zines that we should not be seeing

    my @z = $zine_id ? 
	$tree->get_child_data($zine_id) : 
	$tree->get_topnodes_data();
    while (my $z = shift @z) {
	if ($sid && $z->{section_id} && $z->{section_id} != $sid) {
	    # not ours; prune this branch
	    $tree->delnode($z->{article_id});
	    next;
	}
	elsif (! $z->{section_id} && ! $see_general_zines) {
	    # we cannot view global zines
	    $tree->delnode($z->{article_id});
	    next;
	}
	elsif ($type && ! $tree->count({type=>$type},$z->{article_id})) {
	    # none of the desired type in this branch
	    $tree->delnode($z->{article_id});
	    next;
	}
	my @child = $tree->get_child_data($z->{article_id});
	foreach my $c (@child) {
	    if ($type && ! $tree->count({type=>$type},$c->{article_id})) {
		# none of the desired type in this sub-branch
		$tree->delnode($c->{article_id});
		next;
	    }
	}
    }
    
    # display the zines in a tabular format

    my $title;
    if ($type) {
	$title = $this->confrule("whatami",undef,$type) . "s";
    }
    else {
	$title = "Zines";
    }
    my $atype = $type eq "forum" ? "Threads" : "Articles";
    my $out = "<table class=\"Report\">
<tr><th class=\"ReportTitle\" colspan=4>$title</th></tr>
<tr><th>$this->{config}{article_type}{$type}</th><th>$atype</th><th>Comments</th><th>Most Recent Post</th></tr>\n";

    my @zine = $tree->get_topnodes_data();
    foreach my $z (@zine) {
	$out .= $this->zinedex_r($type,$z,$tree);
    }
    $out .= "</table>\n";
    return $out;
}

sub zinedex_r {
    my ($this,$type,$article,$tree) = @_;
    my ($out,$db,@articles,$article_count,@comments,$comment_count);
    $db = $share{DB};
    $this->set($article);

    # count the activity
    @comments = $db->dbselect(["article"],
			      ["thread","mtime"],
			      {reference=>$article->{article_id},
			       type=>"comment",
			       status=>"active"},
			      ["mtime"]);
    @articles = $db->dbselect(["article"],
			      ["thread","mtime"],
			      {reference=>$article->{article_id},
			       type=>"article",
			       status=>"active"},
			      ["mtime"]);
    # FIXME: cache the results!
    $comment_count = scalar @comments;
    $article_count = 0;
    if ($article->{type} eq "forum") {
#### FIXME: both of these give the wrong count
#	my $ctree = new ExSite::Tree("article_id","parent_id",@comments);
#	my @thread = $ctree->get_topnodes_data();
#	$article_count = scalar @thread;
	foreach my $c (@comments) { 
	    $article_count++ if (! $c->{thread});
	}
    }
    else {
	$article_count = scalar @articles;
    }
    # FIXME: report active/archived counts

    if ($type && $article->{type} ne $type) {
	# wrong type; assume it is an oganizational category
	$out .= "<tr><th class=\"ReportTitle\" colspan=4>$article->{title}</th></tr>\n";
    }
    else {
	my $time;
	if (@articles > 0) { $time = $articles[-1]{mtime}; }
	if (@comments > 0 && $comments[-1]{mtime} > $time) { $time = $comments[-1]{mtime}; }
	my $t = new ExSite::Time($time,"sql_timestamp");
	my $date = $t->write("datetime");
#	my $url = $this->link(zine=>"show",aid=>$article->{article_id},type=>undef);
#	my $title = $article->{title};
	my $title;
	if ($article->{section_id}) {
	    my $s = $db->fetch("section",$article->{section_id});
	    $title = $s->{title}.": ";
	}
	$title .= $this->show_link();
	if (! $type) { $title .= " <i>(".ucfirst($article->{type}).")</i>"; }
	my $description = $article->{summary} ? "<br><span class=\"ZineReportAsbstract\">$article->{summary}</span>" : "";
#	$out .= "<tr class='A'><td><span class=\"ZineReportTitle\"><a href=\"$url\">$title</a></span>$description</td><td>$article_count</td><td>$comment_count</td><td>$date</td></tr>\n";
	$out .= "<tr class='A'><td><span class=\"ZineReportTitle\">$title</span>$description</td><td>$article_count</td><td>$comment_count</td><td>$date</td></tr>\n";
    }

    # recurse into child zines
    foreach my $z ($tree->get_child_data($article->{article_id})) {
	next if (! $z->{section_id} && ! $db->is_admin());  # global zine
	$out .= $this->zinedex_r($type,$z,$tree);
    }
    return $out;
}

# index_latest: summarize recently added articles

sub index_latest {
    my ($this,$type,$id,$number) = @_;
    return undef if ($type !~ /^(article|comment)$/);
    my $out;
    my @latest = $this->fetch_latest($type,$id,$number);
    if (@latest > 0) {
	# conf file option to turn off header
	unless ( $config{$this->module_name}{no_header} == 1 ) { 
	    $out = "<div class=\"Zine\">\n<h1>Recent ".ucfirst($type)."s</h1>\n";
	}
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
	    my $auth = $this->author();
	    my $author = $auth ? "by $auth" : "";
	    my $reftitle = $this->reftitle();
	    my $date = $this->date;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= "<p class=\"ZineTeaser\"><a href=\"$url\" class=\"ZineTitle\">$title</a> $author $date in <a href=\"$rurl\" class=\"ZineRef\">$reftitle</a></p>\n";
	}
	$out .= "</div>\n";
    }
    return $out;
}

sub index_latest_articles {
    my ($this,$id,$number) = @_;
    return $this->index_latest("article",$id,$number);
}

sub index_latest_comments {
    my ($this,$id,$number) = @_;
    return $this->index_latest("comment",$id,$number);
}

1;



