#!/usr/bin/perl

package Modules::Zine::Article;

#==========================================================================
#==========================================================================

use strict;
use ExSite::Config;
use ExSite::ReportBuilder;
use ExSite::Misc;
use ExSite::Util;
use ExSite::HTML;
use ExSite::Image;
use ExSite::Time;
use ExSite::RSS;
use ExSite::Tree;
use Modules::Zine::Base;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Base);

sub css {
    my $this = shift;
    return "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Zine/zine.css\">\n";
}

sub wrap {
    my ($this,$class,$contents,$tag) = @_;
    return undef if (!$contents);
    $tag = "div" if (! $tag);
    my $c = $this->{debug} ?
	"<div class=\"ZineDebug\">$class</div>$contents" : $contents;
    return "<$tag class=\"$class\">$c</$tag>";
}

######################### show parts of articles

sub header {
    my $this = shift;
    my $stat = $this->handler("ZineHeader");
    return $stat if ($stat);
    my $icon = $this->moderator ? $this->status_icon() : undef;
    my $title = $icon.$this->clean_text($this->title);
    my $class = $this->class;
    my $n = $config{$this->module_name}{heading_level}{$class};
    if ($n =~ /^[123456]$/) {
	$title = "<h$n>$title</h$n>";
    }
    my $subtitle = $this->clean_text($this->show_data("subtitle"));
    if ($subtitle) { $subtitle = $this->wrap("ZineSubtitle","<em>$subtitle</em>"); }
    my $bookmark = "<a name='Article$this->{article_id}'></a>";
    return $this->wrap("ZineHeader",$bookmark.$this->wrap("ZineTitle",$title).$subtitle);
}

sub about {
    my ($this,$datetype) = @_;
    my $stat = $this->handler("ZineAbout");
    return $stat if ($stat);

    my ($show_auth, $show_date, $about, $author, $date, $summary);
    $show_auth = $this->rule("showauthor");
    $show_date = $this->rule("showdate");
    
#    if ($this->type eq "comment") {
#	$show_auth = 1;
#    }
#    elsif ($this->get("reference")) {
#	# this article belongs to a zine
#	$show_auth = $this->{config}{show_author}{$this->reftype}[1];
#    }
#    else {
#	# this article IS a zine
#	$show_auth = $this->{config}{show_author}{$this->type}[0];
#    }
    if ($show_auth) {
	$author = $this->author;
	if ($author) { 
	    $about = $this->wrap("ZineAuthor","by ".$this->author,"span"); 
	}
    }
#    if ($this->class =~ /article|comment/) {
    if ($show_date) {
	$date = $this->date($show_date);
	$about .= ($author ? ", " : "").$this->wrap("ZineDate",$date,"span");
    }
    # this line escapes all HTML codes:
    #$summary = $this->show_data("summary");
    # this line permits HTML:
    $summary = $this->{article}{summary};
    if ($summary) {
	$about .= $this->wrap("ZineSummary",$summary);
    }

    # show status prominently in adminstrator views
#    if ($this->{admin}) {
#	if ($this->{article}{status} ne "active") {
#	    my $status = ucfirst $this->{article}{status};
#	    $about .= "<br>".$this->wrap("Zine$status",
#					 $this->{article}{status},
#					 "span");
#	}
#    }

    #return $this->wrap("ZineAbout",$about);
    return $about; #$this->wrap("ZineAbout",$about);
}

sub footer {
    my $this = shift;
    my $stat = $this->handler("ZineFooter");
    return $stat if ($stat);
    my $footnotes = $this->show_data("about");
    my $tools = $this->tools;
    # disable conditional wrapping - we need the wrapper in some cases,
    # even if there is no content.
    #return $this->wrap("ZineFooter",$footnotes.$tools);
    return "<div class=\"ZineFooter\">$footnotes$tools</div>";
}

# should we show photos for some article types??
sub abstract {
    my $this = shift;
    my $stat = $this->handler("ZineAbstract");
    return $stat if ($stat);

    if ($this->{article}{summary}) {
	# this escapes HTML codes:
	#return $this->show_data("summary");
	# this allows HTML codes:
	return $this->{article}{summary};
    }
    my $out = $this->body;
    # remove HTML 
    $out =~ s/<.+?>//g;
    my $teaserlen = $config{$this->module_name}{teaser_length} || 200;
    if (length($out) > $teaserlen) {
	my $pos = $teaserlen; 
	my $char;
	while (($pos < length($out)) && ($char = substr($out,$pos,1))) {
	    last if ($char =~ /\s/);
	    $pos++;
	}
	substr($out,$pos) = "...";
    }
    return $out;
}

sub body {
    my $this = shift;
    my $stat = $this->handler("ZineBody");
    return $stat if ($stat);
    my $body = $this->get("body");

    # convert to legal HTML
    $body = $this->preprocess_html($body);

    if ($this->class eq "article") {

	# cross-link to other articles

	$body =~ s/\(\((.+?)(\|(.+?))?\)\)/$this->link_to_name($1,$3)/egs;
    }
    elsif ($this->class eq "comment") {

	# try to impose some style rules on the quoted sections, in case
	# the designer wants to highlight them

	my @text = split /\n/,$body;
	my $quote = 0;
	foreach my $line (@text) {
	    if ($line =~ /^(>|&gt;)/) {
		if ($this->{config}{remove_quote_char}) {
		    $line =~ s/^(>|&gt;)//;
		}
		if (! $quote) {
		    $line = "<div class=\"ZineQuote\">".$line;
		    $quote = 1;
		}
	    }
	    elsif ($quote) {
		$line = "</div>".$line;
		$quote = 0;
	    }
	}
	$body = join("\n",@text)."\n";

	# emoticons

	if ($this->{config}{emoticons}) {
	    foreach my $key (@{$this->{config}{emoticon_re}}) {
		$body =~ s/$key/<img src="$config{server}{HTMLpath}\/_Modules\/Zine\/$this->{config}{emoticon_img}{$key}">/g;
	    }
	}

	if ($this->{config}{body_strut_size} || $this->{admin}) {
	    # This is a kludge for pseudo-table layouts, where the body
	    # column may end up being shorter than the about column.  You
	    # can add a body strut to force a minimum body height.  
	    # Ironically, in order to make the wrapper DIVs look like they
	    # are really a table, we have to use a real table.  
	    # Gotta love CSS...
	    my $strut_height = $this->{config}{body_strut_size} || 75;
	    $body = "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\"><tr>
<td valign=\"top\">$body</td>
<td><img src=\"$config{server}{HTMLpath}/_ExSite/images/null.gif\" width=\"1\" height=\"$strut_height\" align=\"right\" border=\"0\"></td>
</tr></table>\n";
	}

    }

    # attachments?

    return $body;
}

# preprocess a piece of text to make it HTML-friendly
# Article elements are often entered as plain text or partial-html, so
# we need to prep it for insertion into a full-html document.

sub preprocess_html {
    my ($this,$text) = @_;

    # kill/neuter any hidden CMS tags
    if (! $this->{config}{allow_cms_tags}) {
	$text =~ s/<!--/<!-- /g;
    }
    if (! $this->{config}{allow_cms_links}) {
	$text =~ s/\{\{(?![^<>]*>)/\{&\#00;\{/g;
	$text =~ s/\}\}(?![^<>]*>)/\}&\#00;\}/g;
	$text =~ s/\[\[(?![^<>]*>)/\[&\#00;\[/g;
	$text =~ s/\]\](?![^<>]*>)/\]&\#00;\]/g;
    }

    # do we trust any embedded html?
    if (! $this->is_trusted()) {
	$text = &safehtml($text);
    }

    # process linebreaks
    if ($this->class eq "comment") {
	$text =~ s/\n/<br>\n/g;       # convert NL to <br>
    }
    elsif ($this->{article}{content_type} !~ /html/i) {
	# insert paragraph tags at any double-linebreaks
	$text =~ s/\r\n\r\n/<P>\n/g;  # handles CR-LF linebreaks
	$text =~ s/\n\n/<P>\n/g;      # handles LF linebreaks
    }
    return $text;
}

sub size {
    my $this = shift;
    return length $this->{article}{body};
}

######################### show complete articles

# display a complete article and its associated indexes

sub show {
    my ($this,%opt) = @_;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, this $type is not available for viewing.");
    }
    my $stat = $this->handler("ZineShow");
    return $stat if ($stat);
    my $out = $this->show_path(%opt);
    $out .= $this->show_article(%opt);

    my $class = $this->class();
    my $refclass = $this->refclass();
    my ($zindex,$aindex,$cindex);

    # zine index

#    if ($class eq "zine") {
#	$opt{mode} = "zine";    # use a type, not a class
#	$aindex = $this->index(%opt);
#    }

    # article index

    if ((($class eq "zine") && $this->rule("allow","article")) ||
	(($class eq "article") && $this->rule("nest","article"))) {
	$opt{mode} = "article";
	$aindex = $this->index(%opt);
    }

    # comment index
    
    if (($class eq "article") || 
	($class eq "zine" && $this->rule("allow","comment")) ||
	($class eq "comment" && $this->rule("nest","comment"))) {
	$opt{mode} = "comment";
	$cindex = $this->index(%opt);
    }

    if ($aindex) {
	if ($cindex) { 
	    my $type = ucfirst $this->rule("whatami","article");
	    $out .= "<h3>${type}s</h3>\n"; 
	}
	$out .= $aindex;
    }
    if ($cindex) {
	if ($aindex || $class ne "comment") { 
	    my $type = ucfirst $this->rule("whatami","comment");
	    $out .= "<h3>${type}s</h3>\n"; 
	}
	$out .= $cindex;
    }
    return $out;
}

# display a complete article

sub show_article {
    my $this = shift;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, $type $this->{article_id} is not available for viewing.");
    }
    my $stat = $this->handler("ZineShowArticle");
    return $stat if ($stat);
    # build the article structure up from component elements
    #
    ### should we conditionally float the picture, depending on its size?
    ### Or, depending on whether there is any body to float around?
    #
    my $picture = $this->picture();
    my $attachment = $this->wrap("ZineAttachment",$this->attachment);
    my $body = $this->wrap("ZineBody",$picture.$this->body.$attachment);
    my $about = $this->wrap("ZineAbout",$this->about);
    my $about_body = $this->wrap("ZineAboutBody",$about.$body);
    my $head = $this->header;
    my $foot = $this->footer;  # includes tools
    if ($this->{article}{type} eq "comment") {
	return $this->wrap("ZineComment",$head.$about_body.$foot)."\n";
    }
    else {
	return $this->wrap("ZineArticle",$head.$about_body.$foot)."\n";
    }
}

sub show_teaser {
    my $this = shift;
    my $stat = $this->handler("ZineShowTeaser");
    return $stat if ($stat);
    my $out = $this->show_link;
    my $author = $this->author;
    if ($author) { 
	my $show_auth = $this->rule("showauthor");
	if ($show_auth) { $out .= " by $author<br>\n"; }
    }
    $out .= $this->wrap("ZineAbstract",$this->abstract);
    my $comment_count = $this->count($this->{article_id},"comment");
#    my $comment_count = $share{DB}->count("article",{
#	reference=>$this->{article_id},
#	status=>"active",
#	type=>"comment",
#    });
    my $url = $this->link(zine=>"show",aid=>$this->{article_id});
    $out .= "&rarr; <a href=\"$url\">Read more...</a> ";
    if ($comment_count) {
	$out .= "($comment_count comments)";
    }
    $out .= "\n";
    return $out;
}

sub show_link {
    my ($this,$anchor) = @_;
    my $url = $this->link(zine=>"show",aid=>$this->{article_id},type=>undef);
    my $icon = $this->status_icon();
    my $label = $anchor ? $anchor : $this->title;
    return "$icon<a href=\"$url\" class=\"ZineLink\">$label</a>";
}

sub show_info {
    my $this = shift;
    my $stat = $this->handler("ZineShowInfo");
    return $stat if ($stat);
    my $link = $this->show_link;
    my $author = $this->author;
    my $date = $this->date;
    return $author ? 
	"$link by $author;  $date" :
	"$link &mdash; $date";
}

sub show_row {
    my ($this,$data,@col) = @_;
    my $stat = $this->handler("ZineShowRow",$data,@col);
    return $stat if ($stat);
    if (! defined $data) { $data = 1; }
    if (@col == 0) { @col = qw(title author ctime); }
    my ($celltype,@colout);
    if ($data) {
	foreach my $c (@col) {
	    push @colout, $c eq "title" ? 
		$this->show_link() :
		$this->show_data($c);
	}
	$celltype = "td";
    }
    else {
	foreach my $c (@col) {
	    my %cmap = $share{DB}->{map}->get_column("article",$c);
	    push @colout, $this->label(\%cmap);
	}
	$celltype = "th";
    }
    return "<$celltype>".join("</$celltype>\n<$celltype>",@colout)."</$celltype>\n";
}

# show an article in full, with a summary index

sub show_full {
    my ($this) = @_;
    my $stat = $this->handler("ZineShowFull");
    return $stat if ($stat);
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, you don't have permission to view this $type.");
    }
    my $out = $this->show_article;
    my $article_count = $this->count($this->{article_id},"article");
    my $comment_count = $this->count($this->{article_id},"comment");
    if ($article_count || $comment_count) {
	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
	$out .= "(";
	if ($article_count) {
	    $out .= "$article_count more articles; ";
	}
	if ($comment_count) {
	    my $comment_type;
	    if ($comment_count > 1) { 
		$comment_type = ($this->class() eq "article") ? 
		    "comments" : "replies";
	    }
	    else {
		$comment_type = ($this->class() eq "article") ? 
		    "comment" : "reply";
	    }
	    $out .= "$comment_count $comment_type; ";
	}
	$out .= "<a href=\"$url\">Read more...</a>)";
    }
    return $out;
}

sub status_icon {
    my $this = shift;
    # return undef if ($this->class eq "zine");
    return undef if (! $this->admin_mode);
    return undef if (! $this->moderator);
    return "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/$this->{config}{status_icon}{$this->{article}{status}}\" height=16 width=16 alt=\"($this->{article}{status})\" title=\"($this->{article}{status})\" class=\"ZineStatusIcon\">";
}

##################### show images and attachments

sub attachment {
    my ($this,@att) = @_;
    my $stat = $this->handler("ZineAttachment",@att);
    return $stat if ($stat);
    if (@att == 0) {
	@att = $share{DB}->fetch_match("article",{
	    parent_id=>$this->get("article_id"),
	    type=>"attachment",
	    status=>"active",
	});
	return "" if (@att == 0);
    }
    my @attout;
    foreach my $attachment (@att) {
	my $img = new ExSite::Image($attachment->{picture});
	my $name = $img->name;
	my $type = &MimeType($name);
	my $att;
	if ($type =~ /^image/) {
	    my $url = $this->link(zine=>"show",aid=>$attachment->{article_id});
	    $att = "<a href=\"$url\">".$this->thumb($attachment)."</a>";
	}
	else {
	    my $url = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
	    $att = "<a href=\"$url\">$name</a>";
	}
	push @attout, "<table><tr><td>$att</td></tr><tr><td><div class=\"ZineCaption\">$attachment->{caption}</div></td></tr></table>";
    }
    return join("\n",@attout);
}

sub picture {
    my ($this,$img) = @_;
    my $stat = $this->handler("ZinePicture",$img);
    return $stat if ($stat);
    my $out;
    if (! $img) {
	return undef if (! $this->{article}{picture});
	$img = new ExSite::Image($this->{article}{picture});
    }
    my ($width,$height) = $img->dim;
    my $filename = "$this->{article_id}_".&clean_filename($img->{filename});
    if (! -e $this->diskpath."/".$filename) {
	# not published; go to the database
	$filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
    }
    else {
	$filename = $this->htmlpath."/".$filename;
    }
    $out = "<table class=\"ZineImage\" width=\"$width\"";
    $out .= "><tr><td><img src=\"$filename\" height=\"$height\" width=\"$width\"></td></tr>";
    my $cap = $this->show_data("caption");
    if ($cap) {
	$out .= "\n<tr><td><div class=\"ZineCaption\">$cap</div></td></tr>";
    }
    $out .= "</table>";
    my $img_wrap_size = $config{$this->module_name}{img_max_wrap_size} || 400;
    my $body_wrap_size = $config{$this->module_name}{body_min_wrap_size} || 1000;
    if ($width < $img_wrap_size && $this->size > $body_wrap_size) {
	# we already have table.ZineImage to apply special styles to;
	# conditional wrapping with div.ZineImage allows us to conditionally
	# float the image depending on its size.
	return $this->wrap("ZineImage",$out);
    }
    return $out;
}

sub thumb {
    my ($this,$art) = @_;
    my $stat = $this->handler("ZineThumb");
    return $stat if ($stat);
    my $out;
    $art = $art || $this->{article};
    if ($art->{thumb}) {	
	my $img = new ExSite::Image($art->{thumb});
	my $filename = "$art->{article_id}T_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=thumb&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	$out = "<img class=\"ZineThumb\" src=\"$filename\" border=\"0\"";
    }
    elsif ($art->{picture}) {
	my $img = new ExSite::Image($art->{picture});
	my $filename = "$art->{article_id}_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	my $thumbdim = $config{$this->module_name}{thumbdim} || 100;
	$out .= "<img class=\"ZineThumb\" src=\"$filename\" width=\"$thumbdim\" border=\"0\">";
    }
    return $out;
}

# tools -
# new article
# comment
# reply
# configure
# delete
# approve
# reject
# current/archives
# rss

sub tools {
    my ($this,$labels,$orientation,%conf) = @_;
    my ($out,$url,$ntool,$article,$aid);
    $article = $this->{article};
    $aid = $article->{article_id};

    if (scalar keys %conf == 0) {
	my $allow_comment = $this->allow_comments();
	my $allow_article = $this->allow_articles();
	# show default tools
	$conf{reply} = 
	    $article->{type} eq "comment" && 
	    $allow_comment;
	$conf{comment} = 
	    $this->class() =~ /zine|article/ && 
	    $allow_comment;
#	    $article->{type} =~ /article|forum/ && 
#	    $this->allow_comments;
#	    $this->allow_comments("article");
	$conf{link} = 
	    # permalink in article indexes
	    $article->{type} =~ /article/ && $aid != $this->{input}{aid};
	$conf{new} = 
	    $allow_article && 
	    ($this->{admin} || ($share{Page} && ! $share{Page}->publishing));
### use new policy rules
#	    # all zines get a new article link
#	    ( $this->{admin} && 
#	      $this->class eq "zine" ) || 
#	    # books get a new article link under every article
#	    ( $this->{admin} && 
#	      $this->{type} eq "article" && 
#	      $allow_article ) ||
#	    # blogs get a new article link on normal page views
#	    ( $article->{type} =~ /blog/ &&
#	      $article->{owner} == $share{DB}->my_uid &&
#	      $share{Page} && ! $share{Page}->publishing );
	$conf{edit} = 
	    $this->{admin} ||
	    # commenters can optionally edit their own comments
	    ( $article->{type} eq "comment" && 
	      $share{Page} && ! $share{Page}->publishing &&
#	      $article->{owner} == $share{DB}->my_uid &&
	      $allow_comment );
	$conf{configure} = 
	    $this->{admin};
	$conf{delete} = 
	    $this->{admin};
	$conf{approve} = 
	    $article->{status} =~ /inactive|pending|rejected/ &&
	    $this->moderator;
	# $conf{rss} and $conf{archive} are only set manually
    }

    if (! defined $labels) { 
	$labels = $this->{config}{icon_labels} || "1";
    }
    if (! defined $orientation) { 
	$orientation = $this->{config}{tool_orientation} || "horizontal";
    }
    if (! exists $this->{tools}) { $this->{tools} = {}; }

    my (@toolicon,@toollabel,@toolurl);
    if ($conf{reply}) {
	push @toolurl, $this->link(zine=>"reply",aid=>$aid);
	push @toolicon, $this->tool_icon("reply","reply to this");
	push @toollabel, $this->tool_label("reply");
	$ntool++;
    }
    elsif ($conf{comment}) {
	push @toolurl, $this->link(zine=>"new",aid=>$aid);
	push @toolicon, $this->tool_icon("comment","new comment");
	push @toollabel, $this->tool_label("comment");
	$ntool++;
    }

    if ($conf{link}) {
	push @toolurl, $this->link(zine=>"show",aid=>$aid);
	push @toolicon, $this->tool_icon("link","permalink");
	push @toollabel, $this->tool_label("link");
	$ntool++;
    }

    if ($conf{new}) {
	push @toolurl, $this->link(zine=>"newarticle",aid=>$article->{article_id});
	push @toolicon, $this->tool_icon("new","new article");
	push @toollabel, $this->tool_label("new");
	$ntool++;
    }

    if ($conf{edit}) {
	my $edit = 1;
	if (! $this->{admin}) {
	    # we can only edit our comments within a few minutes of posting
	    my $t = new ExSite::Time;
	    my $t1 = $t->write("unix_timestamp");
	    $t->set($article->{ctime},"sql_timestamp");
	    my $t2 = $t->write("unix_timestamp");
	    $edit = ($t1 - $t2 < $config{$this->module_name}{edit_my_comment_timeout});
	}
	if ($edit) {
	    push @toolurl, $this->link(zine=>"edit",aid=>$article->{article_id});
	    push @toolicon, $this->tool_icon("edit","edit");
	    push @toollabel, $this->tool_label("edit");
	    $ntool++;
	}
    }

    if ($conf{configure}) {
	push @toolurl, $this->link(zine=>"conf",aid=>$article->{article_id});
	push @toolicon, $this->tool_icon("tool","configure");
	push @toollabel, $this->tool_label("configure");
	$ntool++;
    }

    if ($conf{delete}) {
	my $type = $this->type;
	push @toolurl, "javascript:confirm_custom('Delete $type $this->{article_id} and all of its related articles and comments?','".$this->link(zine=>"del",aid=>$article->{article_id})."')";
	push @toolicon, $this->tool_icon("delete","delete"); 
	push @toollabel, $this->tool_label("delete");
	$ntool++;
    }

    if ($conf{approve}) {
	push @toolurl, $this->link(zine=>"approve",aid=>$article->{article_id})."#Article$article->{article_id}";
	push @toolicon, $this->tool_icon("approve","approve");
	push @toollabel, $this->tool_label("approve");
	$ntool++;
	if ($article->{status} ne "rejected") {
	    push @toolurl, $this->link(zine=>"reject",aid=>$article->{article_id})."#Article$article->{article_id}";
	    push @toolicon,  $this->tool_icon("reject","reject");
	    push @toollabel, $this->tool_label("reject");
	    $ntool++;
	}
    }

    if ($conf{back}) {
	push @toolurl, $this->link(zine=>"show",aid=>$article->{reference});
	push @toolicon, $this->tool_icon("back","back");
	push @toollabel, $this->tool_label("back");
	$ntool++;
    }

    if ($conf{archive}) {
	if ($this->{input}{archive}) {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>undef);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : $this->tool_label("current");
	}
	else {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>1);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : $this->tool_label("archives");
	}
	push @toolicon, $this->tool_icon("archive","view archives");
	$ntool++;
    }

    if ($conf{rss} && $share{Page}) {
	my $rssfile = $this->diskpath."/index.rss";
	push @toolurl, (-e $rssfile ? 
			$this->htmlpath."/index.rss" : 
			$this->link(zine=>"rss",aid=>$this->{article_id}));
	push @toolicon, $this->tool_icon("rss","RSS");
	push @toollabel, $this->tool_label("rss");
	$ntool++;
    }

    if ($ntool) {
	$out = "<table class=\"ZineTools\"><tr>\n";
	for (my $itool = 0; $itool < $ntool; $itool++) {
	    my $url = shift @toolurl;
	    my $icon = shift @toolicon;
	    my $label = shift @toollabel;
	    if (! $labels) {
		$out .= "<td><a href=\"$url\">$icon</a></td>\n";
	    }
	    elsif ($orientation eq "horizontal") {
		$out .= "<td><a href=\"$url\">$icon<br>$label</a></td>\n";
	    }
	    else {
		$out .= "<td><a href=\"$url\">$icon</a></td><td><a href=\"$url\">$label</a></td></tr>\n<tr>";
	    }
	}
	$out .= "</tr></table>\n";
    }

    return $out;
}

sub tool_icon {
    my ($this,$iconname,$alt) = @_;
    my $style = $config{$this->module_name}{icon_style} || "gif";
    my $filename = "$iconname.$style";
    return "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/$filename\" height=\"16\" width=\"16\" border=\"0\" alt=\"$alt\" title=\"$alt\">";
}

sub tool_label {
    my ($this,$label) = @_;
    if (! $this->{toollabels}) {
	$this->{toollabels} = $config{$this->module_name}{tool} || {};
    }
    return $this->{toollabels}{$label} || $label;
}

##################### show indexes

sub index {
    my ($this,%opt) = @_;

    # try a handler first
    my $stat = $this->handler("ZineIndex",%opt);
    return $stat if ($stat);

    my $out;
    my ($nart,$narch);
    my $class = $this->class();

    # save current article
    $this->save;

    # get index display parameters
    $opt{mode} = $opt{mode} || "article";
    $opt{sort} = $opt{sort} || $this->indexsort($opt{mode});
    $opt{format} = $opt{format} || $this->indexformat($opt{mode});
    $opt{limit} = $opt{limit} || $this->indexlimit($opt{mode});
    $opt{minimum} = $opt{minimum} || $this->rule("indexmin",$opt{mode});
    $opt{age} = $opt{age} || $this->indexage($opt{mode});
    if ($opt{mode} eq "comment" and $this->type ne "forum") {
	# do not index comments in archive mode
	$opt{archive} = 0;
    }

    # layout
    if ($opt{archive}) {
	$opt{layout} = "archive";
    }
    else {
	if (! $opt{layout}) { $opt{layout} = $this->indextype($opt{mode}); }
	if ($opt{layout} eq "basic") { $opt{layout} = ""; }
    }

    # run the indexing engine
    my $indexengine = ucfirst($opt{layout});
    $indexengine =~ s/[^\w]//g;
    my $indexmod = "Modules::Zine::Index$indexengine";
    #-- make safe for publishing
    $indexmod =~ /^(Modules::Zine::Index\w*)$/;
    my $safe_indexmod = $1;
    #--
    if ($safe_indexmod) {
	if (eval "require $safe_indexmod") {
	    # preserve admin state in the index object
	    my $index = $indexmod->new($this->{article},{admin=>$this->{admin}});
#	    my $index = $indexmod->new($this->{article});
	    if ($index) {
		$out = $index->index(%opt);
		$nart =  $index->article_count();
		$narch = $index->archive_count();
	    }
	    else {
		$out = $this->error("$indexmod: $@");
	    }
	}
	else {
	    $out = $this->error("$indexmod: $@");
	}
    }
    else {
	$out = $this->error("$indexmod: invalid index module");
    }
    $out = $this->wrap("ZineIndex",$out);

    # reset article
    $this->restore;

    # show public zine tools (archive/rss)
    my (%tools);
    if ($opt{archive}) {
	my $url = $this->link(archive=>undef);
	if ($out) {
	    $tools{archive} = "Current $opt{mode}s";
	}
	elsif ($opt{mode} eq "article" && $class eq "zine") {
	    $tools{archive} = "Back to current articles";
	}
    }
    elsif ($narch > 0) {
	$tools{archive} = "Archives";
    }
    if ($class eq "zine" && $opt{mode} eq "article" && $out) {
	$tools{rss} = 1;
    }
    if ($this->{article}{reference} && $out) {
	$tools{back} = 1;
    }

    # In certain cases, it's worth noting that the index is empty.
    if (! $nart &&
	$opt{mode} eq "article" &&
	$class eq "zine" && $this->rule("allow","article") && 
	! $opt{archive}) { 
	# only show the "no articles" message if we're indexing a Zine
	# that allows articles.  ie. don't show it if we're attempting to 
	# show a sub-index under a simple article, or a forum (which only
	# accepts comments, not articles).
	my $article_name = $this->confrule("whatami","article",$this->zinetype);
	$out = "<p>No current ${article_name}s found.</p>";
    }

    if (keys %tools > 0) {
	$out .= $this->tools(1,"vertical",%tools);
    }
    return $out;
}

# RSS feeds

sub rss {
    my ($this, @art) = @_;
    my $stat = $this->handler("ZineRSS");
    return $stat if ($stat);
    my ($title,$description,$currart,$url,@art);
    my $server = $config{server}{server};
    if (@art == 0) {
	my $limit = $this->{config}{rss_limit} || 10;
	my $zinetype = $this->zinetype();
	my $rsstype = $this->confrule("allow","article",$zinetype) ?
	    "article" : "comment";
	$this->save;
	$this->set( $this->class eq "zine" ? 
		    $this->{article} : $this->{reference});
	$title = $this->title;
	$description = $this->abstract;
	@art = $this->fetch_child_articles($rsstype,$limit);
	$url = $this->link(zine=>"show",
			   aid=>$this->class eq "zine" ? 
			   $this->{article_id} : 
			   $this->{reference}{article_id});
	$this->restore;
    }
    else {
	$title = "Articles";
	$description = "from $config{site}{name}";
	$url = $server;
    }
    return undef if (@art == 0);

    my $server = $config{server}{server};

    my $rss = new ExSite::RSS;
    $rss->title(&html_to_plaintext($title));
    $rss->description(&html_to_plaintext($description));
    $rss->link($server.$url);

    $this->save;
    foreach my $a (@art) {
	$this->set($a);
	$rss->item(&html_to_plaintext($this->title),
		   &html_to_plaintext($this->abstract),
		   $this->link(zine=>"show",aid=>$this->{article_id})
		   );
    }

    $this->restore;
    return $rss->write;
}

1;



