#!/usr/bin/perl

package Modules::Zine::Index;

=pod

=head1 Modules::Zine::Index

This is a plug-in indexing engine to display article indexes in a
simple sequence of block elements.  It will require some CSS to
provide visual structure that isn't obvious in the plain output.

A plug-in indexing engine must support the following methods, which
will be called by the main article-handling system:

=over 4

=item * index(%opt)

This receives a hash of indexing options, and must fetch, sort, and
filter the articles as appropriate.  It returns an HTML index allowing
the articles to be browsed by the viewer.  The input options are
generally set up by C<Modules::Zine::Article::index>, based on
configurations in the article, zine, and overall system:

=over 4

=item mode

B<article> or B<comment>, depending on what we are supposed to be indexing

=item sort

A sorting rule for ordering the index.  Options are B<sorted> (ordered
by the articles' C<sortkey>), B<date> (earliest first),
B<reverse-date> (latest first), B<flip-flop> (by date for top-level,
reverse-date otherwise), and B<reverse-flip-flop> (by reverse-date for
top-level, by date otherwise).

=item format

How each item in the index ought to be formatted.  Options are
B<title>, B<basic>, B<summary>, or B<full>.  (See below for
explanation.)

=item limit

The maximum number of items that should be shown in the index.

=item minimum

The minimum number of items that should be shown in the index.

=item age

The maximum age (in days) of items that should be shown in the index.

=item layout

The index engine that should be used;  in this case, it will be "basic".

=back

=item * archive()

This returns an integer representing the number of articles that were
excluded from the index because they were archived due to excessive
age, too many articles to show, or explicit archived status.  This is
also used as a flag indicating whether a link to the archives is
required (the link is automatically provided if C<archive()> returns a
non-zero value).

=back

Many indexing engines will also support a C<fetch_index()> method to
retrieve a list of articles from the database as candidates to be
included in the index.  The parameters accepted by this method, and
the values returned by it, vary from engine to engine, however.  It
may nevertheless be of use with handlers that attempt to override the
indexing algorithm.  Similarly, engines commonly support a
C<show_item()> method to display a single item in the index, but it is
not predefined how individual items interact or nest with other HTML
elements in the index.

The engine may also include other internal methods for selecting
articles and formatting the index.  These may in turn be inherited by
other indexing engines that work on similar principles.

In particular, the engine must choose how to format each item in the
index.  The C<indexformat> parameter gives the format that is
recommended by the webmaster or system.  This is only a suggestion,
and can be ignored/overridden if necessary.  The commonly-used formats
are:

=over 4

=item title

Only the article title is shown.

=item basic

The article title, author, and date are shown.

=item summary

The article title is shown along with a brief teaser, summary, or
abstract.

=item full

The complete article is shown.

=back

The B<Basic> indexing engine supports all of these.

The B<Basic> indexer structures the index as a simple linear list.
Only the immediate children are shown (ie. if the complete index has a
multi-level structure, then only the first level is shown; to view
lower levels, you have to click into particular articles, and view
their sub-indexes).  The article selection method, C<fetch_index>, may
be inherited by other indexers that work on similar principles.

=cut

use strict;
use ExSite::Config;
use Modules::Zine::Article;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Article);

sub index {
    my ($this,%opt) = @_;

    # get all the articles in the index
    my @articles = $this->fetch_index(%opt);

    # format the index
    return $this->index_basic(\%opt,@articles);
}

sub fetch_index {
    my ($this,%opt) = @_;

    my $mode = $opt{mode} || "article";

    # start with all children
    my $opt = $this->admin_mode() ? { status=>undef } : undef;
    my @art = $this->fetch_child_articles($mode,undef,$opt{sort},$opt);

    # find and filter the archived articles
    my @archive;
    my $iart = 0;
    $this->{narch} = 0;
    my $class = $this->class;

    # flag all old/expired articles for the archives
    foreach my $a (@art) {
	$this->set($a);
	if (($class eq $opt{mode}) ||
	    ($opt{mode} eq "article" && $class eq "zine")) {

	    if ($opt{age} && $opt{age} > 0 && $this->age > $opt{age}) {
		# article too old
		$archive[$iart] = 1;
		$this->{narch}++;
	    }
	    elsif ($opt{limit} && $opt{limit} > 0 && 
		   ($iart-$this->{narch}) >= $opt{limit}) {
		# too many articles
		$archive[$iart] = 1;
		$this->{narch}++;
	    }
	    $iart++;
	}
    }

    # now, unflag enough recently expired articles to keep us under the
    # index limit;  ignore for specialized indexes that don't understand
    # minimum (we may end up here in archive mode for those indexes).
    if ($opt{minimum}) {
	$iart = -1;
	if ((scalar @art - $this->{narch}) < $opt{minimum}) {
	    foreach my $a (@art) {
		$iart++;
		next if (!$archive[$iart]);
		$archive[$iart] = 0;
		$this->{narch}--;
		last if ((scalar @art - $this->{narch}) >= $opt{minimum});
	    }
	}
    }

    # build a new list of the non-archived articles
    my @showart;
    my $archive_on = 0;
    for (my $iart = 0; $iart <= $#art; $iart++) {
	if (! $archive[$iart]) {
	    push @showart, $art[$iart];
	}
    }
    $this->{nart} = scalar @showart;
    return wantarray ? @showart : \@showart;
}

sub index_basic {
    my ($this,$opt,@articles) = @_;

    # save current article
    $this->save;

    # build normal list-like indexes
    my ($index);
    foreach my $art (@articles) {
	$this->set($art);
	$index .= $this->show_item($opt->{format})
    }

    # reset article
    $this->restore;

    return $index;
}

sub show_item {
    my ($this,$format,$tree) = @_;
    my $out;
    if ($format eq "title") {
	$out = $this->show_link;
    }
    elsif ($format eq "summary") {
	$out = $this->show_teaser;
	### add summary index
    }
    elsif ($format eq "full") {
	$out = $this->show_full($tree);
	# summary index
    }
    else { #if ($opt{format} eq "basic") {
	$out = $this->show_info;
    }
    return $this->wrap("ZineItem",$out);
}

sub archive_count {
    my $this = shift;
    return $this->{narch};
}

sub article_count {
    my $this = shift;
    return $this->{nart};
}

1;

