#!/usr/bin/perl

package Modules::Zine::IndexExploding;

=pod

=head1 Modules::Zine::IndexExploding

This is a plug-in indexing engine to display article indexes in a 
exploding/collapsing DHTML format.

The index is displayed fully threaded, but only the top article of
each thread is shown by default.  Each article is tagged with a button
(a right- or down- pointing arrow) that can be clicked to explode or 
collapse that sub-thread.

This indexing engine inherits from IndexThreaded, as it borrows some
thread-management routines from there.

Articles are fetched as an ExSite::Tree object, which automatically
constructs the nesting structure for us.

The top-level of articles define the thread; all threads are shown on
one page.  Threads older than C<indexage> are excluded from the index,
and can only be viewed in the archives.  If the total number of
threads exceeds C<indexlimit>, the oldest threads are also moved to
the archives.

=cut

use strict;
use ExSite::Config;
use ExSite::Tree;
use ExSite::HTML;
use Modules::Zine::IndexThreaded;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::IndexThreaded);

sub index {
    my ($this,%opt) = @_;
    my $start = $this->{article_id};
    return undef if (! $start);

    # save current article
    $this->save;

    # get all the articles in the index
    my @articles;
    my $match = {};
    my $class = $this->class();
    my $tree;

    if ($class eq "zine") {
	$match->{reference} = $start;
    }
    elsif ($class eq "article") {
	if ($opt{layout} eq "paged" && $opt{mode} ne "comment") {
	    $start = $this->{article}{reference};
	    $match->{reference} = $start;
	}
	elsif ($opt{mode} eq "comment") {
	    $match->{reference} = $start;
	}
	else {
	    $match->{reference} = $this->{article}{reference};
	}
    }
    else {
	    $match->{reference} = $this->{article}{reference};
    }
    $tree = $this->fetch_tree($start,$match);

    return $this->index_exploding(\%opt,$tree);
}

sub index_exploding {
    my ($this,$opt,$tree) = @_;
    my $stat = $this->handler("ZineIndexExploding",$opt,$tree);
    return $stat if ($stat);

    my $out;
    my $mode = $opt->{mode};
    my $indexage = $opt->{indexage} || $this->indexage($mode);

    # build a tree from the articles
    if (! $tree) { $tree = $this->fetch_tree($this->{article_id}); }

    my $archive;
    $this->{articlecount} = 0;

    # prune uninteresting articles from the thread tree
    #foreach my $art ($tree->get_topnodes_data()) {
    foreach my $art ($tree->get_child_data($this->{article_id})) {
	# drop comments in article indexes
	if ($art->{type} eq "comment" and $mode eq "article") {
	    $tree->delnode($art->{article_id});
	}
	# drop articles in comment indexes
	if ($art->{type} ne "comment" and $mode ne "article") {
	    $tree->delnode($art->{article_id});
	}
	# prune old threads
	if ($indexage > 0 && $this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $this->{archived_threads}++;
	}
    }

    # display remaining threads
    $this->{thread_depth} = 0;
    my @articles = reverse($tree->get_child_data($this->{article_id}));
    $this->{active_threads} = scalar @articles;
    my $dlist = $this->index_exploding_r($opt,$tree,@articles);
    $out .= &ExSite::HTML::DynList(@$dlist);

    return $out;
}

sub index_exploding_r {
    my ($this,$opt,$tree,@articles) = @_;
    my @dlist;
    foreach my $art (@articles) {
	$this->set($art);
	next if ($this->type() eq "attachment");
	my %item;
#	$item{text} = $this->show_full($tree);
	$item{text} = $this->show_full($opt->{format},$tree);

	### FIXME: give some indication about how many nested levels there are

	$item{label} = $this->title()." ".$this->about();       
	my @children = $tree->get_child_data($art->{article_id});
	$item{sublist} = $this->index_exploding_r($opt,$tree,@children);
	push @dlist,\%item;
    }
    return \@dlist;
}

1;



