#!/usr/bin/perl

package Modules::Zine::IndexThreaded;

=pod

=head1 Modules::Zine::IndexThreaded

This is a plug-in indexing engine to display article indexes in a 
fully-threaded format.

Full threading shows each level of the index in an indented block
under its parent particle, eg.

    article 1
    article 2
        article 2.1
        article 2.2
    article 3
        article 3.1
            article 3.1.1
            article 3.1.2
        article 3.2
    article 4

Indenting is maintained to a maximum number of levels, given by the
C<max_thread_depth> configuration parameter (default is 4).  After
that many levels, all levels are shown at the same depth.

Articles are fetched as an ExSite::Tree object, which automatically
constructs the nesting structure for us.

The top-level of articles define the thread; all threads are shown on
one page.  Threads older than C<indexage> are excluded from the index,
and can only be viewed in the archives.  If the total number of
threads exceeds C<indexlimit>, the oldest threads are also moved to
the archives.

=cut

use strict;
use ExSite::Config;
use ExSite::Tree;
use Modules::Zine::Index;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Index);

sub index {
    my ($this,%opt) = @_;
    # get all the articles in the index
    my $tree = $this->fetch_index(%opt);
    if ($tree) {
	my $index = $this->index_threaded(\%opt,$tree);
	return ($this->type eq "comment") ?
	    "<ul class=\"ZineThread\">$index</ul>\n" :
	    $index;
    }
    return undef;
}

sub fetch_index {
    my ($this,%opt) = @_;
    my $start = $this->{article_id};
    return undef if (! $start);

    # get all the articles in the index
    my $match = {};
    my $class = $this->class();

    if ($class eq "zine") {
	$match->{reference} = $start;
    }
    elsif ($class eq "article") {
	if ($opt{layout} eq "paged" && $opt{mode} ne "comment") {
	    $start = $this->{article}{reference};
	    $match->{reference} = $start;
	}
	elsif ($opt{mode} eq "comment") {
	    $match->{reference} = $start;
	}
	else {
	    $match->{reference} = $this->{article}{reference};
	}
    }
    else {
	$match->{reference} = $this->{article}{reference};
    }
    $this->{mod} = $this->moderator();
    if ($this->{mod}) {
	$match->{status} = undef;
    }
    return $this->fetch_tree($start);
    #return $this->fetch_tree($start,$match);
}

sub index_threaded {
    my ($this,$opt,$tree) = @_;
    my $stat = $this->handler("ZineIndexThreaded",$opt,$tree);
    return $stat if ($stat);

    my $out;
    my $mode = $opt->{mode};
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    my $minimum = $opt->{minimum} || $this->get("indexmin",$opt->{mode});

    # automatically archive only when looking at top views
    # when inspecting particular articles or comments, show thread in full
    $this->{archiving} = ($this->class eq "zine");
    $this->{archived_threads} = 0;
    $this->{articlecount} = 0;

    # prune uninteresting articles from the thread tree
    foreach my $art ($tree->get_child_data($this->{article_id})) {
	# drop invalid types
	if ($art->{type} ne $mode) {
	    $tree->delnode($art->{article_id});
	    next;
	}
	# prune disabled threads
	if (! $this->{mod} && $art->{status} ne "active") {
	    $tree->delnode($art->{article_id});
	    next;
	}
	# prune old threads
	if ($this->{archiving} && 
	    $indexage > 0 && $this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $this->{archived_threads}++;
	}
    }

    # display remaining threads
    $this->{thread_depth} = 0;
    my @articles = reverse($tree->get_child_data($this->{article_id}));
    $this->{active_threads} = scalar @articles;
    $this->save;
    $out .= $this->index_thread($indexformat,$indexlimit,$indexage,
				$tree,@articles);
    $this->restore;

    return $out;
}

sub index_thread {
    my ($this,$format,$limit,$age,$tree,@articles) = @_;
    my $out;

    # replies will be indented up to $max_depth levels
    $this->{thread_depth}++;
    my $max_depth = $this->{config}{max_thread_depth} || 4;

    foreach my $art (@articles) {
	# skip disabled articles in public mode
	next if (! $this->{mod} && $art->{status} ne "active");
	$this->set($art);
	if ($this->type() ne "attachment") {
	    $this->{articlecount}++;
	    my $afmt = $format;
	    if ($afmt ne "brief") {
		if ($this->{archiving}) {
		    if ($this->{articlecount} > $limit) {
			$afmt = $afmt =~ /summary|full/ ? "basic" : $afmt;
		    }
		    elsif ($age > 0 && ($this->age > $age)) {
			$afmt = $afmt =~ /summary|full/ ? "basic" : $afmt;
		    }
		}
		elsif ($this->{config}{shrink_anonymous_comments} && ! $art->{owner}) {
		    $afmt = "basic";
		}
	    }
	    $out .= $this->show_item($afmt,$tree);
	    my @children = $tree->get_child_data($art->{article_id});
	    if (@children > 0) {
		my $indent = 0;
		if ($this->{thread_depth} <= $max_depth) { 
		    $indent = 1;
		    $out .= "<ul class=\"ZineThread\">\n"; 
		}
		$out .= $this->index_thread($format,$limit,$age,$tree,@children);
		if ($indent) { $out .= "</ul>\n"; }
	    }
	}
    }
    $this->{thread_depth}--;
    return $out;
}

sub show_item {
    my ($this,$format,$tree) = @_;
    my $out;
    if ($format eq "title") {
	$out = $this->show_link;
    }
    elsif ($format eq "summary") {
	$out = $this->show_teaser;
    }
    elsif ($format eq "full") {
	$out = $this->show_article();
	# we don't use show_full() because we are fully threaded and have
	# no need for a summary index.
    }
    else { #if ($opt{format} eq "basic") {
	$out = $this->show_info;
    }
    return $this->wrap("ZineItem",$out);
}

# the age of a thread is the age of the youngest article in the thread

sub age_of_thread {
    my ($this,$tree,$article) = @_;
    my $youngest = $this->youngest_of_thread($tree,$article);
    my $age = $youngest->{mtime};
    # convert to a day count
    my $time = new ExSite::Time; # now
    # time diff is in seconds; convert to days
    my $tdiff = ($time->diff($age,"sql_timestamp")) / 86400;
    return - int($tdiff);
}

sub youngest_of_thread {
    my ($this,$tree,$article) = @_;
    my @nodelist = ($tree->getnode_data($article->{article_id}));
    my $youngest = $nodelist[0];
    my $age = $youngest->{mtime};
    while (@nodelist > 0) {
	my $art = shift @nodelist;
	next if ($art->{status} ne "active");
	if (($art->{mtime} cmp $age) > 0) { 
	    $youngest = $art; 
	    $age = $art->{mtime};
	}
	push @nodelist,$tree->get_child_data($art->{article_id});
    }
    return $youngest;
}

sub archive_count {
    my $this = shift;
    return $this->{archived_threads};
}

sub article_count {
    my $this = shift;
    return $this->{active_threads};
}

1;



