#!/usr/bin/perl

package Modules::Zine::Zine;

#==========================================================================
# Zine : an e-Zine management system.
#
# For our purposes, an e-Zine is simply a collection of articles,
# along with the tools to link them together into indexes, search, sort, 
# and add comments/feedback.
#
# This package performs general management of article collections, including:
#    - formatting for display in HTML or plain text formats, 
#    - generating indexes for browsing large numbers of articles
#    - generating forms for inserting/updating articles
#    - processing article forms
#    - handling article attachments
#
#==========================================================================

=pod

Features:

- online books with unlimited chapters, with optional comments
- e-Zines with unlimited articles, with optional comments
- Discussion Forums
- Blogs, with optional comments
- PhotoBlogs, with optional comments
- Wikis, with cross-linked articles and optional comments
- comments can be for members only, or for the public
- flexible indexing (threaded, sorted, paged, by date, gallery-style, and more)
- automatic aging of articles
- add images/attachments to any article, with captions & automatic thumbnailing
- flexible CSS-driven formatting
- easy article handling through website front-end
- advanced e-Zine management through control panel back-end
- tamper-proof web forms


TODO

brief format archive indexes
webservice enabling
zine security - config flags to test site ownership of any article
zine security - secure zines do not publish
status flags, archiving
versioning??
moderating
notifications and subscriptions
pagination for large limited indexes
top nav, bottom nav
allow bare ID, bare zine type, or bare article name to be passed to write
view switcher (eg. to gallery or photoindex) on paged indexes
wiki QA

search, search in zine
built-in login/auth



=cut

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use ExSite::Time;
use ExSite::RSS;
use ExSite::Tree;
use ExSite::Captcha;
use Modules::BaseDCD;

use vars qw(@ISA %formtype %required %comment_label %helplabel 
	    %show_author %sort_methods %article_type %article_class
	    @emoticon %emoticon %search_weight );
@ISA = qw(Modules::BaseDCD);

# fields to prompt for on article forms

%formtype = (
	     'e-zine' => ["title","subtitle","summary","about"],
	     book => ["author","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	     forum =>  ["title","subtitle","summary","sortkey"],
	     comment =>  ["author","title","body"],
	     blog =>  ["title","subtitle","summary","about","picture","caption"],
	     wiki =>  ["title","subtitle","summary","about","picture","caption"],
	     photoblog =>  ["title","subtitle","summary","about"],
	     article =>  ["author","name","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	     article_wiki =>  ["name","author","title","body","content_type","picture","caption"],
	     article_blog =>  ["name","title","body","about","picture","caption","status"],
	     article_photoblog =>  ["author","title","picture","caption"],
	     );

# required fields on article forms

%required = (
	     'e-zine' => ["title"],
	     book => ["title"],
	     forum =>  ["title"],
	     comment =>  ["title","body"],
	     blog =>  ["title"],
	     wiki =>  ["title"],
	     photoblog =>  ["title"],
	     article =>  ["title"],
	     article_wiki =>  ["name","title","body"],
	     article_blog =>  ["title"],
	     article_photoblog =>  ["picture"],
	     );

# first value is for zine, second is for articles
%show_author = (
		'e-zine' => [0,1],
		book => [1,0],
		forum => [0,0],
		comment => [1,1],
		blog => [0,0],
		wiki => [0,0],
		photoblog => [1,0],
		article => [1,1],
		attachment => [0,0],
		);

%article_type = (
		 'e-zine' => "e-Zine",
		 book => "Book",
		 forum =>  "Forum",
		 comment =>  "Comment",
		 blog =>  "Blog",
		 wiki =>  "Wiki",
		 photoblog => "Photo Blog",
		 article =>  "Article",
		 article_wiki =>  "Wiki Article",
		 article_blog =>  "Blog Article",
		 article_photoblog =>  "Image",
		 );

%article_class = (
		  'e-zine' => "zine",
		  book => "zine",
		  forum =>  "zine",
		  comment =>  "comment",
		  blog =>  "zine",
		  wiki =>  "zine",
		  photoblog => "zine",
		  article =>  "article",
		  attachment => "attachment",
		 );

%comment_label = (
		  author => "From",
		  body => "Comment",
		  title => "Subject",
		  );

%sort_methods = (
		 sorted => "sortkey",
		 date => "ctime",
		 "reverse-date" => {"ctime"=>"desc"},
		 "flip-flop" => "ctime",
		 "reverse-flip-flop" => {"ctime"=>"desc"},
		 );

%search_weight = (
		  title=>3,
		  body=>1,
		  author=>2,
		  name=>3,
		  subtitle=>2,
		  summary=>1,
		  about=>1,
		  caption=>1,
		  );

# This sets the order in which emoticons are substituted;  it can also be
# used to enable/disable certain emoticons.
@emoticon = ('>:-?\)', '(:-?\)|\(-?:)', '(:-?\(|\)-?:)', ':-?D', ':-?[Oo]',
	     ';-?[\)D]', ':-?P ' );

# This defines the actual emoticons to substitute.
%emoticon = (
	     '(:-?\)|\(-?:)' => "emoticon_smile.png",
	     '(:-?\(|\)-?:)' => "emoticon_unhappy.png",
	     ':-?D' => "emoticon_grin.png",
	     ':-?[Oo]' => "emoticon_surprised.png",
	     ';-?[\)D]' => "emoticon_wink.png",
	     ':-?P ' => "emoticon_tongue.png",
	     '>:-?\)' => "emoticon_evilgrin.png",
	     );

=pod

=head1 Publishing


=cut

sub publish {
    my $this = shift;
    my $db = $share{DB};
    my ($out,@zines);
    if ($this->{input}{aid}) {
	$this->fetch($this->{input}{aid});
	if ($this->{article}{owner} == $db->my_uid) {
	    if ($this->{article}{type} !~ /article|comment/) {
		return $this->publish_zine($this->{article});
	    }
	    else {
		return $this->publish_article($this->{article});
	    }
	}
    }
    else {
	$out = "<h2>Publishing Zines</h2>\n";
	if ($db->is_admin) {
	    @zines = $db->fetch_match("article",{status=>"active",
						 reference=>0,
						 owner=>0});
	    foreach my $z (@zines) { $out .= $this->publish_zine($z); }
	}
	@zines = $db->fetch_match("article",{status=>"active",
					     reference=>0,
					     owner=>$db->my_uid});
	foreach my $z (@zines) { $out .= $this->publish_zine($z); }
    }
    return $out;
}

sub publish_zine {
    my ($this,$zine) = @_;
    if ($zine->{privacy} =~ /members/) {
	return "<p>$zine->{title} is private, and cannot be published.</p>\n";
    }
    my $db = $share{DB};
    my $out = "<h4>Publishing $zine->{title}</h4>\n";
    $out .= $this->publish_article($zine);
    my @articles = $db->fetch_match("article",{status=>"active",
					    reference=>$zine->{article_id}} );
    foreach my $art (@articles) { 
	$out .= $this->publish_article($art);
    }
    $out .= $this->publish_rss($zine);
    return $out;
}

sub publish_rss {
    my ($this,$zine) = @_;
    return undef if ($zine->{privacy} =~ /members/);
    my $db = $share{DB};

    # can only publish RSS feeds if there is a service page to direct
    # feed links to.

    return if (! $zine->{section_id});
    my $p = &ExSite::Module::service_page($this->module_name,
					  $zine->{section_id});
    if (! $p) {
	return "Skipping RSS publishing, because there is no service page.<br>\n";
    }

    # re-jig the environment to get the right RSS links
    my $script_name = $ENV{SCRIPT_NAME};
    my $query_string = $ENV{QUERY_STRING};
    $ENV{SCRIPT_NAME} = "$config{server}{CGIpath}/$config{prog}{page}";
    $ENV{QUERY_STRING} = "_id=".$p->id();

    my $out = "Publishing RSS feed for $zine->{title}...<br>\n";
    $this->set($zine);
    my $rss = $this->rss;

    # restore environment
    $ENV{SCRIPT_NAME} = $script_name;
    $ENV{QUERY_STRING} = $query_string;

    my $file = $this->diskpath."/index.rss";
    my $mkdir = $this->mkdir($this->diskpath);
    if (! $mkdir) {
	return $this->show_diagnostics("error","html");
    }
    $out .= $mkdir;
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	$out .= "Writing $secure_filename... ";
	if (open OUTFILE, ">$secure_filename") {
	    print OUTFILE $rss;
	    close OUTFILE;
	    $out .= "succeeded.<br>\n";
	}
	else {
	    $this->error("Zine::publish: publish $secure_filename failed: $!");
	    $out .= "<SPAN CLASS=error>FAILED: $!</SPAN><br>\n";
	}
    }
    return $out;
}

sub publish_article {
    my ($this,$art) = @_;
    return if ($art->{type} eq "comment");
    my $out;
    my $db = $share{DB};
    if ($art->{picture}) {
	$this->set($art);
	my $zpath = $this->diskpath;
	my $mkdir = $this->mkdir($zpath);
	if (! $mkdir) {
	    return $this->show_diagnostics("error","html");
	}
	$out .= $mkdir;
	my $img = new ExSite::Image($art->{picture});
	my $imgfile = &clean_filename($img->{filename});
	my $picpath = "$zpath/$art->{article_id}_$imgfile";
	$out .= $this->publish_file("picture",$picpath,$img);
	if ($art->{thumb}) {
	    $img->setup($art->{thumb});
	    $picpath = "$zpath/$art->{article_id}T_$imgfile";
	    $out .= $this->publish_file("picture",$picpath,$img);
	}
    }
    my @articles = $db->fetch_match("article",{status=>"active",
					       parent_id=>$art->{article_id}} );
    foreach my $article (@articles) { 
	$out .= $this->publish_article($article);
    }
    return $out;
}

sub publish_file {
    my ($this,$col,$file,$img) = @_;
    my $out;
    #return "<p>Zine::publish: permission denied</p>\n" if (! $this->{admin});
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    $out .= "Writing $secure_filename... ";
    if (open OUTFILE, ">$secure_filename") {
	print OUTFILE $img->get;
	close OUTFILE;
	$out .= "succeeded.<br>\n";
    }
    else {
	$this->error("Zine::publish: publish $secure_filename failed: $!");
	$out .= "<SPAN CLASS=error>FAILED: $!</SPAN><br>\n";
    }
    return $out;
}

sub mkdir {
    my ($this,$dir) = @_;
    if (! $this->{dircache}) { $this->{dircache} = {}; }
    return $this->{dircache}{$dir} if (exists $this->{dircache}{$dir});
    my $out = " ";  # dummy return value
    $dir =~ /(\/[\w-\/]+)/;
    my $secure_dir = $1;
    if (! $secure_dir) { 
	$this->error("mkdir: invalid directory: $dir");
	return undef;
    }
    my @path = split(/\//,$secure_dir);
    shift @path; # first one is a dummy
    my $path;
    foreach my $dir (@path) {
	$path .= "/$dir";
	next if (-e "$path");
	$out .= "<P>Creating Zine directory $dir... ";
	if (! mkdir "$path") {
	    $this->error("mkdir $dir: failed: $!");
	    #$this->error("mkdir $path: failed: $!");
	    $this->{dircache}{$dir} = undef;
	    return undef;
	}
	else {
	    $this->info("created site directory $dir");
	    $out .= "succeeded.</P>\n";
	    # add blank index file, to prevent snooping
	    my $file = "$secure_dir/index.html";
	    if (open OUTFILE, ">$file") {
		print OUTFILE "<html></html>\n";
		close OUTFILE;
	    }
	    $this->{dircache}{$dir} = " ";
	}
    }
    return $out;
}

# publish methods, above, require special priveleges that only the publish
# program possesses.  If we are not in the publish program, we have to call
# these methods indirectly as follows.

sub publish_unprivileged {
    my ($this,$article) = @_;
    my $out;
    if ($article->{picture}) {
	my $query_save = $ENV{QUERY_STRING};
	my $mod = $this->module_name;
	&run_publisher("module",$mod,0,(aid=>$article->{article_id}));
#	$ENV{QUERY_STRING} = "module=$mod&aid=$article->{article_id}";
#	$out = `$config{server}{CGIroot}/publish.cgi`;
#	$this->info($out);
#	$ENV{QUERY_STRING} = $query_save;
    }
    return $out;
}

# diskpath to article's published files

sub diskpath {
    my $this = shift;
    return $config{server}{HTMLroot}.$this->htmlpath;
}

sub htmlpath {
    my $this = shift;
    my $zpath = "$config{server}{HTMLpath}/_Modules/Zine";
    if ($this->{article}{reference}) {
	$zpath .= "/$this->{article}{reference}";
    }
    else {
	$zpath .= "/$this->{article}{article_id}";
    }
    return $zpath;
}

# article path is /zine/article/comment

=pod
sub article_path {
    my $this = shift;
    $this->save;
    my @path = ( $this->name );
    while ($this->{article}{parent_id}) {
	$this->fetch($this->{article}{parent_id});
	unshift @path, $this->name;
    }
    # FIXME...
}
=cut

=pod

=head1 Queries & Finding Articles


=cut

sub fetch ($$) {
    my ($this,$id) = @_;
    if ($id =~ /^\d+$/) {
	if ($id != $this->{article_id}) {
	    $this->{article_id} = $id;
	    $this->{article} = $share{DB}->fetch("article",$id);
	    if ($this->{article}{reference}) {
		$this->{reference} =
		    $share{DB}->fetch("article",$this->{article}{reference});
	    }
	    else {
		$this->{reference} = undef;
	    }
	}
    }
    elsif ($id) {
	# non-numeric article ID; try matching to "name"
	if ($id != $this->{article}{name}) {
	    my $ref_id = $article_class{$this->type} eq "zine" ?
		$this->{article_id} : $this->{reference}{article_id};
	    my @art = $share{DB}->fetch_match("article",{name=>$id,
							 reference=>$ref_id,
						     },"article_id");
	    if (@art > 0) {
		$this->{article} = $art[-1];
		if ($ref_id != $this->{reference}{article_id}) {
		    $this->{reference} =
			$share{DB}->fetch("article",$ref_id);
		}
	    }
	    else {
		$this->{article} = undef;
		$this->{reference} = undef;
	    }
	}
    }
    return $this->{article};
}

sub find {
    my ($this,$path) = @_;
    my @path = split /\//,$path;
    my ($article,$ref);
    $article = { article_id => 0 };
    while (@path) {
	my $name = shift @path;
	next if ($name =~ /^\s*$/);
	if ($name !~ /^[\w-\.]+$/) {
	    $this->error("'$name' is an invalid article name.");
	    last;
	}
	    
	$ref = $article;

	if ($ref->{article_id}) {
	    # if we have a reference article to constrain the query,
	    # things are simple:
	    $article = $share{DB}->fetch_match(name=>$name,reference=>$ref->{article_id})->[0];
	}
	else {
	    # no reference article, so we must constrain the query by the 
	    # sites we have access to.
	    my @sites = $share{DB}->my_real_sites;
	    my @sids;
	    foreach my $s (@sites) { push @sids,$s->{section_id}; }
	    push @sids,0;
	    my $query = "select * from article where name='$name' and parent_id=0 and section_id in ".join(",",@sids);
	    $article = $share{DB}->custom_query($query)->[0];
	}
	if (! $article->{article_id} ) {
	    $this->error("Article '$name' not found.");
	    last;
	}
    }
    if (! $article->{article_id} ) {
	$article = $ref;
	$ref = undef;
    }
    $this->{article} = $article;
    if ($this->{article}{reference} = $this->{parent_id}) {
	$this->{reference} = $ref;
	return $article;
    }
    else {
	return $this->set($article);
    }
}

sub set ($$) {
    my ($this,$art) = @_;
    $this->{article_id} = $art->{article_id};
    $this->{article} = $art;
    if ($this->{article}{reference}) {
	$this->{reference} =
	    $share{DB}->fetch("article",$this->{article}{reference});
    }
    else {
	$this->{reference} = undef;
    }
    return $art;
}

# save/restore - for temporary changing of article context

sub save {
    my $this = shift;
    $this->{article_saved} = $this->{article};
    $this->{article_id_saved} = $this->{article_id};
    $this->{reference_saved} = $this->{reference};
    return;
}

sub restore {
    my $this = shift;
    $this->{article} = $this->{article_saved};
    $this->{article_id} = $this->{article_id_saved};
    $this->{reference} = $this->{reference_saved};
    return;
}

sub fetch_tree {
    my ($this,$id,$match) = @_;
    if (!$id) { $id = $this->{article_id}; }
    if (ref $match ne "HASH") { $match = {}; }
    if ($id && ! $match->{reference}) { $match->{reference} = $id; }
    if (! $this->{admin} && ! $match->{status}) { $match->{status} = "active"; }
    my @art = $share{DB}->fetch_match("article",$match,"ctime");
    # build a tree from the articles
    my $tree = new ExSite::Tree;
    $tree->addnodes(\@art,"article_id","parent_id",[$id]);
    return $tree;
}

sub fetch_child_articles {
    my ($this,$mode,$limit,$sort,$opt) = @_;
    return undef if (! $this->{article_id});
    $mode = $mode || "article";
#   $opt |= {};
    my $match = { 
	parent_id => $this->{article_id},
	status => "active",
	type => $mode,
#	%$opt,
    };

    $sort = $sort || ["sortkey",{"ctime"=>"desc"}];

    return $share{DB}->fetch_match("article",$match,$sort,$limit);
}

sub fetch_latest {
    my ($this,$type,$id,$number) = @_;
    if (! $type) { $type = "article"; }
    # set $number if it was not passed in  
    if (! defined $number ) {
	$number |= $config{$this->module_name}{"new_".$type."s"} || 4;
    }
    my @aid;
    if ($id) {
	@aid = ($id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
    return undef if (scalar @aid == 0);
    return $share{DB}->custom_query("select * from article where reference in (".join(",",@aid).") and type=\"$type\" and status=\"active\" order by article_id desc limit $number");
}

# return array of zines owned by the current user

sub my_zines {
    my ($this) = @_;
    my $uid = $share{DB}->my_uid;
    return () if (! $uid);
    my %match = (status=>"active", owner=>$uid, reference=>0);
    return $share{DB}->fetch_match("article",\%match);
}

# return array of zines avialable to the current site

sub our_zines {
    my ($this,$sid) = @_;
    if (! $sid) { $sid = $share{Page} ? $share{Page}->id("section") : 0; }
    return () if (! $sid);
    
    my %match = (status=>"active", section_id=>0, reference=>0);
    my @art;
    # fetch global zines
    @art = $share{DB}->fetch_match("article",\%match);
    # add local Zines
    $match{section_id} = $sid;
    push @art, $share{DB}->fetch_match("article",\%match);
    return wantarray ? @art : \@art;
}

# sort an array of articles

sub sort {
    my ($this,$sortrule,@article) = @_;
    if ($sortrule eq "sorted") {
	return sort { $a->{sortkey} cmp $b->{sortkey} } @article;
    }
    elsif ($sortrule eq "reverse-date" || $sortrule eq "reverse-flip-flop") {
	return sort { $b->{ctime} cmp $a->{ctime} } @article;
    }
    # elsif ($sortrule eq "date" || $sortrule eq "flip-flop")
    return sort { $a->{ctime} cmp $b->{ctime} } @article;
}

# count descendant articles of a certain type, starting
# from an arbitrary article id.

sub count {
    my ($this,$id,$class,$tree) = @_;
    my $count = 0;
    if (! $id) { $id = $this->{article_id}; }
    if (! $class) { $class = "comment"; }
    ### FIXME: tree method only works if we're counting articles of the 
    ### same type as the parent article.  If we want to count comments
    ### under an article, however, the comments are not in our tree.
    if (ref $tree && $class eq $this->{article}{type}) {
	my @article = $tree->collapse($id);
	foreach my $a (@article) {
	    if ($a->{status} eq "active" &&           # count active articles
		$article_class{$a->{type}} eq $class) # count matching type
	    { $count++; }
	}
    }

    # WARNING: technically, the above tree algorithm and the following 
    # DB query algorithms do not do the same thing.  The tree algorithm
    # searches the full depth of the tree, whereas the DB algorithms only
    # search the immediate children for articles and attachments.  The 
    # latter is correct behaviour, but the the former will give the same
    # results except in some strange zine configurations that are not expected.

    elsif ($class eq "article") {
	$count = $share{DB}->count("article",{
	    parent=>$id,
	    status=>"active",
	    type=>"article",
	});
    }
    elsif ($class eq "comment") {
	$count = $share{DB}->count("article",{
	    reference=>$id,
	    status=>"active",
	    type=>"comment",
	});
    }
    elsif ($class eq "attachment") {
	$count = $share{DB}->count("article",{
	    parent=>$id,
	    status=>"active",
	    type=>"attachment",
	});
    }
    return $count;
}

=pod

=head1 Displaying Articles

=cut

sub css {
    my $this = shift;
    return "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Zine/zine.css\">\n";
}

sub wrap {
    my ($this,$class,$contents) = @_;
    return $contents ? "<div class=\"$class\">\n$contents</div>\n" : undef;
}

sub show {
    my ($this,%opt) = @_;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, this $type is not available for viewing.");
    }
    my $stat = $this->handler("ZineShow");
    return $stat if ($stat);
    my $out = $this->show_path(%opt);
    $out .= $this->show_article(%opt);
    $opt{mode} = "article";
    my $aindex = $this->index(%opt);
    $opt{mode} = "comment";
    my $cindex = $this->index(%opt);
    if ($aindex) {
	if ($cindex) { $out .= "<h3>Articles</h3>\n"; }
	#$out .= $this->wrap("ZineIndex",$aindex);
	$out .= $aindex;
    }
    if ($cindex) {
	if ($aindex) { $out .= "<h3>Comments</h3>\n"; }
	#$out .= $this->wrap("ZineIndex",$cindex);
	$out .= $cindex;
    }
    return $out;
}

# display a complete article

sub show_article {
    my $this = shift;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, $type $this->{article_id} is not available for viewing.");
    }
    my $stat = $this->handler("ZineShowArticle");
    return $stat if ($stat);
    # build the article structure up from component elements
    #
    ### should we conditionally float the picture, depending on its size?
    ### Or, depending on whether there is any body to float around?
    #
    my $picture = $this->picture();
    my $attachment = $this->wrap("ZineAttachment",$this->attachment);
    my $body = $this->wrap("ZineBody",$picture.$this->body.$attachment);
    my $about = $this->wrap("ZineAbout",$this->about);
    my $about_body = $this->wrap("ZineAboutBody",$about.$body);
    my $head = $this->header;
    my $foot = $this->footer;  # includes tools
    if ($this->{article}{type} eq "comment") {
	return $this->wrap("ZineComment",$head.$about_body.$foot)."\n";
    }
    else {
	return $this->wrap("ZineArticle",$head.$about_body.$foot)."\n";
    }
}

sub show_link {
    my ($this,$anchor) = @_;
    my $url = $this->link(zine=>"show",aid=>$this->{article_id});
    my $label = $anchor ? $anchor : $this->title;
    return "<a href=\"$url\" class=\"ZineLink\">$label</a>";
}

sub show_info {
    my $this = shift;
    my $stat = $this->handler("ZineShowInfo");
    return $stat if ($stat);
    my $link = $this->show_link;
    my $author = $this->author;
    my $date = $this->date;
    return $author ? 
	"$link by $author;  $date" :
	"$link;  $date";
}

sub show_teaser {
    my $this = shift;
    my $stat = $this->handler("ZineShowTeaser");
    return $stat if ($stat);
    my $author = $this->author;
    my $title = $this->show_link;
    my $out = "$title";
    if ($author) { $out .= " by $author<br>\n"; }
    $out .= $this->wrap("ZineAbstract",$this->abstract);
    my $comment_count = $share{DB}->count("article",{
	reference=>$this->{article_id},
	status=>"active",
	type=>"comment",
    });
    my $url = $this->link(zine=>"show",aid=>$this->{article_id});
    $out .= "&rarr; <a href=\"$url\">Read more...</a> ";
    if ($comment_count) {
	$out .= "($comment_count comments)";
    }
    $out .= "\n";
    return $out;
}

sub show_row {
    my ($this,$data,@col) = @_;
    my $stat = $this->handler("ZineShowRow",$data,@col);
    return $stat if ($stat);
    if (! defined $data) { $data = 1; }
    if (@col == 0) { @col = qw(title author ctime); }
    my ($celltype,@colout);
    if ($data) {
	foreach my $c (@col) {
	    push @colout, $c eq "title" ? 
		$this->show_link() :
		$this->show_data($c);
	}
	$celltype = "td";
    }
    else {
	foreach my $c (@col) {
	    my %cmap = $share{DB}->{map}->get_column("article",$c);
	    push @colout, $this->label(\%cmap);
	}
	$celltype = "th";
    }
    return "<$celltype>".join("</$celltype>\n<$celltype>",@colout)."</$celltype>\n";
}

# show an article in full, with a summary index

sub show_full {
    my ($this,$tree,$index) = @_;
    my $stat = $this->handler("ZineShowFull");
    return $stat if ($stat);
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, you don't have permission to view this $type.");
    }
    my $out = $this->show_article;
    return $out if (! $index);
    my $article_count = $this->count($this->{article_id},"article",$tree);
    my $comment_count = $this->count($this->{article_id},"comment",$tree);
    if ($article_count || $comment_count) {
	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
	$out .= "(";
	if ($article_count) {
	    $out .= "$article_count more articles; ";
	}
	if ($comment_count) {
	    my $comment_type;
	    if ($comment_count > 1) { 
		$comment_type = ($this->class() eq "article") ? 
		    "comments" : "replies";
	    }
	    else {
		$comment_type = ($this->class() eq "article") ? 
		    "comment" : "reply";
	    }
	    $out .= "$comment_count $comment_type; ";
	}
	$out .= "<a href=\"$url\">Read more...</a>)";
    }
    return $out;
}

# takes the aid of a parent article and displays a random child article
sub index_random {
    my ($this,$id) = @_;
    my ($out,@aid);
    if ($id) {
	@aid = split(/,/, $id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
	my $sql = "select * from article where reference in (".join(",",@aid).") and type=\"article\" and status=\"active\" order by article_id desc";
    my @latest = $share{DB}->custom_query($sql);
    if (@latest > 0) {
	my $total_articles = scalar @latest;
	my $random_number = int(rand($total_articles));
	my $a = $latest[$random_number];
    $this->fetch($a->{article_id});
	$out = $this->show;
    }
    return $out;
}
=pod

=head2 Displaying Article Components


=cut

sub header {
    my $this = shift;
    my $stat = $this->handler("ZineHeader");
    return $stat if ($stat);
    my $title = $this->show_data("title");
    my $class = $this->class;
    my $n = $config{$this->module_name}{heading_level}{$class};
    if ($n =~ /^[123456]$/) {
	$title = "<h$n>$title</h$n>";
    }
    my $subtitle = $this->show_data("subtitle");
    if ($subtitle) { $subtitle = $this->wrap("ZineSubtitle","<em>$subtitle</em>"); }
    return $this->wrap("ZineHeader",$this->wrap("ZineTitle",$title).$subtitle);
}

sub about {
    my $this = shift;
    my $stat = $this->handler("ZineAbout");
    return $stat if ($stat);

    my ($show_auth, $about, $date, $summary);
    if ($this->type eq "comment") {
	$show_auth = 1;
    }
    elsif ($this->get("reference")) {
	# this article belongs to a zine
	$show_auth = $show_author{$this->reftype}[1];
    }
    else {
	# this article IS a zine
	$show_auth = $show_author{$this->type}[0];
    }
    if ($show_auth) {
	my $author = $this->author;
	if ($author) { $about = "by ".$this->author.", "; }
    }
    if ($this->class =~ /article|comment/) {
	$date = $this->date;
	$about .= $date;
    }
    $summary = $this->show_data("summary");
    if ($summary) {
	$about .= $this->wrap("ZineSummary",$summary);
    }

    # show status prominently in adminstrator views
    if ($this->{admin}) {
	if ($this->{article}{status} ne "active") {
	    my $status = ucfirst $this->{article}{status};
	    $about .= "<br><span class=\"Zine$status\">$this->{article}{status}</span>";
	}
    }

    #return $this->wrap("ZineAbout",$about);
    return $about; #$this->wrap("ZineAbout",$about);
}

sub footer {
    my $this = shift;
    my $stat = $this->handler("ZineFooter");
    return $stat if ($stat);
    my $footnotes = $this->show_data("about");
    my $tools = $this->tools;
    # disable conditional wrapping - we need the wrapper in some cases,
    # even if there is no content.
    #return $this->wrap("ZineFooter",$footnotes.$tools);
    return "<div class=\"ZineFooter\">$footnotes$tools</div>";
}

# should we show photos for some article types??
sub abstract {
    my $this = shift;
    my $stat = $this->handler("ZineAbstract");
    return $stat if ($stat);
    if ($this->{article}{summary}) {
	return $this->show_data("summary");
    }
    my $out = $this->body;
    # remove HTML 
    $out =~ s/<.+?>//g;
    my $teaserlen = $config{$this->module_name}{teaser_length} || 200;
    if (length($out) > $teaserlen) {
	my $pos = $teaserlen; 
	my $char;
	while (($pos < length($out)) && ($char = substr($out,$pos,1))) {
	    last if ($char =~ /\s/);
	    $pos++;
	}
	substr($out,$pos) = "...";
    }
    return $out;
}

sub body {
    my $this = shift;
    my $stat = $this->handler("ZineBody");
    return $stat if ($stat);
    my $body = $this->get("body");

    # run wiki articles through the wiki filter
    if ($this->reftype eq "wiki") {
	# use this if linking method is same as CMS [[...]]
	#$body =~ s/\[\[(.+?)\]\]/$this->wikilink($1)/egs;
	# use this for single-bracket link method [...]
	$body =~ s/\[(.+?)\]/$this->wikilink($1)/egs;
    }

    # kill/neuter any hidden CMS tags
    if (! $config{$this->module_name}{allow_cms_tags}) {
	$body =~ s/<!--/<!-- /g;
    }
    if (! $config{$this->module_name}{allow_cms_links}) {
	$body =~ s/\{\{(?![^<>]*>)/\{&\#00;\{/g;
	$body =~ s/\}\}(?![^<>]*>)/\}&\#00;\}/g;
	$body =~ s/\[\[(?![^<>]*>)/\[&\#00;\[/g;
	$body =~ s/\]\](?![^<>]*>)/\]&\#00;\]/g;
    }

    # process linebreaks
    if ($this->{article}{type} eq "comment") {
	$body =~ s/\n/<br>\n/g;       # convert NL to <br>
    }
    elsif ($this->{article}{content_type} !~ /html/i) {
	# insert paragraph tags at any double-linebreaks
	$body =~ s/\r\n\r\n/<P>\n/g;  # handles CR-LF linebreaks
	$body =~ s/\n\n/<P>\n/g;      # handles LF linebreaks
    }

    # special body mods for comments:
    if ($this->{article}{type} eq "comment") {

	# try to impose some style rules on the quoted sections, in case
	# the designer wants to highlight them

	my @text = split /\n/,$body;
	my $quote = 0;
	foreach my $line (@text) {
	    if ($line =~ /^(>|&gt;)/) {
		if ($config{$this->module_name}{remove_quote_char}) {
		    $line =~ s/^(>|&gt;)//;
		}
		if (! $quote) {
		    $line = "<div class=\"ZineQuote\">".$line;
		    $quote = 1;
		}
	    }
	    elsif ($quote) {
		$line = "</div>".$line;
		$quote = 0;
	    }
	}
	$body = join("\n",@text)."\n";

	# emoticons

	if ($config{$this->module_name}{emoticons}) {
	    foreach my $key (@emoticon) {
		$body =~ s/$key/<img src="$config{server}{HTMLpath}\/_Modules\/Zine\/$emoticon{$key}">/g;
	    }
	}

	if ($config{$this->module_name}{body_strut_size} || $this->{admin}) {
	    # This is a kludge for pseudo-table layouts, where the body
	    # column may end up being shorter than the about column.  You
	    # can add a body strut to force a minimum body height.  
	    # Ironically, in order to make the wrapper DIVs look like they
	    # are really a table, we have to use a real table.  
	    # Gotta love CSS...
	    my $strut_height = $config{$this->module_name}{body_strut_size} || 75;
	    $body = "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\"><tr>
<td valign=\"top\">$body</td>
<td><img src=\"$config{server}{HTMLpath}/_ExSite/images/null.gif\" width=\"1\" height=\"$strut_height\" align=\"right\" border=\"0\"></td>
</tr></table>\n";
	}

    }

    # attachments



    return $body;
}

sub size {
    my $this = shift;
    return length $this->{article}{body};
}

=pod

=head1 Attributes


=cut

# returns the article type;

sub type {
    my $this = shift;
    return $this->{article}{type} || "article";
}

# returns the reference article type;  determines general formatting rules

sub reftype {
    my $this = shift;
    return $this->{reference}{article_id} ?
	($this->{reference}{type} || "article") :
	($this->{article}{type} || "article");
}

# returns the zine type;  determines general formatting rules

sub zinetype {
    my $this = shift;
    return $this->{article}{type} if ($article_class{$this->{article}{type}} eq "zine");
    return $this->{reference}{type} if ($article_class{$this->{reference}{type}} eq "zine");
    my $a = $this->{reference};
    while ($a->{reference}) {
	$a = $share{DB}->fetch("article",$a->{reference});
	return $a->{type} if ($article_class{$a->{type}} eq "zine");
    }
    return undef;
}

# returns the article type class.  Either "comment" (ie. external source) 
# or "article" (internal source).
### FIXME: are wiki articles another class?
### FIXME: add attributes

sub class {
    my $this = shift;
    return $article_class{$this->type} || "article";
}

sub refclass {
    my $this = shift;
    my $type = $this->reftype();
    # articles belonging to wikis have special handling
    return $type eq "wiki" ? "wiki_article" :
	($article_class{$this->reftype} || "article");
}

sub get { 
    my ($this,$attribute,$class) = @_;

    # look in the current article
    
    if ($this->{article}{$attribute}) {
	return $this->{article}{$attribute};
    }

    # For article indexes, look through the reference articles for 
    # inheritable attributes.  We don't do this for comments, because
    # comment indexes are generally quite different from article indexes.

    if ($class eq "article" && $this->reftype !~ /forum/ &&
	$attribute =~ /comment_policy|index(age|limit|type|format)/) {
	my $val = $this->getref($attribute);
	return $val if ($val);
    }

    # for forums, we do the opposite, since forum attributes are
    # assumed to apply primarily to comments, not the articles.

    if ($class eq "comment" && $this->reftype =~ /forum/ &&
	$attribute =~ /comment_policy|index(age|limit|type|format)/) {
	my $val = $this->getref($attribute);
	return $val if ($val);
    }

    # check the configuration file for default attributes

    my $zinetype = $this->zinetype;
    if (! $class) { $class = $this->class; }
    # zine-specific configurations
    if ($config{$this->module_name}{$zinetype}{$class}{$attribute}) {
	return $config{$this->module_name}{$zinetype}{$class}{$attribute};
    }
#    my $reftype = $this->reftype;
#    if (! $class) { $class = $this->class; }
#    # zine-specific configurations
#    if ($config{$this->module_name}{$reftype}{$class}{$attribute}) {
#	return $config{$this->module_name}{$reftype}{$class}{$attribute};
#    }
    # general configurations
    if ($config{$this->module_name}{$class}{$attribute}) {
	return $config{$this->module_name}{$class}{$attribute};
    }

    return undef;
}

sub getref {
    my $this = shift;
    $_ = shift;
    if ($this->{article} && ! $this->{reference}) {
	$this->{reference} =
	    $share{DB}->fetch("article",$this->{article}{reference});
    }
    return $this->{reference}{$_} ? $this->{reference}{$_} : undef;
}

sub show_data {
    my $this = shift;
    my $col = shift;
    my $data = $this->{article}{$col};

    # kill/neuter any hidden CMS tags
    if (! $config{$this->module_name}{allow_cms_tags}) {
	$data =~ s/<!--/<!-- /g;
    }
    if (! $config{$this->module_name}{allow_cms_links}) {
	$data =~ s/\{\{/\{&\#00;\{/g;
	$data =~ s/\}\}/\}&\#00;\}/g;
	$data =~ s/\[\[/\[&\#00;\[/g;
	$data =~ s/\]\]/\]&\#00;\]/g;
    }
    return $share{DB}->show_data_noauth("article",$col,$data,
					$this->{article_id});
}

sub label {
    my ($this,$col,$type) = @_;
    if (! $type) { $type = $this->type; }
    return ($type eq "comment") ?
	$comment_label{$col->{column}} || $col->{label} || $col->{column} :
	$col->{label} || $col->{column};
}

sub title {
    my $this = shift;
    return $this->get("title");
}

sub name {
    my $this = shift;
    my $name = $this->get("name") || $this->{article_id};
    return $name;
}

sub reftitle {
    my $this = shift;
    return $this->getref("title");
}

sub author {
    my $this = shift;
    my $stat = $this->handler("ZineAuthor");
    return $stat if ($stat);
    my $author = $this->get("author");
    if ($author) {
	if ($this->{article}{type} eq "comment" && 
	    $config{$this->module_name}{mark_unauthenticated_comments} && 
	    ! $this->{article}{owner}) {
	    my $mark = $config{$this->module_name}{unauthenticated_comment_mark};
	    return "$author $mark";
	}
	else {
	    return $author;
	}
    }
    elsif ($this->type eq "comment") {
	if ($this->{article}{owner}) {
	    my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	    return $owner{name} || $owner{login};
	}
	else {
	    return $config{$this->module_name}{anonymous_name} || "Anonymous";
	}
    }
    return "";
}

sub author_email {
    my $this = shift;
    if ($this->{article}{owner}) {
	my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	return $owner{email};
    }
    return undef;
}

sub author_login {
    my $this = shift;
    if ($this->{article}{owner}) {
	my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	return $owner{$config{auth}{user_login_column}};
    }
    return undef;
}

# basic timestamp formatting

sub timestamp {
    my ($this,$dtype) = @_;
    if (! $dtype) { $dtype = "ctime"; }  # alternative is mtime
    return $this->show_data($dtype);
}

# article date

sub date {
    my ($this) = @_;
    my ($timestamp, $timetype, $label);
    if ($config{Zine}{datetype} eq "simple") {
	$label = "";
	$timetype = "ctime";
    }
    elsif ($config{Zine}{datetype} eq "creation") {
	$label = "posted on ";
	$timetype = "ctime";
    }
    else { #if ($config{Zine}{datetype} eq "update") {
	$timetype = ($this->{article}{mtime} ne $this->{article}{ctime}) ? 
	    "mtime" : "ctime";
	$label = $timetype eq "mtime" ? "updated on " : "posted on ";
    }
    $timestamp = $this->timestamp($timetype);
    return $label.$timestamp;
}

# return age of article in days (approximate - does not account for leap years)

sub age {
    my $this = shift;
    my $time = new ExSite::Time; # (now)
    # time diff is in seconds; convert to days
    my $age = ($time->diff($this->{article}{ctime},"sql_timestamp")) / 86400;
    return - int($age);
}

# indexlimit: the maximum number of items that will be displayed in an index
# if indexlimit == 0, the index size is unlimited
# if indexlimit is undefined, it defaults to some number

sub indexlimit {
    my ($this,$class) = @_;
    # custom article setting
    my $limit = $this->get("indexlimit",$class);
    return $limit if ($limit);
    if (! $class) { $class = $this->class; }
    return 10 if ($class eq "article");
    return 25; # comments
}

# indexage: the maximum age of items that will be displayed in an index
# if indexage == 0, the index age is unlimited
# if indexage is undefined, it defaults to some number

sub indexage {
    my ($this,$class) = @_;
    # custom article setting
    my $age = $this->get("indexage",$class);
    return $age if ($age);
    if (! $class) { $class = $this->class; }
    return 14 if ($class eq "article");
    return 30; # comments
}

# indextype: the rules for laying out the index
#
# basic - each article is wrapped in a div
# deep - like basic, but the full depth of the tree is shown
# list - each article is a bullet item in a list
# numbered - each article is a numbered item in a list
# table - each article is a row in a table
# gallery - each article is a cell in a table
# threaded - each article is nested under its parent
# semithreaded - table for top-level; deep for sublevels  ###TODO
# paged - next/previous links to page through the articles
# wiki - use special wiki rules

sub indextype {
    my ($this,$class) = @_;
    # custom article setting
    my $type;
    if (! $class) { $class = $this->class; }
#    if ($class eq "article" || ($this->{article}{type} eq "forum")) {
	$type = $this->get("indextype",$class);
#    }
    return $type if ($type);
    return "threaded" if ($class eq "comment");
    my $reftype= $this->reftype;
    return "numbered" if ($reftype eq "book");
    return "wiki" if ($reftype eq "wiki");
    return "basic";
}

# indexformat: the rules for formatting each article in the index
#
# full - show the whole article
# summary - show an article summary/abstract
# basic - show linked title + author + date
# title - show linked title

sub indexformat {
    my ($this,$class) = @_;
    # custom article setting
    my $fmt = $this->get("indexformat",$class);
    return $fmt if ($fmt);
    if (! $class) { $class = $this->class; }
    return "full" if ($class eq "comment");
    my $reftype = $this->reftype;
    return "summary" if ($reftype eq "e-zine");
    return "photoindex" if ($reftype eq "photoblog" && $class eq "article");
    return "full" if ($reftype eq "blog");
    return "summary" if ($class eq "article");
#    return "sorted" if ($class eq "article");
    return "full"; # comments
}

# indexsort: the rules for sorting the index
#
# sorted - by sortkey
# date - by date, most recent last
# reverse-date - by date, most recent first
# flip-flop - by date on top-level; by reverse-date on sub-levels
# reverse-flip-flop - reverse-date on top-level; by date on sub-levels

sub indexsort {
    my ($this,$class) = @_;
    # custom article setting
    my $fmt = $this->get("indexsort",$class);
    return $fmt if ($fmt);
    if (! $class) { $class = $this->class; }
    my $type = $this->type;
    #return "reverse-flip-flop" if ($class eq "comment");
    #return "reverse-date" if ($type =~ /blog|e-zine/);
    return "date";
}

# comment_policy: whether or not comments are accepted

sub comment_policy {
    my ($this,$class) = @_;
    # custom article setting
    my $com = $this->get("comment_policy",$class);
    return $com if ($com);
    # configuration setting
    my $ref = $this->reftype;
    ### FIXME: should test for site membership, not just access level
    my $access = $share{DB}->authorize;
    if ($access) {
	$com = $config{$this->module_name}{$ref}{comment_policy}{member};
    }
    else {
	$com = $config{$this->module_name}{$ref}{comment_policy}{public};
    }
    return $com if ($com);
    # comments are forbidden by default, unless this is a forum or comment
    if ($access && $this->{article}{type} =~ /forum|comment/) {
	return "allowed";
    }
    return "forbidden";  
}

sub attachment {
    my ($this,@att) = @_;
    my $stat = $this->handler("ZineAttachment",@att);
    return $stat if ($stat);
    if (@att == 0) {
	@att = $share{DB}->fetch_match("article",{
	    parent_id=>$this->get("article_id"),
	    type=>"attachment",
	    status=>"active",
	});
	return "" if (@att == 0);
    }
    my @attout;
    foreach my $attachment (@att) {
	my $img = new ExSite::Image($attachment->{picture});
	my $name = $img->name;
	my $type = &MimeType($name);
	my $att;
	if ($type =~ /^image/) {
	    my $url = $this->link(zine=>"show",aid=>$attachment->{article_id});
	    $att = "<a href=\"$url\">".$this->thumb($attachment)."</a>";
	}
	else {
	    my $url = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
	    $att = "<a href=\"$url\">$name</a>";
	}
	push @attout, "<table><tr><td>$att</td></tr><tr><td><div class=\"ZineCaption\">$attachment->{caption}</div></td></tr></table>";
    }
    return join("<br>\n",@attout);
}

sub picture {
    my ($this,$img) = @_;
    my $stat = $this->handler("ZinePicture",$img);
    return $stat if ($stat);
    my $out;
    if (! $img) {
	return undef if (! $this->{article}{picture});
	$img = new ExSite::Image($this->{article}{picture});
    }
    my ($width,$height) = $img->dim;
    my $filename = "$this->{article_id}_".&clean_filename($img->{filename});
    if (! -e $this->diskpath."/".$filename) {
	# not published; go to the database
	$filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
    }
    else {
	$filename = $this->htmlpath."/".$filename;
    }
    $out = "<table class=\"ZineImage\" width=\"$width\"";
    $out .= "><tr><td><img src=\"$filename\" height=\"$height\" width=\"$width\"></td></tr>";
    my $cap = $this->show_data("caption");
    if ($cap) {
	$out .= "\n<tr><td><div class=\"ZineCaption\">$cap</div></td></tr>";
    }
    $out .= "</table>";
    my $img_wrap_size = $config{$this->module_name}{img_max_wrap_size} || 400;
    my $body_wrap_size = $config{$this->module_name}{body_min_wrap_size} || 1000;
    if ($width < $img_wrap_size && $this->size > $body_wrap_size) {
	# we already have table.ZineImage to apply special styles to;
	# conditional wrapping with div.ZineImage allows us to conditionally
	# float the image depending on its size.
	return $this->wrap("ZineImage",$out);
    }
    return $out;
}

sub thumb {
    my ($this,$art) = @_;
    my $stat = $this->handler("ZineThumb");
    return $stat if ($stat);
    my $out;
    $art = $art || $this->{article};
    if ($art->{thumb}) {	
	my $img = new ExSite::Image($art->{thumb});
	my $filename = "$art->{article_id}T_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=thumb&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	$out = "<img class=\"ZineThumb\" src=\"$filename\" border=\"0\"";
    }
    elsif ($art->{picture}) {
	my $img = new ExSite::Image($art->{picture});
	my $filename = "$art->{article_id}_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	my $thumbdim = $config{$this->module_name}{thumbdim} || 100;
	$out .= "<img class=\"ZineThumb\" src=\"$filename\" width=\"$thumbdim\" border=\"0\">";
    }
    return $out;
}

=pod

=head1 Indexes


=cut


# build links to articles

# index:  list articles from a certain point down
# There are two modes:
# - article: indexed articles identified using parent_id field
# - comment: indexed articles identitfied using reference field
#
# There are 3 main formats:
# - threaded (related articles grouped using indentation)
# - list (articles listed in some sort order)
# - paged (next/previous links)

# index style:
# numbered
# bulleted
# summary
# table
# 

# simple article fetching tool - 
# returns articles in a sorted array; with same sortkey, newest articles 
# appear first

# RSS feeds - 
# If a forum is loaded, feed returns comments of that zine.
# If a zine is loaded, feed returns articles of that zine.
# If not, feed returns recent articles.

sub rss {
    my $this = shift;
    my $stat = $this->handler("ZineRSS");
    return $stat if ($stat);
    my $limit = $config{$this->module_name}{rss_limit} || 10;
    my ($title,$description,$currart,$url,@art);
    if ($this->reftype eq "forum") {
	# for forums, we make an RSS index of comments
	$this->save;
	$this->set( $this->type eq "forum" ? 
		    $this->{article} : $this->{reference});
	$title = $this->title;
	$description = $this->abstract;
	@art = $this->fetch_child_articles("comment",$limit);
	$url = $this->link(zine=>"show",aid=>$this->type eq "forum" ? 
			   $this->{article_id} : $this->{reference}{article_id});
	$this->restore;
    }
    elsif ($this->refclass eq "zine") {
	# for other zines, we make an RSS index of articles
	$this->save;
	$this->set( $this->class eq "zine" ? 
		    $this->{article} : $this->{reference});
	$title = $this->title;
	$description = $this->abstract;
	@art = $this->fetch_child_articles("article",$limit);
	$url = $this->link(zine=>"show",aid=>$this->class eq "zine" ? 
			   $this->{article_id} : $this->{reference}{article_id});
	$this->restore;
    }
    else {
	# for all other cases, we make RSS index of recent articles
	if ($share{Page}) {
	    $title = $share{Page}->get("section","title");
	    $description = $share{Page}->get("section","description");
	    $url = $share{Page}->get_url();
	}
	else {
	    $title = $config{site}{name};
	    $description = "";
	    $url = "";
	}
	@art = $this->fetch_latest("article",undef,$limit);
    }
    return undef if (@art == 0);
    my $server = $config{server}{server};

    my $rss = new ExSite::RSS;
    $rss->title($title);
    $rss->description($description);
    $rss->link($server.$url);

    $this->save;
    foreach my $a (@art) {
	$this->set($a);
	$rss->item($this->title,
		   $this->abstract,
		   $this->link(zine=>"show",aid=>$this->{article_id})
		   );
    }

    $this->restore;
    return $rss->write;
}

=pod

INDEXING PARAMETERS

mode
	article
	comment
	attachment

layout
	basic - series of <p>
	list - series of <li>
	table - series of <tr>
	threaded - nested series of <ul>
        paged - sequence of pages

sort
	sorted - by sortkey
	date - earliest first
	reverse-date - latest first
	flip-flop - earliest first for top-level, latest first otherwise
	reverse-flip-flop - latest first for top-level, earliest first otherwise

format
	title - title link only
	basic - title, date, author
	summary - title, date, author, abstract, index summary
	full - entire article, index summary

=cut

###### BUG: indextype & indexformat:
# 


=pod

# trimtree: remove uninteresting articles from the thread tree

sub trimtree {
    my ($this,$tree,$archive,$allow_irregular_indexes) = @_;

    my %archivebin;
    foreach my $art ($tree->get_topnodes_data()) {
	# drop comments in article indexes
	if (! $allow_irregular_indexes && 
	    $art->{type} eq "comment" and $mode eq "article") {
	    $tree->delnode($art->{article_id});
	}
	# drop articles in comment indexes
	if (! $allow_irregular_indexes && 
	    $art->{type} ne "comment" and $mode ne "article") {
	    $tree->delnode($art->{article_id});
	}
	# prune old threads
	if ($this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $archive = 1;
	}
	# flag old articles in live threads, to that threaded
	# indexes can display them differently if they choose to.
	if ($this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $archive = 1;
	}

	# make a first pass through the article tree, organizing
	# everything by age
	my @old_thread_age;
	my @nodelist = ($tree->getnode_data($art->{article_id}));
	my $age = 0;
	while (@nodelist > 0) {
	    my $cart = shift @nodelist;
	    $this->set($cart);
	    $cage = $this->age();
	    if ($cage > $age) { $age = $cage; }
	    push @nodelist,$tree->get_child_data($cart->{article_id});
	    $archivebin{$art->{article_id}} = $cage;
	}
	if ($age > $indexage) {
	    $tree->delnode($art->{article_id});
	    push @old_thread_age, $age;
	}

	# make a second pass through the article tree, organizing
	# everything by age
	my @old_thread_age;
	my @nodelist = ($tree->getnode_data($art->{article_id}));
	my $age = 0;
	while (@nodelist > 0) {
	    my $cart = shift @nodelist;
	    $this->set($cart);
	    $cage = $this->age();
	    if ($cage > $age) { $age = $cage; }
	    push @nodelist,$tree->get_child_data($cart->{article_id});
	    $archivebin{$art->{article_id}} = $cage;
	}
    }
    if ($
    foreach my $art ($tree->get_topnodes_data()) {
	

	# make a first pass through the article tree, organizing
	# everything by age
	my @old_thread_age;
	my @nodelist = ($tree->getnode_data($art->{article_id}));
	my $age = 0;
	while (@nodelist > 0) {
	    my $cart = shift @nodelist;
	    $this->set($cart);
	    $cage = $this->age();
	    if ($cage > $age) { $age = $cage; }
	    push @nodelist,$tree->get_child_data($cart->{article_id});
	    $archivebin{$art->{article_id}} = $cage;
	}
	if ($age > $indexage) {
	    $tree->delnode($art->{article_id});
	    push @old_thread_age, $age;
	}

	# make a second pass through the article tree, organizing
	# everything by age
	my @old_thread_age;
	my @nodelist = ($tree->getnode_data($art->{article_id}));
	my $age = 0;
	while (@nodelist > 0) {
	    my $cart = shift @nodelist;
	    $this->set($cart);
	    $cage = $this->age();
	    if ($cage > $age) { $age = $cage; }
	    push @nodelist,$tree->get_child_data($cart->{article_id});
	    $archivebin{$art->{article_id}} = $cage;
	}
    }


}

=cut



sub index {
    my ($this,%opt) = @_;
    my $stat = $this->handler("ZineIndex",%opt);
    return $stat if ($stat);

    # save current article
    $this->save;

    # get index display parameters
    my $archive = $opt{archive};
    $opt{mode} = $opt{mode} || "article";
    $opt{layout} = $opt{layout} || $opt{archive} ? "basic" : $this->indextype($opt{mode});
    $opt{sort} = $opt{sort} || $this->indexsort($opt{mode});
    $opt{format} = $opt{format} || $this->indexformat($opt{mode});
    $opt{limit} = $opt{limit} || $this->indexlimit($opt{mode});
    $opt{minimum} = $opt{minimum} || $this->get("indexmin",$opt{mode});
    $opt{age} = $opt{age} || $this->indexage($opt{mode});

    # get all the articles in the index, in tree format
    my $tree;
    my $id = $this->{article_id};
    ### FIXME: overloaded $class here
    my $class = $this->class();
    if ($class eq "zine") {
	$tree = $this->fetch_tree($id,{reference=>$id});
    }
    elsif ($class eq "article") {
	if ($opt{layout} eq "paged" && $opt{mode} ne "comment") {
	    $tree = $this->fetch_tree($this->{article}{reference},
				      {reference=>$this->{article}{reference}});
	}
	elsif ($opt{mode} eq "comment") {
	    $tree = $this->fetch_tree($id,{reference=>$id});
	}
	else {
	    $tree = $this->fetch_tree($id,{reference=>$this->{article}{reference}});
	}
    }
    #elsif ($class eq "comment") {
    else { # comment or attachment
	$tree = $this->fetch_tree($id,{reference=>$this->{article}{reference}});
    }

    # some index types require special handling
    if ($opt{layout} eq "threaded") {
	return $this->index_threaded(\%opt,$tree);
    }
    elsif ($opt{layout} eq "semi-threaded") {
	if ($article_class{$this->type} eq "zine") {
	    # semi-threaded indexes show thread heads at the top level
	    return $this->index_semithreaded(\%opt,$tree);
	}
	# otherwise they show a deep index
	$opt{layout} = "deep";
    }
    elsif ($opt{layout} eq "wiki" && $opt{mode} eq "article") {
	return $this->index_wiki(\%opt,$tree);
    }

    my @art;
    if ($opt{layout} eq "deep") { 
	# We need to show the full depth of the tree.
	@art = $this->sort($opt{sort},$tree->collapse());
    }
    else {
	# We only need to show child articles.  User can link
	# through to view more.
	@art = $this->sort($opt{sort},$tree->get_topnodes_data());
    }

    if ($opt{layout} eq "paged") {
	return $this->index_paged(\%opt,@art);
    }
    elsif ($opt{layout} eq "gallery") {
	return $this->index_gallery(0,\%opt,@art);
    }
    elsif ($opt{layout} eq "grid") {
	return $this->index_gallery(1,\%opt,@art);
    }

    # find and filter the archived articles
    my @archive;
    my $iart = 0;
    my $narch = 0;

    # flag all old/expired articles for the archives
    foreach my $a (@art) {
	$this->set($a);
	#next if ($this->class() ne $opt{mode});
	if (($this->class() eq $opt{mode}) ||
	    ($opt{mode} eq "article" && $this->class eq "zine")) {

	    if ($opt{age} && $opt{age} > 0 && $this->age > $opt{age}) {
		$archive[$iart] = 1;
		$narch++;
	    }
	    elsif ($opt{limit} && $opt{limit} > 0 && ($iart-$narch) >= $opt{limit}) {
		$archive[$iart] = 1;
		$narch++;
	    }
	    $iart++;
	}
    }

    # now, unflag enough recently expired articles to keep us under the
    # index limit;  ignore for specialized indexes that don't understand
    # minimum (we may end up here in archive mode for those indexes).
    if ($opt{minimum} && $this->indextype($opt{mode}) !~ /threaded|wiki/) {
	$iart = -1;
	if ((scalar @art - $narch) < $opt{minimum}) {
	    foreach my $a (@art) {
		$iart++;
		next if (!$archive[$iart]);
		$archive[$iart] = 0;
		$narch--;
		last if ((scalar @art - $narch) >= $opt{minimum});
	    }
	}
    }

    my @showart;
    my $archive_on = 0;
    for (my $iart = 0; $iart <= $#art; $iart++) {
	if (($archive && $archive[$iart]) || 
	    (! $archive && ! $archive[$iart])) {
	    push @showart, $art[$iart];
	}
    }

    # build normal list-like indexes
    my ($index);
    foreach my $art (@showart) {
	$this->set($art);
	unless ($opt{mode} eq "article" && $this->class eq "zine") {
	    # ugh;  we do this to allow indexes to contain more zines
	    next if ($this->class() ne $opt{mode});
	}
	my $item;
	if ($opt{layout} eq "table") {
	    $item = $this->show_row;
	}
	elsif ($opt{format} eq "title") {
	    $item = $this->show_link;
	}
	elsif ($opt{format} eq "summary") {
	    $item = $this->show_teaser;
	    ### add summary index
	}
	elsif ($opt{format} eq "full") {
	    $item = $this->show_full($tree,($opt{layout} ne "deep"));
	}
	else { #if ($opt{format} eq "basic") {
	    $item = $this->show_info;
	}
	# add the article to the index listing
	if ($opt{layout} =~ /list|numbered/) {
	    $index .= "<li class=\"ZineItem\">\n$item\n</li>\n";
	}
	elsif ($opt{layout} eq "table") {
	    $index .= "<tr class=\"ZineItem\">\n$item\n</tr>\n";
	}
	else {
	    $index .= "<div class=\"ZineItem\">\n$item\n</div>\n";
	}
    }

    # reset article

    $this->restore;
    #$this->set($currart);

    # wrap the whole index

    if ($index) {
	if ($opt{layout} eq "list") {
	    $index = "<ul class=\"ZineIndex\">\n$index\n</ul>\n";
	}
	elsif ($opt{layout} eq "numbered") {
	    $index = "<ol class=\"ZineIndex\">\n$index\n</ol>\n";
	}
	elsif ($opt{layout} eq "table") {
	    my $row1 = $this->show_row(0);
	    $index = "<table class=\"ZineIndex\">\n<tr>$row1</tr>\n$index\n</table>\n";
	}
	else {
	    $index = "<div class=\"ZineIndex\">\n$index\n</div>\n";
	}
    }

    # show public zine tools (archive/rss)

    my (%tools);
    if ($opt{archive}) {
	#$out .= "<h2>Archives</h2>\n";
	my $url = $this->link(archive=>undef);
	if ($index) {
	    $tools{archive} = "Current $opt{mode}s";
	}
	elsif ($opt{mode} eq "article" && $class eq "zine") {
	    $tools{archive} = "Back to current articles";
	}
    }
    elsif ($narch > 0) {
	$tools{archive} = "Archives";
    }
    if ($this->class eq "zine" && $opt{mode} eq "article" && $index) {
	$tools{rss} = 1;
    }
    if ($this->{article}{reference} && $index) {
	$tools{back} = 1;
    }

    # In certain cases, it's worth noting that the index is empty.
    if (! $index &&
	$opt{mode} eq "article" &&
	$this->type !~ /forum/ &&
	$article_class{$this->type} eq "zine" &&
	! $opt{archive}) { 
	# only show the "no articles" message if we're indexing a Zine
	# ie. don't show it if we're attempting to show a sub-index under
	# a simple article.  Also, not on forums, since they normally have
	# no articles.
	$index = "<p>No current articles found.</p>";
    }

    if (keys %tools > 0) {
	$index .= $this->tools(1,"vertical",%tools);
    }
    return $index;
}

# A gallery indexes a zine using its photos
# $grid==TRUE means show articles in a grid, otherwise use a list

sub index_gallery {
    my ($this,$grid,$opt,@art) = @_;

    my ($out,$archive);

    my $n = 0;
    if (@art > 0) {
	my $article_save = $this->{article};
	foreach my $art (@art) {
	    next if ($art->{type} eq "comment" and $opt->{mode} eq "article");
	    next if ($art->{type} ne "comment" and $opt->{mode} ne "article");
	    $this->set($art);
	    if ($opt->{age} >= 0 && ($this->age > $opt->{age}) && !$opt->{archive}) {
		$archive = 1;
		next;
		#last;
	    }
	    if ($grid) {
		my $thumb = $this->thumb || "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/nophoto.gif\" height=\"50\" width=\"50\" border=\"0\">"; 
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<td align=\"center\" class=\"ZineThumb\"><a href=\"$url\">$thumb</a></td>\n";
	    }
	    else {
		my $thumb = $this->thumb || "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/nophoto.gif\" border=\"0\">";
		my $summary = $this->summarize("teaser");
		my $url = $this->link(zine=>"show",aid=>$this->{article_id});
		$out .= "<tr><td width=\"100\" align=\"center\" class=\"ZineThumb\"><a href=\"$url\">$thumb</a></td><td>$summary</td></tr>\n";
	    }
	    $n++;
	    if ($n == $opt->{limit} && ! $opt->{archive}) {
		$archive = 1;
		last;
	    }
	    if ($grid) {
		my $gwidth = $config{$this->module_name}{gallery_width} || 4;
		if (($n % $gwidth) == 0) {
		    $out .= "</tr><tr>\n";
		}
	    }
	}
	$this->set($article_save);
    }
    if ($out) {
	if ($grid) {
	    $out = "<tr>$out</tr>";
	}
	$out = "<table class=\"ZineIndex\">\n$out\n</table>\n";
    }

    my %tools;
    if ($opt->{archive}) {
	my $url = $this->link(archive=>undef);
	if ($out) {
	    $tools{archive} = "Current $opt->{mode}s";
	}
	elsif ($opt->{mode} eq "article" && $this->class eq "zine") {
	    $tools{archive} = "Back to current articles";
	}
    }
    elsif ($archive > 0) {
	$tools{archive} = "Archives";
    }
    if ($this->class eq "zine" && $opt->{mode} eq "article") {
	$tools{rss} = 1;
    }
#    if ($this->{article}{reference}) {
#	$tools{back} = 1;
#    }

    if (keys %tools > 0) {
	$out .= $this->tools(1,"vertical",%tools);
    }

    return $out;
}

sub index_threaded {
    my ($this,$opt,$tree) = @_;
    my $stat = $this->handler("ZineIndexThreaded",$opt,$tree);
    return $stat if ($stat);

    my $out;
    my $mode = $opt->{mode};
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    my $comment_policy = $opt->{comment_policy} || $this->comment_policy($mode);
    my $minimum = $opt->{minimum} || $this->get("indexmin",$opt->{mode});

    # build a tree from the articles
    if (! $tree) { $tree = $this->fetch_tree($this->{article_id}); }

    my $archive;
    $this->{articlecount} = 0;

    # prune uninteresting articles from the thread tree
    foreach my $art ($tree->get_topnodes_data()) {
	# drop comments in article indexes
	if ($art->{type} eq "comment" and $mode eq "article") {
	    $tree->delnode($art->{article_id});
	}
	# drop articles in comment indexes
	if ($art->{type} ne "comment" and $mode ne "article") {
	    $tree->delnode($art->{article_id});
	}
	# prune old threads
	if ($indexage > 0 && $this->age_of_thread($tree,$art) > $indexage) {
	    $tree->delnode($art->{article_id});
	    $archive = 1;
	}
    }

    # display remaining threads
    $this->{thread_depth} = 0;
    my @articles = reverse($tree->get_topnodes_data());
    $out .= $this->index_thread($indexformat,$indexlimit,$indexage,
				$tree,@articles);

#    if ($archive) {
#	# there appear to be some threads that are turned off due to age
#	my $url = $this->link(archive=>1);
#	my $type = $opt->{mode} eq "comment" ? "comment" : undef;
#	$out .= "<p><a href=\"$url\">Show $type archives</a>.</p>\n";
#    }

    my %tools;
    if ($archive) {
	my $url = $this->link(archive=>1);
	my $type = $opt->{mode} eq "comment" ? "comment" : undef;
	$tools{archive} = "Show $type archives";
    }
    if ($this->class eq "zine" && $opt->{mode} eq "article") {
	$tools{rss} = 1;
    }

    if (keys %tools > 0) {
	$out .= $this->tools(1,"vertical",%tools);
    }

    return $out;
}

sub index_thread {
    my ($this,$format,$limit,$age,$tree,@articles) = @_;
    my $out;

    # replies will be indented up to $max_depth levels
    $this->{thread_depth}++;
    my $max_depth = $config{Zine}{max_thread_depth} || 4;

    foreach my $art (@articles) {
	my $fmt = $format;
	$this->set($art);
	next if ($this->type() eq "attachment");
	$this->{articlecount}++;
	if ($fmt ne "brief") {
	    if ($this->{articlecount} > $limit) {
		$fmt = "brief";
	    }
	    elsif ($age > 0 && ($this->age > $age)) {
		$fmt = "brief";
	    }
	    elsif ($config{$this->module_name}{shrink_anonymous_comments} && ! $art->{owner}) {
		$fmt = "brief";
	    }
	}
	if ($fmt eq "full") {
	    $out .= $this->show_article();
	}
	elsif ($fmt eq "brief") {
	    $out .= "<div class=\"ZineItem\">\n".$this->show_info()."</div>\n";
	}
	my @children = $tree->get_child_data($art->{article_id});
	if (@children > 0) {
	    my $indent = 0;
	    if ($this->{thread_depth} <= $max_depth) { 
		$indent = 1;
		$out .= "<ul>\n"; 
	    }
	    $out .= $this->index_thread($format,$limit,$age,$tree,@children);
	    if ($indent) { $out .= "</ul>\n"; }
	}
    }
    return $out;
}

# a semithreaded index displays the head article of each thread in a
# table;  this article links to a "deep" index of the articles in that
# thread.  This index method is popular with some BBS-style forums.

sub index_semithreaded {
    my ($this,$opt,$tree) = @_;
    my $stat = $this->handler("ZineIndexSemiThreaded",$opt,$tree);
    return $stat if ($stat);

    my $out;
    my $mode = $opt->{mode};
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    my $comment_policy = $opt->{comment_policy} || $this->comment_policy($mode);

    # build a tree from the articles
    if (! $tree) { $tree = $this->fetch_tree($this->{article_id}); }

    my $archive;
    $this->{articlecount} = 0;

    # prune uninteresting articles from the thread tree
    my @articles;
    foreach my $art (reverse $tree->get_topnodes_data()) {
	# ignore attachments
	next if ($art->{type} eq "attachment");
	# drop comments in article indexes
	if ($art->{type} eq "comment" and $mode eq "article") {
	    next;
	}
	# drop articles in comment indexes
	if ($art->{type} ne "comment" and $mode ne "article") {
	    next;
	}
	# prune old threads
	if ($indexage > 0 && $this->age_of_thread($tree,$art) > $indexage) {
	    $archive = 1;
	    next;
	}
	push @articles,$art;
    }

    my $count_type = $opt->{mode} eq "article" ? "Articles" : "Replies";
    $this->save;
    my $nart = 0;
    my @data;
    foreach my $art (@articles) {
	$nart++;
	$this->set($art);
	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
	my $title = $this->title;
	my $author = $this->author;
	my $date = $this->date;
	my @clist = $tree->collapse($this->{article_id});
	my $count = 0;
	foreach my $c (@clist) { if ($c->{type} eq $opt->{mode}) { $count++; } }
	push @data, ["<a href=\"$url\">$title</a>",$author,$date,$count];
    }
    $this->restore;

    my (%tools, $tools);
    if ($archive) {
	$tools{archive} = 1;
    }
    if ($this->class eq "zine" && $opt->{mode} eq "article" && @data > 0) {
	$tools{rss} = 1;
    }
    if ($this->{article}{reference} && @data > 0) {
	$tools{back} = 1;
    }
    if (keys %tools > 0) {
	$tools = $this->tools(1,"vertical",%tools);
    }

    my $out;
    if (@data > 0) {
	my $rpt = new ExSite::ReportBuilder(
					    headers=>["Thread","Author","Date",$count_type],
					    data=>\@data,
					    class=>"ZineIndex",
					    #tools=>$tools,
					    );
	$out = $rpt->make;
    }
    $out .= $tools;
    return $out;
}

# the age of a thread is the age of the youngest article in the thread

sub age_of_thread {
    my ($this,$tree,$article) = @_;
    
    my @nodelist = ($tree->getnode_data($article->{article_id}));
    my $age = $nodelist[0]{mtime};
    while (@nodelist > 0) {
	my $art = shift @nodelist;
        if ($config{timestamp_format} eq "num") {
	    $age = $art->{mtime} < $age ? $art->{mtime} : $age;
        }
        else {
	    $age = ($art->{mtime} cmp $age < 0) ? $art->{mtime} : $age;
        }
	push @nodelist,$tree->get_child_data($art->{article_id});
    }
    # convert to a day count
    my $time = new ExSite::Time; # now
    # time diff is in seconds; convert to days
    my $tdiff = ($time->diff($age,"sql_timestamp")) / 86400;
    return - int($tdiff);
}

sub my_zine {
    my $this = shift;
    my $class = $this->class;
    if ($class eq "zine") {
	return $this->{article};
    }
    elsif ($class eq "article" || ($this->reftype eq "forum" && $class eq "comment")) {
	return $this->{reference};
    }
    elsif ($class eq "comment") { 
	return $this->fetch("article",$this->{reference}{reference});
    }
}

sub my_article {
    my $this = shift;
    my $class = $this->class;
    if ($class eq "zine") {
	return undef;
    }
    elsif ($class eq "article") {
	return $this->{article};
    }
    elsif ($class eq "comment" && $this->reftype eq "forum") { 
	return undef;
    }
    elsif ($class eq "comment") { 
	return $this->{reference};
    }
}

sub my_comment {
    my $this = shift;
    my $class = $this->class;
    if ($class eq "zine") {
	return undef;
    }
    elsif ($class eq "article") {
	return undef;
    }
    elsif ($class eq "comment") {
	return $this->{article};
    }
}

sub show_path {
    my ($this,%opt) = @_;
    my ($url,@plink);
    if ($this->class ne "zine") {
	if ($config{$this->module_name}{show_top}) {
	    $url = $this->link(zine=>undef,aid=>undef,
			       zadm=>undef,archive=>undef,goto=>undef);
	    push @plink, "<a href=\"$url\">Top</a>";
	}
	my $art = $this->my_zine;
	if ($art) {
	    $url = $this->link(zine=>"show",aid=>$art->{article_id},
			       zadm=>undef,archive=>undef,goto=>undef);
	    push @plink, "<a href=\"$url\">$art->{title}</a>";
	    $art = $this->my_article;
	    if ($art) {
		$url = $this->link(zine=>"show",aid=>$art->{article_id},
				   zadm=>undef,archive=>undef,goto=>undef);
		push @plink, "<a href=\"$url\">$art->{title}</a>";
		$art = $this->my_comment;
		if ($art) {
		    $url = $this->link(zine=>"show",aid=>$art->{article_id},
				       zadm=>undef,archive=>undef,goto=>undef);
		    push @plink, "<a href=\"$url\">Comment $art->{article_id}</a>";
		}
	    }
	    elsif ($this->reftype eq "forum") {
		# forums can have comments without having articles
		$art = $this->my_comment;
		if ($art) {
		    $url = $this->link(zine=>"show",aid=>$art->{article_id},
				       zadm=>undef,archive=>undef,goto=>undef);
		    push @plink, "<a href=\"$url\">Comment $art->{article_id}</a>";
		}
	    }

	}
    }
    if (@plink > 0) {
	return "<p>".join(" &gt; ",@plink)."</p>\n";
    }
    return undef;
}

sub index_paged {
    my ($this,$opt,@art) = @_;
    my $index;
    my @art2;

    # get articles of correct type
    foreach my $a (@art) {
	next if ($article_class{$a->{type}} ne $opt->{mode});
	push @art2,$a;
    }
    return undef if (@art2 == 0);

    # figure out where we are in the index
    my $id = $this->{article_id};
    my $pos = 0;
    foreach my $a (@art2) { 
	last if ($a->{article_id} == $id);
	$pos++;
    }

    if ($pos > $#art2) {
	# we do not appear to be in the index yet.
	# Show the first article.
	$this->save;
	$this->set($art2[0]);
	$index .= $this->show_article();
	$this->restore;
	$pos = 0;
    }

    my $next = $pos < $#art2 ? $pos + 1 : -1;
    my $prev = $pos > 0 ? $pos - 1 : -1;

    my ($url,@indexlink);
    if ($prev >= 0) {
	$url = $this->link(aid=>$art2[$prev]{article_id},zine=>"show");
	push @indexlink, "<a href=\"$url\">&lt;&nbsp;Prev</a>";
    }
    my $n = 0;
    foreach my $a (@art2) {
	$n++;
	$url = $this->link(aid=>$a->{article_id},zine=>"show");
	my $class = ($n - 1) == $pos ? " class=\"ZinePagedIndexCurrent\"" : undef;
	push @indexlink, "<a href=\"$url\"$class>$n</a>";
    }
    if ($next >= 0) {
	$url = $this->link(aid=>$art2[$next]{article_id},zine=>"show");
	push @indexlink, "<a href=\"$url\">Next&nbsp;&gt;</a>";
    }

    if (@indexlink > 0) {
	$index .= "<div class=\"ZinePagedIndex\">More: ".join("&nbsp;",@indexlink)."</div>\n";
    }
    return $index;
}

sub index_wiki {
    my ($this,$opt) = @_;
    my $stat = $this->handler("ZineIndexWiki",$opt);
    return $stat if ($stat);
    my ($index,$out,$body,@refs,$start);

    # build an index based on the current article
    $body = $this->body;
    @refs;
    while ($body =~ /\[([^\]])\]/) {
	push @refs,$1;
	$body =~ s/\[([^\]])\]//g;
    }
    foreach my $ref (sort @refs) {
	my $url = $this->link(goto=>$ref);
	$index .= "<li><a href=\"$url\">$ref</a></li>\n";
    }

    if ($this->type eq "wiki") {
	# if we're under the wiki itself, let's search for related 
	# articles to display
	my $wiki_id = $this->type eq "wiki" ?
	    $this->{article_id} : $this->{reference}{article_id};
	$this->save;
	#my $currart = $this->{article};
	# now lookup any requested articles;  goto "=start" by default
	my $goto = $opt->{term} || lc($this->{input}{goto}) || "=start";
	if ($goto) {
	    #$out .= "<p>Looking for articles about \"$goto\"...</p>\n";
	    $goto =~ s/\s/_/g;
	    my @wart = $share{DB}->fetch_match("article",
					       {reference=>$wiki_id,
						name=>"%$goto%",
						status=>"active"},
					       "title");
	    if (@wart == 0 && $goto =~ /s$/) {
		# hmm, no hits;  maybe we should drop the "s"
		my $singular_goto = $goto;
		$singular_goto =~ s/s$//;
		@wart = $share{DB}->fetch_match("article",
						{reference=>$wiki_id,
						 name=>"%$singular_goto%",
						 status=>"active"},
						"title");
	    }
	    foreach my $wart (@wart) {
		$this->set($wart);
		$out .= $this->show_full(undef,1);
		my $st_body = $this->body;
		while ($st_body =~ /\[([^\]])\]/) {
		    push @refs,$1;
		    $st_body =~ s/\[([^\]])\]//g;
		}
		foreach my $ref (sort @refs) {
		    my $url = $this->link(goto=>$ref);
		    $index .= "<li><a href=\"$url\">$ref</a></li>\n";
		}
	    }
	    if (@wart == 0) {
		$out .= "<p>No articles found for \"$goto\".</p>\n";
	    }	    
	    $this->restore;
	    #$this->set($currart);
	}
    }
#    else {
#	# if we're under a particular wiki article, then let's lets just 
#	# give a link back to the wiki index
#	my $url = $this->link(aid=>$this->{reference}{article_id});
#	$out .= "<p><a href=\"$url\">Back to $this->{reference}{title} index.</a></p>\n";
#    }

    if ($index) { 
	$out .= "<h4>Related articles:</h4>\n<ul>\n$index\n</ul>\n";
    }

    # TODO: should we also show the articles that link _to_ us?

    return $out;
}

sub summarize {
    my ($this,$format) = @_;
    my $out;
    my $url = $this->link(aid=>$this->{article_id},zine=>"show");
    my $title = $this->title || "(no title)";
    if ($format eq "teaser") {
	my $subtitle = $this->show_data("subtitle");
	my $author = $this->author;
	my $body = $this->body;
	#---
	# disable oysters in teasers
	$body =~ s/<!--\&/<!-- \&/g;
	#---
	my $head = "<div class=\"ZineTeaserHeader\"><a href=\"$url\">$title</a>";
	if ($this->{article}{subtitle}) {
	    $head .= ", <em>$subtitle</em>";
	}
	$head .= "<br>";
	if ($author && $this->{article}{type} !~ /blog/ && $this->reftype() !~ /blog/) {
	    # don't show individual article authors in blogs
	    $head .= "<em>by $author</em> ";
	}
	if ($this->{article}{type} =~ /comment/ || $this->reftype() =~ /blog|e-zine/) {
	    # show article posting dates in comments, blogs, and e-zines only
	    $head .= $this->date;
	}
	$head .= "</div>\n";

	my $teaserlen = $config{$this->module_name}{teaser_length} || 200;
	my $index_summary = $this->index_summary;
	my $summary = $this->abstract;
	if ($index_summary) { $index_summary = "($index_summary)"; }
	$out .= "$head\n<div class=\"ZineTeaser\">$summary<br>$index_summary</div>\n";
    }
    elsif ($format eq "full") {
	$_ = $this->type;
	if (/comment/) { $out .= $this->show_comment; }
	else { $out .= $this->show_article; }
    }
    else { #if ($format eq "brief") {
	$out .= "<div class=\"ZineTeaser\"><a href=\"$url\">$title</a>";
	if ($this->{article}{type} eq "comment") {
	    my $date = $this->show_data("ctime");
	    my $author = $this->author;
	    $out .= " (posted on $date by $author)";
	}
	else {
	    my $summ = $this->show_data("summary");
	    my $author = $this->author;
	    if ($author) { $out .= ", <i>by $author</i>" }
	    if ($summ) { $out .= "<br><font size=-1>$summ</font>"; }
	}
	$out .= "</div>";
    }
    return $out;
}

# not a real index, since we don't link to other articles, simply
# to ourself

sub index_summary {
    my $this = shift;
    my $nart = $share{DB}->count("article",{ parent_id=>$this->{article_id},
					     type=>"article",
					     status=>"active" } );
    my $ncom = $share{DB}->count("article",{ reference=>$this->{article_id},
					     type=>"comment",
					     status=>"active" } );
    my @counts;
    if ($nart) { push @counts, "$nart article".($nart > 1 ? "s" : ""); }
    if ($ncom) { push @counts, "$ncom comment".($ncom > 1 ? "s" : ""); }
    my $out = join ("; ",@counts);
    if ($out) { 
	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
	return "<a href=\"$url\">$out</a>";
    }
    return "";
}

# index_latest: summarize recently added articles

sub index_latest {
    my ($this,$type,$id,$number) = @_;
    return undef if ($type !~ /^(article|comment)$/);
    my $out;
    my @latest = $this->fetch_latest($type,$id,$number);
    if (@latest > 0) {
	# conf file option to turn off header
	unless ( $config{$this->module_name}{no_header} == 1 ) { 
	    $out = "<div class=\"Zine\">\n<h1>Recent ".ucfirst($type)."s</h1>\n";
	}
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
	    my $auth = $this->author();
	    my $author = $auth ? "by $auth" : "";
	    my $reftitle = $this->reftitle();
	    my $date = $this->date;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= "<p class=\"ZineTeaser\"><a href=\"$url\" class=\"ZineTitle\">$title</a> $author $date in <a href=\"$rurl\" class=\"ZineRef\">$reftitle</a></p>\n";
	}
	$out .= "</div>\n";
    }
    return $out;
}

sub index_toggle {
    my ($this,$type,$id) = @_;
    return undef if ($type !~ /^(article|comment)$/);
    my ($out,@aid);
    if ($id) {
	@aid = ($id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
    return undef if (scalar @aid == 0);
    my @latest = $share{DB}->custom_query("select * from article where reference in (".join(",",@aid).") and type=\"$type\" and status=\"active\" order by article_id desc");
    if (@latest > 0) {
	# conf file option to turn off header
		$out = <<"DONE";

<script language="JavaScript">
function toggle(link, div1id) {
	var div1 = document.getElementById(div1id);
	if (div1.style.display == 'none') {
		div1.style.display = 'block';
	} 
	else {
		div1.style.display = 'none';
	}
}

</script>
DONE
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
		my $body = $this->body;
	    my $auth = $this->author();
	    my $author = $auth ? "by $auth" : "";
	    my $reftitle = $this->reftitle();
	    my $time = $this->timestamp;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= "<p><div class=\"ZineTeaser\"><a onclick=\"toggle(this,$this->{article}{article_id});\" href=\"javascript:void(0);\">$title</a></div></p>\n";
		
		$out .= "<div class=\"ZineBody\" style=\"display: none;\" id=$this->{article}{article_id}><div id=msg>$body</div></div>";
	}
	$out .= "</div>\n";
    }
    return $out;
}



sub index_latest_articles {
    my ($this,$id,$number) = @_;
    return $this->index_latest("article",$id,$number);
}

sub index_latest_comments {
    my ($this,$id,$number) = @_;
    return $this->index_latest("comment",$id,$number);
}

# index of available zines (site-specific)

sub topview {
    my ($this,$opt) = @_;
    my $stat = $this->handler("ZineTopView",$opt);
    return $stat if ($stat);
    my %match = (status=>"active", section_id=>0, reference=>0);
    my (@art,$type);
    if ($opt && $opt !~ /=/) { 
	# request to view a particular class of top-level articles
	$type = $opt; 
    }

    # get all zines for this site
    @art = $this->our_zines();

    my $out;
    if (@art == 1) {
	$this->save();
	$this->set($art[0]);
	$out = $this->show();
	$this->restore();
    }
    elsif (@art > 0) {
	$out .= "<dl class=\"ZineIndex\">\n";
	foreach my $art (@art) {
	    next if ($type && $art->{type} ne $type);
	    $this->set($art);
	    my $url = $this->link(zine=>"show",aid=>$art->{article_id});
	    my $title = $this->title;
	    my $author = $this->author;
	    my $summary = $this->show_data("summary");
	    $out .= "<dt class=\"ZineItem\"><a href=\"$url\">$title</a>&nbsp;&nbsp;&nbsp;";
	    if ($author && $art->{type} !~ /forum|wiki/) {
		$out .= "by $author";
	    }
	    $out .= "</dt>\n";
	    if ($summary) {
		$out .= "<dd><i>$summary</i>";
	    }
	    $out .= "<p>\n";
	}
	$out .= "</dl>\n";
    }
    return $out;
}

# index of all zines

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return "<p class=error>Permission denied.</p>\n";
    }

    my (@art,$match);
    if ($share{DB}->is_admin) {
	$match = {reference=>0};
    }
    else {
	$match = {reference=>0,owner=>$share{DB}->my_uid()};
    }
    if ($opt) { $match->{type} = $opt; }
    @art = $share{DB}->fetch_match("article",$match);

    my $out;
    my @adata;
    if (@art > 0) {
	foreach my $art (@art) {
	    $this->set($art);
	    my $url = $this->link(zine=>"show",zadm=>undef,aid=>$art->{article_id});
	    my $title = "<a href=\"$url\">".$this->title,"</a>";
	    my $summary = $this->show_data("summary");
	    my $site;
	    if ($art->{section_id}) {
		my $s = $share{DB}->fetch("section",$art->{section_id});
		$site = $s->{title};
	    }
	    else {
		$site = "(shared)";
	    }
	    my $type = $this->type;
	    my $tools = $this->tools($art);
	    push @adata, [$title,$type,$site,$tools];
	}
    }
    my @head = ("Zine","Type","Section","Options");
    my $zrpt = new ExSite::ReportBuilder(
					 title=>"Zines",
					 headers=>\@head,
					 data=>\@adata,
					 );
    return $zrpt->make;
}





=pod

=head1 Posting & Updating Articles


=cut






=pod

=head1 Security


=cut






=pod

=head1 Indexes


=cut


# tools -
# new article
# comment
# reply
# configure
# delete
# approve
# reject
# current/archives
# rss


sub tools {
    my ($this,$labels,$orientation,%conf) = @_;
    my ($out,$url,$ntool,$article,$aid);
    $article = $this->{article};
    $aid = $article->{article_id};

    if (scalar keys %conf == 0) {
	# show default tools
	$conf{reply} = 
	    $article->{type} eq "comment" && 
	    $this->allow_comments;
	$conf{comment} = 
	    $article->{type} =~ /article|forum/ && 
	    $this->allow_comments;
#	    $this->allow_comments("article");
	$conf{new} = 
	    # all zines get a new article link
	    ( $this->{admin} && 
	      $this->class eq "zine" ) || 
	    # books get a new article link under every article
	    ( $this->{admin} && 
	      $this->{type} eq "article" && 
	      $this->reftype eq "book") ||
	    # blogs get a new article link on normal page views
	    ( $article->{type} =~ /blog/ &&
	      $article->{owner} == $share{DB}->my_uid &&
	      $share{Page} && ! $share{Page}->publishing );
	$conf{edit} = 
	    $this->{admin} ||
	    # commenters can optionally edit their own comments
	    ( $article->{type} eq "comment" && 
	      $share{Page} && ! $share{Page}->publishing &&
	      $article->{owner} == $share{DB}->my_uid &&
	      $this->allow_comments );
	$conf{configure} = 
	    $this->{admin};
	$conf{delete} = 
	    $this->{admin};
	$conf{approve} = 
	    $article->{status} =~ /inactive|pending|rejected/ &&
	    $this->moderator;
	# $conf{rss} and $conf{archive} are only set manually
    }

    if (! defined $labels) { 
	$labels = $config{$this->module_name}{icon_labels} || "1";
    }
    if (! defined $orientation) { 
	$orientation = $config{$this->module_name}{tool_orientation} || "horizontal";
    }
    if (! exists $this->{tools}) { $this->{tools} = {}; }

    my (@toolicon,@toollabel,@toolurl);
    if ($conf{reply}) {
	push @toolurl, $this->link(zine=>"reply",aid=>$aid);
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/reply.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"reply\" title=\"reply to this\">";
	push @toollabel, "reply";
	$ntool++;
    }
    elsif ($conf{comment}) {
	push @toolurl, $this->link(zine=>"new",aid=>$aid);
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/comment.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new comment\" title=\"new comment\">";
	push @toollabel, "comment";
	$ntool++;
    }

    if ($conf{new}) {
	push @toolurl, $this->link(zine=>"newarticle",aid=>$article->{article_id});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"comment\" title=\"new article\">";
	push @toollabel, "new";
	$ntool++;
    }

    if ($conf{edit}) {
	my $edit = 1;
	if (! $this->{admin}) {
	    # we can only edit our comments within a few minutes of posting
	    my $t = new ExSite::Time;
	    my $t1 = $t->write("unix_timestamp");
	    $t->set($article->{ctime},"sql_timestamp");
	    my $t2 = $t->write("unix_timestamp");
	    $edit = ($t1 - $t2 < $config{$this->module_name}{edit_my_comment_timeout});
	}
	if ($edit) {
	    push @toolurl, $this->link(zine=>"edit",aid=>$article->{article_id});
	    push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/edit.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"edit\" title=\"edit\">";
	    push @toollabel, "edit";
	    $ntool++;
	}
    }

    if ($conf{configure}) {
	push @toolurl, $this->link(zine=>"conf",aid=>$article->{article_id});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/tool.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"configure\" title=\"configure\">";
	push @toollabel, "conf";
	$ntool++;
    }

    if ($conf{delete}) {
	push @toolurl, $this->link(zine=>"del",aid=>$article->{article_id});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/delete.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"delete\" title=\"delete\">"; 
	push @toollabel, "delete";
	$ntool++;
    }

    if ($conf{approve}) {
	push @toolurl, $this->link(zine=>"approve",aid=>$article->{article_id});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/approve.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"approve\" title=\"approve\">";
	push @toollabel, "approve";
	$ntool++;
	push @toolurl, $this->link(zine=>"reject",aid=>$article->{article_id});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/reject.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"reject\" title=\"reject\">";
	push @toollabel, "reject";
	$ntool++;
    }

    if ($conf{back}) {
	push @toolurl, $this->link(zine=>"show",aid=>$article->{reference});
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/back.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"back\" title=\"back\">";
	push @toollabel, "top";
	$ntool++;
    }

    if ($conf{archive}) {
	if ($this->{input}{archive}) {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>undef);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : "current";
	}
	else {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>1);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : "archives";
	}
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/archive.gif\" border=\"0\">";
	$ntool++;
    }

    if ($conf{rss}) {
	my $rssfile = $this->diskpath."/index.rss";
	push @toolurl, (-e $rssfile ? 
			$this->htmlpath."/index.rss" : 
			$this->link(zine=>"rss",aid=>$this->{article_id}));
	push @toolicon, "<img src=\"$config{server}{HTMLpath}/_Modules/Zine/rss.gif\" border=\"0\">";
	push @toollabel, "rss";
	$ntool++;
    }

    if ($ntool) {
	$out = "<table class=\"ZineTools\"><tr>\n";
	for (my $itool = 0; $itool < $ntool; $itool++) {
	    my $url = shift @toolurl;
	    my $icon = shift @toolicon;
	    my $label = shift @toollabel;
	    if (! $labels) {
		$out .= "<td><a href=\"$url\">$icon</a></td>\n";
	    }
	    elsif ($orientation eq "horizontal") {
		$out .= "<td><a href=\"$url\">$icon<br>$label</a></td>\n";
	    }
	    else {
		$out .= "<td><a href=\"$url\">$icon</a></td><td><a href=\"$url\">$label</a></td></tr>\n<tr>";
	    }
	}
	$out .= "</tr></table>\n";
    }

    return $out;
}

#=============================================================================
# LEGACY CODE

sub show_wiki {
    my ($this,$term) = @_;
    if (! $this->approve) {
	return "<p class=error>Sorry, this article is not available for viewing.</p>\n";
    }

    # remember current article
    $this->save;
    #my $currart = $this->{article};

    my $wiki_id;
    if ($this->{article}{type} eq "wiki") {
	$wiki_id = $this->{article}{article_id};
    }
    else {
	$wiki_id = $this->{article}{reference};
    }

    my $out;
    my $goto = $term || lc($this->{input}{goto}) || "=start";

    if ($goto) {
	my @wart = $share{DB}->fetch_match("article",
					   {reference=>$wiki_id,
					    name=>"%$goto%",
					    status=>"active"},
					   "title");
	if (@wart == 0 && $goto =~ /s$/) {
	    # hmm, no hits
	    my $singular_goto = $goto;
	    $singular_goto =~ s/s$//;   # try singular
	    @wart = $share{DB}->fetch_match("article",
					    {reference=>$wiki_id,
					     name=>"%$singular_goto%",
					     status=>"active"},
					    "title");
	}
	foreach my $wart (@wart) {
	    $this->set($wart);
	    $out .= "<p style=\"border-bottom:1px solid black;\" align=\"right\">&nbsp;</p>\n";
	    $out .= "<h1 class=\"Zine\">$wart->{title}</h1>\n";
	    if ($wart->{subtitle}) {
		$out .= "<h3 class=\"Zine\">$wart->{subtitle}</h3>\n";
	    }
#	    if ($wart->{type} =~ /book|article/) {
#		my $author = $this->author($wart);
#		if ($author) {
#		    $out .= "<h4 class=\"Zine\">By $author</h3>\n";
#		}
#	    }
	    if ($wart->{summary}) {
		$out .= "<blockquote><i>$wart->{summary}</i></blockquote>\n";
	    }
	    my $body .= $this->body($wart);
	    $body =~ s/\[(.+?)\]/$this->wikilink($1)/egs;
	    $out .= "<p>$body</p>";
	    if ($wart->{about}) {
		$out .= "<br><p><i>".$this->show_data("about")."</i></p>\n";
	    }
	    $out .= $this->tools($wart);

	    $out .= $this->index("comment",(archive=>$this->{input}{archive}));

	}
	if (@wart == 0) {
	    $out .= "<p>No articles found for \"$goto\".</p>\n";
	}	    
    }

    # wiki search form

    $goto = lc($this->{input}->{goto});
    my $url = $this->link(goto=>undef);
    $out .= "<p><form method=\"POST\" action=\"$url\">
<input type=\"text\" name=\"goto\" size=\"20\" value=\"$goto\">
<input type=\"submit\" value=\" Go \">
</form></p>\n";

    # back link
    
    if ($this->{input}{goto}) {
	my $backurl = $this->link(zine=>"show",aid=>$this->{article}{reference},goto=>undef);
	$out .= "<p><a href=\"$backurl\">Back to top</a>.</p>\n";
    }

    # restore current article
    $this->restore;
    #$this->set($currart);

    return $out;
}

sub wikilink {
    my ($this,$name) = @_;
    my $url = $this->link(goto=>$name);
    return "<a href=\"$url\">$name</a>";
}

#==========================================================================
# posting methods

# notify an author/owner when their article has been commented on

sub notify {
    my ($this,$data) = @_;
    return if (! $config{$this->module_name}{notify});
    return if (! $data->{parent_id});
    $this->fetch($data->{parent_id});
    my $email = $this->author_email;
    return if (! $email);
    my $url = $this->link(aid=>$data->{article_id},zine=>"show");
    my $purl = $this->link(aid=>$data->{parent_id},zine=>"show");
    my $turl;
    if ($data->{thread}) {
	$turl = $this->link(aid=>$data->{thread},zine=>"show");
    }
    my $zurl = $this->link(aid=>$data->{reference},zine=>"show");
    my $msg = "
$data->{author} has commented on your message '".this->title."' at
$config{server}{server}

---------------------------------------------------------------------
From: $data->{author}
Subject: $data->{title}

$data->{body}
---------------------------------------------------------------------

This message alone may be viewed (and replied to) at:
$config{server}{server}$url

Your original message (including replies) may be viewed at:
$config{server}{server}$purl\n";
    if ($turl) {  ### fixme: assumes threaded comment index
	$msg .= "
The whole thread may be viewed at:
$config{server}{server}$turl\n";
    }
    $msg .= "
The whole discussion may be viewed at:
$config{server}{server}$zurl
";
    return &SendMail("morgan\@exware.com",
		     $config{site}{administrator},
		     "[$config{site}{name}] $data->{title}",
		     $msg);
}

# newmsg: wrapper for compose

sub newmsg {
    my $this = shift;
    my $subject = $this->title;
    if ($subject !~ /^Re:/i) {
	$subject = "Re: $subject";
    }
    return $this->makeform({type=>"comment",title=>$subject},$this->{article});
}

# reply: wrapper for compose, which inserts content from parent message

sub reply {
    my $this = shift;
    my $art = $this->{article};
    # add "Re:" to subject
    my $subject = $this->title;
    if ($subject !~ /^Re:/i) {
	$subject = "Re: $subject";
    }
    # quote parent article
    my $quote .= "&gt;".$this->{article}{body};
    $quote =~ s/\n/\n\&gt\;/g;
    $quote = $this->author." wrote:\n".$quote;
    my $out = $this->makeform({type=>"comment",title=>$subject,body=>$quote},
			      $this->{article});

    if ($out =~ /<form/i) {
	$out .= "<p>Please edit any quoted text for brevity, before submitting your comments.</p>\n";
    }
    return $out;
}

#==========================================================================
# we have our own form processing tools for Zine

sub makeform {
    my ($this,$data,$parent) = @_;
    my $db = $share{DB};
    if (! $data->{type}) { $data->{type} = "article"; }
    if (! $parent) { $parent = $this->fetch($data->{parent_id}); }

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);

    if (! $data->{parent_id}) { $data->{parent_id} = $parent->{article_id}; }
    if (! $data->{reference}) { 
	if ($data->{type} eq "comment" && $this->type ne "comment") {
	    # reset the reference article if this is a top-level comment
	    $data->{reference} = $parent->{article_id};
	}
	elsif ($article_class{$parent->{type}} eq "zine") {
	    # we should reference the nearest Zine to get good indexing behaviour
	    
	    $data->{reference} = $parent->{article_id};
	}
	else {
	    # maintain the same reference article
	    $data->{reference} = $parent->{reference} || $parent->{article_id}; 
	}
    }
    if (! $data->{thread} && $data->{type} eq "comment") { 
	$data->{thread} = $parent->{thread} || $parent->{article_id};
    }

    my $type = $data->{type};
    if ($type eq "article" && $parent->{type} eq "blog") {
	$type .= "_blog";
    }
    elsif ($type eq "article" && $parent->{type} eq "photoblog") {
	$type .= "_photoblog";
    }
    elsif ($type eq "article" && $parent->{type} eq "wiki") {
	$type .= "_wiki";
    }

    my $columns = $formtype{$type};
    my @hide;
    if ($this->{admin}) {
	$data->{owner} = $db->my_uid;
	if (! $data->{section_id}) {
	    $data->{section_id} = $db->this_site()->{section_id};
	}
	if ($db->is_admin) {
	    push @$columns, "owner";
	    push @$columns, "section_id";
	}
	else {
	    @hide = ("owner", "section_id");
	}
    }

    my $out;
    if ($data->{article_id}) {
	$out = "<h3>Edit $article_type{$type}</h3>\n";
    }
    else {
	$out = "<h3>New $article_type{$type}</h3>\n";
    }

    my $map = $db->{map}->get_map("article");

    my $form;
    foreach my $col (@$columns) {
	if ($col eq "content_type" && $db->{htmleditor}) {
	    # don't prompt for content type if we're using the HTMLeditor
	    next;
	}
	if ($col eq "author" && ! $data->{author} && $db->authorize()) {
	    $data->{author} = $db->my_name();
	}
        my %column = $db->{map}->get_column("article",$col);

	my $label = $this->label(\%column,$type);
	my $reqd = $column{validate} =~ /hard|soft/ ? "(required)" : "";
 	$form .= "<p>$label $reqd<br>\n";

	if ($col eq "body" && 
	    $type =~ /article/ &&
	    $this->{input}{input_method} ne "text" && 
	    (($this->reftype ne "wiki" && 
	      $db->use_wysiwyg) || 
	     ($this->{input}{input_method} eq "html"))) {
	    # try to use the HTML editor for normal articles
	    my $page = $this->{input}{_id};
	    my $section = $this->{reference}{section_id} || $this->{article}{section_id};
	    $form .= $db->input_htmleditor(name=>$col,
					   editor_height=>350,
					   editor_width=>550,
					   table=>"article",
					   column=>"body",
					   record=>$data->{article_id},
					   page=>$page,
					   section=>$section,
					   links=>1,
					   editor=>{
					       op_image=>($db->owns),
					       op_link=>1,
					       op_table=>1,
					       op_dyncontent=>0,
					   });
	    my $edtype = $db->use_wysiwyg ? "HTML" : "mixed";
	    $form .= $db->input_html(name=>"content_type",
				     type=>"hidden",
				     value=>$edtype,
				     );
	    my $switchurl = $this->link(input_method=>"text");
	    $form .= "&rarr; <a href=\"$switchurl\">plain text editor</a> (warning: changes will be discarded)";
	}
	else {
	    $form .= $db->input_exsite(name=>$col,
				       size=>$column{size},
				       datatype=>$column{datatype},
				       value=>$data->{$col},
				       table=>"article",
				       column=>$col,
				       record=>$data->{article_id},
				       );
	    if ($col eq "body" && $type=~ /article/) {
		my $switchurl = $this->link(input_method=>"html");
		$form .= "<br>&rarr; <a href=\"$switchurl\">rich text editor</a> (warning: changes will be discarded)";
	    }
	}
	$form .= "</p>\n";
    }

    # attachments
    my $natt = $this->get("attachments",$article_class{$data->{type}});
    if ($natt > 0) {
	$form .= "<table class=\"ZineAttachmentForm\">\n";
	for (my $iatt=1; $iatt <= $natt; $iatt++) {
	    my $i = $natt ? "#$iatt" : "";
	    $form .= "<tr><td>Attachment $i</td><td><input type=\"file\" name=\"attachment_${iatt}\"></td><td>Caption:&nbsp;<input type=\"text\" name=\"caption_${iatt}\"></td></tr>\n";
	}
	$form .= "</table>\n";
    }

    # hidden fields
    foreach my $col (@hide) {
        my %column = $db->{map}->get_column("article",$col);
	my $label;
	if ($type eq "comment") {
	    $label = $comment_label{$col} || $column{label} || $column{column};
	}
	else {
	    $label = $column{label} || $column{column};
	}
	$form .= $db->input_html(name=>$col,
				type=>"hidden",
				value=>$data->{$col},
				);
	$form .= "\n";
    }

    my $url = $this->link(zine=>"submit");
    if ($db->{htmleditor}) {
	$out = $db->editor_javascript();
#	$out .= "<form METHOD=\"POST\" ACTION=\"$config{server}{CGIpath}/debug.cgi\" onSubmit=\"validate()\">\n";
	$out .= "<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\" onSubmit=\"validate()\" id=\"exsiteform\">\n";
	$out .= "<input type=\"hidden\" name=\"$db->{htmleditor}\">\n";
    }
    else {
	$out .= "<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\">\n";
    }
    $out .= $form;

    # catpcha for unauthenticated contributors
    if (! $db->level && $config{$this->module_name}{use_captcha}) {
	my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	$out .= $c->make();
    }

    # add (protected) control info 
    my ($ctrlinfo,$ctrl_crypt);
    $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    $ctrl_crypt = $config{$this->module_name}{secure_forms} ?
	ExSite::Crypt->new()->encrypt($ctrlinfo) :
	$ctrlinfo;
    $out .= $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $out .= $db->make_form_buttons;
    $out .= "</form>\n";

    return $out;
}

#==========================================================================
# configure the article

sub confform {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};

    # security checks
    my $stat = $this->allow_post($data);
    return $stat if ($stat);

    my $columns = ["type","status","content_type","indexlimit","indexage",
		   "indextype","indexformat","comment_policy","privacy"];

    my $type = ucfirst($data->{type});
    my $url = $this->link(zine=>"submit");
    my $out = "<h3>Configure $type</h3>
<form METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" ACTION=\"$url\">\n";

    my $map = $db->{map}->get_map("article");

    foreach my $col (@$columns) {
        my %column = $db->{map}->get_column("article",$col);
	my $label = $column{label} || $column{column};
	my $reqd = $column{validate} =~ /hard|soft/ ? "(required)" : "";
 	$out .= "<p>$label $reqd<br>\n";
	$out .= $db->input_exsite(name=>$col,
				  size=>$column{size},
				  datatype=>$column{datatype},
				  value=>$data->{$col},
				  table=>"article",
				  column=>$col,
				  record=>$data->{article_id},
				  );
	$out .= "</p>\n";
    }

    # add protected control info 
    my $ctrlinfo = "$data->{type};$data->{article_id};$data->{parent_id};$data->{reference};$data->{thread}";
    my $ctrl_crypt = $config{$this->module_name}{secure_forms} ?
	ExSite::Crypt->new()->encrypt($ctrlinfo) :
	$ctrlinfo;
    $out .= $db->input_html(name=>"ctrlinfo",type=>"hidden",value=>$ctrl_crypt);
    #### reply screen?

    $out .= $db->make_form_buttons;
    $out .= "</form>\n";

    return $out;
}

sub doform {
    my $this = shift;
    my $in = new ExSite::Input;
    my $data = $in->post;
    my $db = $share{DB};
    my ($error,@err);
    my ($is_image);
    my $submit = $data->{submit_action};
    if ($submit eq "Cancel") { return (); } 
    delete $data->{submit_action};

    # process the catpcha for unauthenticated contributors
    if (! $db->level && $config{$this->module_name}{use_captcha}) {
	if ($data->{captcha}) {
	    my $c = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	    if (! $c->pass) {
		$error = "You did not answer the anti-spam question correctly.";
	    }
	}
	else {
	    $error = "Anti-spam question was missing.";
	}
	if ($error) {
	    $this->error($error);
	    return ($error);
	}
	else {
	    delete $data->{captcha};
	    delete $data->{captcha_solution};
	}
    }

    # decode control info
    my $ctrlinfo = $config{$this->module_name}{secure_forms} ?
	ExSite::Crypt->new()->decrypt($data->{ctrlinfo}) :
	$data->{ctrlinfo};
    if ($ctrlinfo !~ /^[-\w]*;\w*;\w*;\w*;\w*$/) {
	# tampering detected - abort
	#$error = "bad control data: $ctrlinfo";
	$error = "bad control data";
	$this->error($error);
	return ($error);
    }
    delete $data->{ctrlinfo};
    my @ctrlinfo = split(/;/,$ctrlinfo);
    my $ctype = shift @ctrlinfo;
    if (! $data->{type}) { $data->{type} = $ctype; }
    ($data->{article_id},$data->{parent_id},
     $data->{reference},$data->{thread}) = @ctrlinfo;

    foreach my $key (qw(parent_id reference thread)) {
	if (! defined $data->{$key}) { $data->{$key} = 0; }
    }

    # type 
    if (!$data->{type}) {
	# no type - better abort
	return $this->error("undefined article type");
    }
    my $type = $data->{type};
    my $parent;
    if ($data->{parent_id}) {
	$parent = $db->fetch("article",$data->{parent_id});
	if ($type eq "article" && $parent->{type} eq "blog") {
	    $type .= "_blog";
	}
	elsif ($type eq "article" && $parent->{type} eq "photoblog") {
	    $type .= "_photoblog";
	}
	elsif ($type eq "article" && $parent->{type} eq "wiki") {
	    $type .= "_wiki";
	}
    }

    # tidy up the record data
    my $sc = $config{form}{sepchar};

    # fix body field
    if (exists $data->{"body${sc}html"}) {
    	# Unescape %7B to { ...etc
    	require ExSite::Util;
	$data->{body} = &restore_cms_tags($data->{"body${sc}html"});
	delete $data->{"body${sc}html"};
    }
    
    # owner info
    if (exists $data->{"owner${sc}key"}) {
	$data->{owner} = $data->{"owner${sc}key"};
	delete $data->{"owner${sc}key"};
    }
    else {
	my $uid = $share{DB}->my_uid;
	if ($uid) { $data->{owner} = $uid; }
    }
    # section info
    if (exists $data->{"section_id${sc}key"}) {
	$data->{section_id} = $data->{"section_id${sc}key"};
	delete $data->{"section_id${sc}key"};
    }
    else {
	##### auto-set section ??
	
    }

    # clean all text input fields -
    # Full HTML allowed in body for normal articles, but not comments ...
    if (exists $data->{body}) {
      $data->{body} = $this->clean_text($data->{body},
					$type eq "comment" ? 1 : 2);
    }
    my $allow_html = ($type =~ /comment/) ?
	$config{$this->module_name}{allow_html_in_comments} : undef;
    # undef means we defer to basic ExSite rules for whether or not
    # to allow html;  ie. config setting report.text.honor_html
    foreach my $f (qw(author title subtitle summary about caption)) {
	if (exists $data->{$f}) {
	    $data->{$f} = $this->clean_text($data->{$f},$allow_html);
	}
    }

    # set the article name
    if (exists $data->{name} && $type !~ /comment|attachment/) {
	if (! $data->{name}) {
	    $data->{name} = $data->{title};
	}
	if ($data->{reference}) {
	    my $aref = $share{DB}->fetch($data->{reference});
	    if ($aref ne "wiki") {
		$data->{name} =~ s/[^\w-]/_/g;
		$data->{name} =~ s/_{2,}/_/g;
	    }
	}
    }

    # encode picture, if any
    my $max_size = $config{$this->module_name}{max_picture_size} || 48;
    my $max_dim = $config{$this->module_name}{max_picture_dim} || 500;
    if ($data->{"picture${sc}clear"}) {
	# remove picture
	$data->{picture} = "";
	$data->{thumb} = "";
	$data->{caption} = "";
    }
    elsif ($data->{"picture${sc}filename"}) { 
	my $fname = $data->{"picture${sc}filename"};
	if ($fname !~ /\.(gif|png|jpe?g)$/i && 
	    ! $config{$this->module_name}{allow_documents}) {
	    # wrong file type
	    $error = "picture must be GIF, PNG, or JPEG file type";
	    $this->error($error);
	    push @err, $error;
	}
	else {
	    $is_image = 1;
	}
	return @err if (@err > 0);

	my $fdata = undef;
	my $size = 0;
	# read file contents
	my $continue = 1;
	while ($continue) {
	    $continue = read($data->{"picture${sc}filename"},$fdata,1024,$size);
	    $size += $continue;
	}

	if ($is_image) {
	    my $img = new ExSite::Image($fname,$fdata);
	    my ($img_x,$img_y) = $img->dim;
	    if ($img_x > $max_dim || $img_y > $max_dim || $size/1024 > $max_size) {
		# picture too big
		$img->shrink(dim=>$max_dim,size=>$max_size);
	    }
	    $data->{picture} = $img->encode;
	    # thumbnail the pic
	    if ($img->thumb) {
		$data->{thumb} = $img->encode;
	    }
	}
	elsif ($size > $max_size) {
	    push @err, "$fname is too large (${max_size}K max)";
	}
    }
    delete $data->{"picture${sc}filename"};
    delete $data->{"picture${sc}clear"};
    return @err if (@err > 0);

    # attachments
    my @att_name;
    my @att_file;
    my @att_thumb;
    my @att_caption;
    my $natt = 1;
    while (exists $data->{"attachment_${natt}"}) { 
	my ($aname,$afile,$athumb,$acap);
	my $aname = $in->fetch_file("attachment_${natt}","name");
	if ($aname) {
	    my $fdata = undef;
	    my $size = 0;
	    # read file contents
	    my $continue = 1;
	    while ($continue) {
		$continue = read($data->{"attachment_${natt}"},
				 $fdata,1024,$size);
		$size += $continue;
	    }

	    # shrink & thumbnail images
	    if ($aname =~ /\.(gif|png|jpe?g)$/i) {
		my $img = new ExSite::Image($aname,$fdata);
		my ($img_x,$img_y) = $img->dim;
		if ($img_x > $max_dim || $img_y > $max_dim || $size/1024 > $max_size) {
		    # picture too big
		    $img->shrink({dim=>$max_dim,size=>$max_size});
		}
		$afile = $img->encode;
		# thumbnail the pic
		if ($img->thumb) {
		    $athumb = $img->encode;
		}
	    }
#	    else {
	    elsif ($size > $max_size) {
		push @err, "attachment $aname is too large (${max_size}K max)";
		#$error = "$aname is not an image";
	    }
	    $acap = $data->{"caption_${natt}"};
	
	    push @att_name, $aname;
	    push @att_file, $afile;
	    push @att_thumb, $athumb;
	    push @att_caption, $acap;
	}
	delete $data->{"attachment_${natt}"};
	delete $data->{"caption_${natt}"};
	$natt++;
    }
    return @err if (@err > 0);

    # check for required fields
    
    foreach my $req (@{$required{$type}}) {
	if (exists $data->{$req}) {  # field is present
	    if (! $data->{$req}) {   # data is missing
		$error = "$req is missing";
		$this->warn($error);
		push @err,$error;
	    }
	}
    }   
    if (@err > 0) {
	return @err;
    }

    # set default status

    if (! $data->{status}) { 
	$data->{status} = $config{$this->module_name}{$type}{status} || "active";
    }

    # all is good - save the article

    my $null = "NULL";
    if ($data->{article_id}) {
	$error = $db->update("article",$data);
	if (! $error) { 
	    # update our internal copy of the article
	    foreach my $key (keys %$data) { 
		$this->{article}{$key} = $data->{$key};
	    }
	}
    }
    else {
	$data->{ctime} = \$null;  # forces auto-setting of MySQL timestamp
	my $aid = $db->insert("article",$data);
	if (!$aid) { 
	    $error = $db->{query_error};
	}
	else {
	    $data->{article_id} = $aid;
	}
    }

    # save the attachments
    
    if (! $error) {
	foreach (my $iatt = 1; $iatt <= (scalar @att_name); $iatt++) {
	    my $adata = {
		title=>$att_name[$iatt-1],
		name=>$att_name[$iatt-1],
		picture=>$att_file[$iatt-1],
		thumb=>$att_thumb[$iatt-1],
		caption=>$att_caption[$iatt-1],
		ctime=>\$null,
		type=>"attachment",
		parent_id=>$data->{article_id},
		reference=>$data->{reference},
		status=>"active",
	    };
	
	    my $aid = $db->insert("article",$adata);
	    if (!$aid) { 
		$error .= $db->{query_error}."; ";
	    }
	    else {
		$this->publish_unprivileged($adata);
	    }
	}
    }

    if ($error) {
	$this->error($error);
	return ($error);
    }
    elsif ($data->{status} ne "active") {
	$this->please_moderate($data);
    }
    
    $this->publish_unprivileged($data);
    return ();
}

#deprecated?
sub post_doform {
    my $this = shift;
    # this is a dummy;  it's purpose is to be overloaded by other
    # article handling systems
    return;
}

# status : update article status only

sub status {
    my ($this,$action) = @_;
    my ($db,$stat,$error);
    $db = $share{DB};
    # security checks
    if ($this->moderator) {
	if ($action eq "approve") {
	    $error = $db->update("article",{status=>"active"},{article_id=>$this->{article_id}});
	    ### TODO: notification?
	}
	elsif ($action eq "reject") {
	    $error = $db->update("article",{status=>"rejected"},{article_id=>$this->{article_id}});
	    ### TODO: notification?
	}
	return $db->show_diagnostics("error","html") if ($error);
	if ($ENV{HTTP_REFERER}) {
	    # bounce the user back where they came from
	    &redirect($ENV{HTTP_REFERER});
	}
	else {
	    # bounce them back to a suitable screen
	    my $out = $this->info("$action $this->{article}{type} $this->{article_id} -- DONE.");
	    $out .= "<p><a href='javascript:window.close()'>[X] Close</p>";
	    return $out;
	}
    }
    else {
	return $this->error("Moderation rejected - you do not have moderation privileges here");
    }
    return undef;
}

sub please_moderate {
    my ($this,$data) = @_;
    my $ref = $this->my_zine;
    $this->save;
    $this->set($ref);
    my $moderator_email = $this->author_email;
    my $moderator_login = $this->author_login;
    $this->restore;
    return if (! $moderator_email);

    my $url = $this->link(aid=>$data->{article_id},zine=>"show");
    my $purl;
    if ($data->{parent_id}) {
	$purl = $this->link(aid=>$data->{parent_id},zine=>"show");
    }
    my $turl;
    if ($data->{thread}) {
	$turl = $this->link(aid=>$data->{thread},zine=>"show");
    }
    my $msg = "
$data->{author} has posted a comment to $ref->{type} '$ref->{title}'
at $config{server}{server}

---------------------------------------------------------------------
From: $data->{author}
Subject: $data->{title}

$data->{body}
---------------------------------------------------------------------

The comment can also be viewed at:
$config{server}{server}$url\n";
    if ($purl) {
	$msg .= "
It may be viewed in the context of the message being replied to at:
$config{server}{server}$purl\n";
    }
    if ($turl) { 
	$msg .= "
The whole thread may be viewed at:
$config{server}{server}$turl\n";
    }

    my $approve = $this->link(zine=>"approve",aid=>$data->{article_id});
    my $authtoken = $share{DB}->make_authtoken($moderator_login,0,$approve);
    $approve  = $this->link(zine=>"approve",aid=>$data->{article_id},'_auth'=>$authtoken);

    my $reject = $this->link(zine=>"reject",aid=>$data->{article_id});
    $authtoken = $share{DB}->make_authtoken($moderator_login,0,$reject);
    $reject  = $this->link(zine=>"reject",aid=>$data->{article_id},'_auth'=>$authtoken);
    $msg .= "\n=== MODERATION LINKS ===

APPROVE this comment: $config{server}{server}$approve

REJECT this comment: $config{server}{server}$reject\n";

    return &SendMail($moderator_email,
		     $config{site}{administrator},
		     "[$config{site}{name}] $ref->{title} - MODERATION REQUESTED",
		     $msg);
}

#==========================================================================
# delete the article and its children

sub delete {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};
    
    my $out;
    if ($this->{admin} && $this->owns) {
	# deletions are restricted to the control panel and owner
	$db->trash_r("article",$this->{article_id},1);
	$out .= "<p>".ucfirst($data->{type})." ".$data->{article_id}.
	    " was moved to the trash.</p>\n";
    }
    else {
	$out .= "<p>Sorry, you do not have permission to delete this article.</p>\n";
    }
    my $url;
    if ($data->{parent_id}) {
	$url = $this->link(zine=>"show",aid=>$data->{parent_id});
    }
    else {
	$url = $this->link(zine=>undef,aid=>undef);
    }
    $out .= "<p><a href=\"$url\">Continue</a></p>\n";
    return $out;
}

sub show_data {
    my $this = shift;
    my $col = shift;
    return $share{DB}->show_data_noauth("article",$col,
				 $this->{article}{$col},$this->{article_id});
}

sub clean_text { 
    my ($this,$text,$allow_basic_html) = @_;

    # Escape all CMS tags so they show clearly;  this allows us to 
    # write about the CMS system for example.

    # NB: this is normally used when INSERTING data into the database.
    # When displaying, we allow DB->clean_text() to clean the data, 
    # which will not mask all CMS tags.  That means you can force
    # CMS tags into an article by inserting them directly (not via
    # this module's doform() method).

    # escape double-bracket CMS tags not inside HTML tags
    $text =~ s/(?!<[^<>]*)\[\[(?![^<>]*>)/\&\#91;\&\#91;/g;
    $text =~ s/(?!<[^<>]*)\]\](?![^<>]*>)/\&\#93;\&\#93;/g;
    $text =~ s/(?!<[^<>]*)\{\{(?![^<>]*>)/\&\#123;\&\#123;/g;
    $text =~ s/(?!<[^<>]*)\}\}(?![^<>]*>)/\&\#125;\&\#125;/g;

    # escape comment-style CMS tags
    $text =~ s/<!--([\$\&].*?)-->/&lt;!--$1--&gt;/g;
    $text =~ s/<!--(content\(.*?\))-->/&lt;!--$1--&gt;/g;

    # now proceed with standard text clean-up
    return $share{DB}->clean_text($text,$allow_basic_html);
}

#------------------------------------------------------------------------
# article security

# owns : tests whether the viewer has permission to modify/update the article

sub owns {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $uid = $db->my_uid;
    return 0 if ( ! $uid );
    return ($uid == $this->{article}{owner});
}

# privacy : is this a public or members-only article

sub privacy { 
    my $this = shift;
    my $privacy = $this->{reference}{privacy} || "public";
    if ($privacy eq "public") {
	# public articles in private zines are private
	$privacy = $this->{article}{privacy} || "public";
    }
    if ($privacy eq "public" && $this->class eq "comment") {
	# comments to articles in private zines are private
	$this->save;
	my $zine = $this->fetch($this->{reference}{reference});
	$privacy = $zine->{privacy} || "public";
	$this->restore;
    }
    return $privacy;
}

# approve : test whether the user has permission to view the article

sub approve { 
    my $this = shift;
    my $db = $share{DB};

    # managers/admins see all
    return 1 if ($db->is_manager);
    
    # article privacy check
    return 0 if ($this->privacy =~ /members/ && ! $db->is_member);

    # article status check (must be active or unset [default is active])
    return 1 if (! $this->{article}{status} || 
		 $this->{article}{status} eq "active");

    return 0;
}

# moderator : test whether the user has permission to moderate comments

sub moderator {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $article;
    if ($this->{article}{type} eq "comment") {
	$article = $this->{reference};
    }
    else {
	$article = $this->{article};
    }
    my $uid = $db->my_uid;
    return 0 if ( ! $uid ); 
    return ($uid == $article->{owner});
}

# allow_comments : test whether comments are accepted on the current article

sub allow_comments {
    my ($this,$class) = @_;
    # custom article setting
    my $com = $this->comment_policy($class);
    return ($com ne "forbidden");
}

# allow_post : test whether user is allowed to post;  returns an error
# string if DISallowed, undef if allowed.  Article data must be provided.

sub allow_post {
    my ($this,$data) = @_;
    if ($data->{article_id} > 0) {
	# we appear to be editing an existing article
	if ($data->{type} eq "comment") {
	    if (! $share{DB}->is_admin) {
		if ($data->{owner} == $share{DB}->my_uid()) {
		    my $t = new ExSite::Time;
		    my $t1 = $t->write("unix_timestamp");
		    $t->set($data->{ctime},"sql_timestamp");
		    my $t2 = $t->write("unix_timestamp");
		    if ($t1 - $t2 > $config{$this->module_name}{edit_my_comment_timeout}) {
			return "<p>This comment is now frozen.</p>\n";
		    }
		    # else, edit comment is allowed
		    return undef;
		}
		return "<p>You don't have permission to edit this comment.</p>\n";
	    }
	}
	if ($data->{type} eq "article" && ! $this->owns) {
	    # user does not own the Zine
	    return "<p>Sorry, you do not have permission to manage this article.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine management only from control panel
	    return "<p>Zines can only be managed from the administrator control panel.</p>\n";
	}
    }
    else {
	# we appear to be creating a new article
	if ($data->{type} eq "comment" && ! $this->allow_comments) {
	    # comments are disabled under this Zine
	    if ($config{$this->module_name}{$this->reftype}{comment_policy}{member}) {
		return "<p>Sorry, only members can post comments here.</p>\n";
		# login link might be nice...
	    }
	    return "<p>Sorry, comments are not permitted.</p>\n";
	}
	if ($data->{type} eq "article" && ! $this->owns) {
	    # user does not own the Zine
	    return "<p>Sorry, you do not have permission to post articles here.</p>\n";
	}
	if ($data->{type} !~ /article|comment/ && ! $this->{admin}) {
	    # Zine creation only from control panel	
	    return "<p>Sorry, you do not have permission to create new Zines.</p>\n";
	}
    }
    return undef;
}

#------------------------------------------------------------------------
# link - optionally reference articles using pathinfo

sub link {
    my ($this,%opt) = @_;
    if ($config{$this->module_name}{link_style} eq "path") {
	my $in = new ExSite::Input;
	my $q = $in->query();

	# %opt2 will contain the new query options in the link
	my %opt2 = %$q;
	foreach my $key (keys %opt) {
	    $opt2{$key} = $opt{$key};
	}

	my $cmd = $opt2{zine} || "show";
	if ($cmd eq "show" && $opt{aid}) {
	    
	    # we appear to have a simple request to view an article

	    #### FIXME

	}

    }
    return $this->SUPER::link(%opt);
}

# alink - shortcut link notation for cross-linking articles
# body text of the form [[tag|anchor]] is converted to:
# <a href=tag-url>anchor</a>.  Anchor defaults to tag.

sub alink {
    my ($this,$name,$anchor) = @_;
    if (! $anchor) {
	$anchor = $name;
	$anchor =~ s/_/ /g;
    }
    my $url = $this->link(goto=>$name);
    return "<a href=\"$url\">$anchor</a>";
}

sub wikilink {
    my ($this,$name) = @_;
    ### FIXME: allow for path-based linking
    my $url = $this->link(goto=>$name);
    return "<a href=\"$url\">$name</a>";
}

#------------------------------------------------------------------------
# searching
# search returns a list of articles that match the search term

# search does a direct search

sub search {
    my ($this,$term,$opt) = @_;

    # which zines should we search?
    my @zid;
    if ($this->{article_id}) {
	@zid = ( $this->{article}{reference} || $this->{article_id} );
    }
    else {
	my @z = $this->our_zines();
	@zid = map $_->{article_id}, @z;
    }
    my $zlist = join(",",@zid);

    # compose query

    return undef if (!$zlist);
    my $basequery = "select * from article where reference in ($zlist) and type=\"article\" and ";

    # set up match criteria
    $term =~ s/[^\w]/ /g;  # kill all non-word characters, to be safe
    my $qterm = $share{DB}{dbh}->quote("%$term%");
    my $field = "title";
    my $query = $basequery."$field like $qterm";

    # 1st pass - search titles
    
    my @art1 = $share{DB}->custom_query($query);
    my (@art, %art);
    foreach my $a (@art1) {
	push @art, $a;
	$art{$a->{article_id}} += 3;
    }

    # additional passes

    my $last_field = "title";
    foreach my $field (keys %search_weight) {
	next if ($field eq "title");
	if ($search_weight{$field}) {
	    $query = $basequery."$field like $qterm";
	    my @art2 = $share{DB}->custom_query($query);
	    foreach my $a (@art2) {
		if (! exists $art{$a->{article_id}}) {
		    push @art, $a;
		}
		$art{$a->{article_id}} += 1;
	    }
	    $last_field = $field;
	}
    }
    return sort { $art{$a->{article_id}} > $art{$b->{article_id}} } @art;
}

# search_index adds links to the system search index
# $search = an ExSite::Search object, which can be used to update the index
# $site = an ExSite::Section object, to find site-specific zines/articles
# $page = the service page that will display all results

sub search_index {
    my ($this,$search,$site,$page) = @_;
    my $out;
    my $url = $page->get_url_dynamic();
    $this->save;
    # loop over every zine we have access to
    foreach my $zine ($this->our_zines()) {
	$this->set($zine);
	$out .= "==> Indexing '".$this->title."'...<br>";
	my $id = $this->{article_id};
	my $tree = $this->fetch_tree($id,{reference=>$id});
	my @art = $tree->collapse();
	# loop over every article in the zine
	foreach my $a (@art) {
	    next if ($a->{type} eq "article" &&
		     ! $config{$this->module_name}{search}{article});
	    next if ($a->{type} eq "comment" &&
		     ! $config{$this->module_name}{search}{comment});
	    $this->set($a);
	    my $aid = $this->{article_id};
	    my $aurl = $url."&aid=${aid}&zine=show";
	    $search->add_url($aurl,
			     $site->id(),
			     $this->title(),
			     $this->abstract(),
			     $this->privacy(),
			     );
	    # strip the article down to its bare text
	    #my $text = &html_to_plaintext($this->show_article());
	    #$search->index($aurl,1,$text);
	    my $headtext = &html_to_plaintext($this->header());
	    $search->index($aurl,5,$headtext);
	    my $abouttext = &html_to_plaintext($this->about());
	    $search->index($aurl,2,$abouttext);
	    my $bodytext = &html_to_plaintext($this->body());
	    $search->index($aurl,1,$bodytext);
	}
    }
    $this->restore;
    return $out;
}

1;



